#ifndef __H_XLAYER_H__
#define __H_XLAYER_H__
//////////////////////////////////////////////////////////////////////////
#include "m_afx.h"
#include "xstream.h"
#include "xlog.h"
//////////////////////////////////////////////////////////////////////////
#define XLAYER_TEST 0
//////////////////////////////////////////////////////////////////////////
#define WIDTH_MAX 65535
#define HEIGHT_MAX 65535
#define XLAYER_LINE_MAX 65535
//////////////////////////////////////////////////////////////////////////
/* graphics functions, as in GC.alu */
#define	GXclear			0x0		/* 0 */
#define GXand			0x1		/* src AND dst */
#define GXandReverse		0x2		/* src AND NOT dst */
#define GXcopy			0x3		/* src */
#define GXandInverted		0x4		/* NOT src AND dst */
#define	GXnoop			0x5		/* dst */
#define GXxor			0x6		/* src XOR dst */
#define GXor			0x7		/* src OR dst */
#define GXnor			0x8		/* NOT src AND NOT dst */
#define GXequiv			0x9		/* NOT src XOR dst */
#define GXinvert		0xa		/* NOT dst */
#define GXorReverse		0xb		/* src OR NOT dst */
#define GXcopyInverted		0xc		/* NOT src */
#define GXorInverted		0xd		/* NOT src OR dst */
#define GXnand			0xe		/* NOT src OR NOT dst */
#define GXset			0xf		/* 1 */
//////////////////////////////////////////////////////////////////////////
/* fillStyle */
#define FillSolid		0
#define FillTiled		1
#define FillStippled		2
#define FillOpaqueStippled	3

/* fillRule */
#define EvenOddRule		0
#define WindingRule		1

/* CoordinateMode for drawing routines */
#define CoordModeOrigin		0	/* relative to the origin */
#define CoordModePrevious       1	/* relative to previous point */
////////////////////////////-//////////////////////////////////////////////
////////////////////////////-//////////////////////////////////////////////
typedef struct _PixelColour
{
	u32 red;
	u32 green;
	u32 blue;
} PixelColour;

#define SPLITCOLOUR15(colour, rv) \
{ \
	rv.r = ((colour >> 7) & 0xf8) | ((colour >> 12) & 0x7); \
	rv.g = ((colour >> 2) & 0xf8) | ((colour >> 8) & 0x7); \
	rv.b = ((colour << 3) & 0xf8) | ((colour >> 2) & 0x7); \
}

#define SPLITCOLOUR16(colour, rv) \
{ \
	rv.r = ((colour >> 8) & 0xf8) | ((colour >> 13) & 0x7); \
	rv.g = ((colour >> 3) & 0xfc) | ((colour >> 9) & 0x3); \
	rv.b = ((colour << 3) & 0xf8) | ((colour >> 2) & 0x7); \
} \

#define SPLITCOLOUR24(colour, rv) \
{ \
	rv.b = (colour & 0xff0000) >> 16; \
	rv.g = (colour & 0x00ff00) >> 8; \
	rv.r = (colour & 0x0000ff); \
}

#define BSWAP16(x) { x = (((x & 0xff) << 8) | (x >> 8)); }
#define BSWAP24(x) { x = (((x & 0xff) << 16) | (x >> 16) | (x & 0xff00)); }
#define BSWAP32(x) { x = (((x & 0xff00ff) << 8) | ((x >> 8) & 0xff00ff)); \
	x = (x << 16) | (x >> 16); }

/* The following macros output the same octet sequences
on both BE and LE hosts: */

#define BOUT16(o, x) { *(o++) = x >> 8; *(o++) = x; }
#define BOUT24(o, x) { *(o++) = x >> 16; *(o++) = x >> 8; *(o++) = x; }
#define BOUT32(o, x) { *(o++) = x >> 24; *(o++) = x >> 16; *(o++) = x >> 8; *(o++) = x; }
#define LOUT16(o, x) { *(o++) = x; *(o++) = x >> 8; }
#define LOUT24(o, x) { *(o++) = x; *(o++) = x >> 8; *(o++) = x >> 16; }
#define LOUT32(o, x) { *(o++) = x; *(o++) = x >> 8; *(o++) = x >> 16; *(o++) = x >> 24; }
//////////////////////////////////////////////////////////////////////////
typedef struct _xPoint { int x , y ; } xPoint ;
typedef struct _xRect { xPoint a , b ; } xRect ;
typedef struct _xRect2 { xPoint f , j ; } xRect2 ;
typedef struct _xRect4 { int x , y , w , h ; } xRect4 ;
typedef struct _xArc
{
	xPoint a , b ;
	int angle1 ;
	int angle2 ;
} xArc ;
typedef struct _xPoly
{
	xPoint * points ;
	int num ;
} xPoly ;
typedef struct _xColor
{
	u8 b ;
	u8 g ;
	u8 r ;
	u8 alpha_reserve ;
} xColor ;

typedef struct _xColor16
{
	u8 bg ;
	u8 gra ;
} xColor16 ;
//////////////////////////////////////////////////////////////////////////
typedef struct _xImage
{
	int w ;
	int h ;
	char * data ;
	int bpp ;
} xImage ;
//////////////////////////////////////////////////////////////////////////
typedef struct _xColorMap
{
	unsigned int count ;
	xColor * color ;
} xColorMap ;
//////////////////////////////////////////////////////////////////////////
struct _xLayer ;
typedef struct _xGC
{
	int function ;
	xColor pixcel ;
	xColor * pen ;

	xColor f ;
	xColor b ;

	int fill_style ;
	int fill_rule ;

	int tsox ;
	int tsoy ;

	struct _xLayer * tile ;
	struct _xLayer * stipple ;

	int clipvalid ;
	xRect clip ;
	struct _xLayer * clip_mask;	/* bitmap clipping; other calls for rects */

	xPoint minp ;
	xPoint maxp ;
} xGC ;
//////////////////////////////////////////////////////////////////////////
typedef struct _xLayer
{
	int linesize ;
	int bpp ;
	int x ;
	int y ;
	int width ;
	int height ;
	XSTREAM s ;
	xGC gc ;

	int disable_free ;

	int compress_flag ;
} xLayer ;
//////////////////////////////////////////////////////////////////////////
extern xColor tWhite ;
extern xColor tBlack ;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
/************************************************************************/
/* static functions                                     */
#define xlayer_valid_xy(l,x,y)\
{ \
	if ( (x) < 0 ) (x) = 0 ; \
	if ( (y) < 0 ) (y) = 0 ; \
	if ( (x) > ((l) ->width - 1) ) (x) = (l) ->width - 1 ; \
	if ( (y) > ((l) ->height - 1) ) (y) = (l) ->height - 1 ; \
}

#define xlayer_rect_and_rect(x1,y1,w1,h1,x2,y2,w2,h2,r) { \
	(r).a.x = max((x1), (x2)) ; \
	(r).a.y = max((y1), (y2)) ; \
	if((w1>0)&&(h1>0)&&(w2>0)&&(h2>0)) \
{ \
	int a1 , b1 , a2 , b2 ; \
	a1 = (x1) + (w1) - 1 ; \
	b1 = (y1) + (h1) - 1 ; \
	a2 = (x2) + (w2) - 1 ; \
	b2 = (y2) + (h2) - 1 ; \
	(r).b.x = min(a1, a2) ; \
	(r).b.y = min(b1, b2) ; \
	(r).b.x = (r).b.x >= (r).a.x?(r).b.x - (r).a.x + 1:0 ; \
	(r).b.y = (r).b.y >= (r).a.y?(r).b.y - (r).a.y + 1:0 ; \
} \
else \
{ \
	(r).b.x = (r).b.y = 0 ; \
} \
}

#define xlayer_clip(l,px,py,w,h) if ( (l) ->gc.clipvalid ) {\
	xRect r ; \
	xlayer_rect_and_rect((l)->gc.clip.a.x,(l)->gc.clip.a.y,(l)->gc.clip.b.x,(l) ->gc.clip.b.y,(px),(py),(w),(h),r); \
	(px) = r.a.x ; \
	(py) = r.a.y ; \
	(w) = r.b.x ; \
	(h) = r.b.y ; \
}

#define xlayer_update_modifiy_rect(l,px,py,w,h) if(((w)>0)&&((h)>0)) {\
	int a , b ; \
	if ( (px) < (l) ->gc.minp.x ) (l) ->gc.minp.x = (px) ; \
	if ( (py) < (l) ->gc.minp.y ) (l) ->gc.minp.y = (py) ; \
	a = (px)+(w)-1 ; \
	b = (py)+(h)-1 ; \
	if ( a > (l) ->gc.maxp.x ) (l) ->gc.maxp.x = a ; \
	if ( b > (l) ->gc.maxp.y ) (l) ->gc.maxp.y = b ; \
}

#define xlayer_mv(l,a,b) (((l)->s ->p += ( (b - ((l) ->y)) * (l) ->linesize + (a - ((l) ->x)) * (l) ->bpp )) , (l) ->x = a , (l) ->y = b )

#define xlayer_mv_x_plus(l) ((l) ->s ->p += (l) ->bpp , (l) ->x ++ )
#define xlayer_mv_x_sub(l) ((l) ->s ->p -= (l) ->bpp , (l) ->x -- )
#define xlayer_mv_y_plus(l) ((l) ->s ->p += (l) ->linesize , (l) ->y ++ )
#define xlayer_mv_y_sub(l) ((l) ->s ->p -= (l) ->linesize , (l) ->y -- )

#define xlayer_get(l)			(*((xColor *)((l) ->s ->p)))
#define xlayer_set(l,c)    (*((xColor *)((l) ->s ->p))) = (c)
#define xlayer_equal(l,c)    ((*((unsigned int *)((l) ->s ->p))) == (*((unsigned int *)(&c))))
#define xlayer_unequal(l,c)    ((*((unsigned int *)((l) ->s ->p))) != (*((unsigned int *)(&c))))

#define xlayer_setc(l,c)			while(1){ \
	if ( (l) ->gc.clipvalid ) \
{ \
	if ( (((l) ->x) < ((l) ->gc.clip.a.x)) \
	|| ( ((l) ->x) >= ((l) ->gc.clip.a.x + (l) ->gc.clip.b.x)) \
	|| ( ((l) ->y) < ((l) ->gc.clip.a.y)) \
	|| ( ((l) ->y) >= ((l) ->gc.clip.a.y + (l) ->gc.clip.b.y)) ) \
	break ; \
} \
	if ( (*((unsigned int *)((l) ->s ->p))) == (*((unsigned int *)(&c))) ) break ; \
	xlayer_set((l),(c)) ; \
	if ( (l) ->x < (l) ->gc.minp.x ) (l) ->gc.minp.x = (l) ->x ; \
	if ( (l) ->y < (l) ->gc.minp.y ) (l) ->gc.minp.y = (l) ->y ; \
	if ( (l) ->x > (l) ->gc.maxp.x ) (l) ->gc.maxp.x = (l) ->x ; \
	if ( (l) ->y > (l) ->gc.maxp.y ) (l) ->gc.maxp.y = (l) ->y ; \
	break ;\
}

#define xlayer_setp(l) xlayer_setc((l),((l) ->gc.pixcel))

//bc[rgb(b.alpha)]--
//					------>mix[rgb(b.alpha)]
//ac[rgb(a.alpha)]--
#define xlayer_mix(l,f,background,foreground,c) { \
	xColor lc ; \
	u32 a , b , retur ; \
	switch (f) \
{ \
	case 0x0: c = background ; break ; \
	case 0xf: c = foreground ; break ; \
	case 0x3: break ; \
	case 0x5: c = xlayer_get(l) ; break ; \
	default: \
	lc = xlayer_get(l) ; \
	b = *((u32*)(&lc)) ; \
	a = *((u32*)(&(c))) ; \
	switch(f) \
{ \
	case 0x1: retur = b & a ; break ; \
	case 0x2: retur = a & (~b ) ; break ; \
	case 0x4: retur = (~a) & b ; break ; \
	case 0x6: retur = a ^ b ; break ; \
	case 0x7: retur = a | b ; break ; \
	case 0x8: retur = (~a) & (~b) ; break ; \
	case 0x9: retur = (~a) ^ b ; break ; \
	case 0xa: retur = (~b ) ; break ; \
	case 0xb: retur = a | ( ~b ) ; break ; \
	case 0xc: retur = (~a) ; break ; \
	case 0xd: retur = (~a) | b ; break ; \
	case 0xe: retur = (~a) | (~b) ; break ; \
	default: retur = (*((u32*)(&foreground))) ; break ; \
} \
	c = *((xColor*)&retur) ; \
	break ; \
} \
}

#define xlayer_mixset(l,f,background,foreground,c)	{ \
	xColor xc = (c) ; \
	xlayer_mix(l,f,background,foreground,xc); \
	xlayer_set ( (l) , xc ) ; \
}

#define xlayer_mixsetc(l,f,background,foreground,c)	{ \
	xColor xc = (c) ; \
	xlayer_mix(l,f,background,foreground,xc); \
	xlayer_setc ( (l) , xc ) ; \
}

/************************************************************************/
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
/*translate_colour*/
#define xlayer_translate_colour(colour,depth,colormap,pc) { \
	switch (depth) \
{ \
	case 8:\
	pc = ((colormap) && ((colour) < ((colormap) ->count))) ? \
	((colormap) ->color[ (colour) ]) : tBlack ; \
	break ; \
case 15: \
	SPLITCOLOUR15((colour), (pc)); \
	break; \
case 16: \
	SPLITCOLOUR16((colour), (pc)); \
	break; \
case 24: \
case 32: \
	SPLITCOLOUR24((colour), (pc)); \
	break; \
default: \
	(pc).r = 0; \
	(pc).g = 0; \
	(pc).b = 0; \
	break; \
} \
}
//////////////////////////////////////////////////////////////////////////
xImage * xlayer_allocImage ( int w , int h , char * data , int bpp ) ;
xImage * xlayer_staticImage ( int w , int h , char * data , int bpp ) ;
xImage * xlayer_dupImage ( xImage * image ) ;
void xlayer_freeImage ( xImage ** image ) ;

xLayer * xlayer_Image2Layer ( xImage * imager , xColorMap * colormap ) ;
xLayer * xlayer_staticImage2Layer ( xImage * imager , xColorMap * colormap ) ;
//////////////////////////////////////////////////////////////////////////
xLayer * xlayer_allocLayer ( int w , int h ) ;
void xlayer_zeroInitLayer ( xLayer * layer , int w , int h ) ;
xLayer * xlayer_initLayer ( xLayer * layer , int w , int h , char * data , int dsize ) ;
void xlayer_freeLayer ( xLayer ** layer ) ;
xLayer * xlayer_Dup ( xLayer * layer ) ;
//////////////////////////////////////////////////////////////////////////
#define xlayer_setBackground( l , c )		( (l) ->gc.b = (c) )
#define xlayer_getBackground( l )			( (l) ->gc.b )
#define xlayer_setForeground( l , c )		( (l) ->gc.f = (c) )
#define xlayer_getForeground( l )			( (l) ->gc.f )

#define xlayer_resetPen( l )					( (l) ->gc.pen = NULL )
#define xlayer_setPen( l , pc )				( (l) ->gc.pen = (pc) )
#define xlayer_getPen( l )						( (l) ->gc.pen )
#define xlayer_setPix( l , c )				( (l) ->gc.pixcel = c )

#define xlayer_setFunction( l , f )			( (l) ->gc.function = (f) )

#define xlayer_setFillStyle( l , style )		( (l) ->gc.fill_style = (style) )
#define xlayer_getFillStyle( l )				( (l) ->gc.fill_style )

#define xlayer_setTSOrigin( l , x , y )		{ (l) ->gc.tsox = x ; (l) ->gc.tsoy = y ; }
//////////////////////////////////////////////////////////////////////////
/*Tile && Stipple*/
#define xlayer_setTile(l,t) \
{ \
	if ( (l) && (t) ) \
{ \
	if ( (l) ->gc.tile ) xlayer_freeLayer ( &((l) ->gc.tile) ) ; \
	(l) ->gc.tile = xlayer_Dup ( (t) ) ; \
} \
}

#define xlayer_setStipple(l,s) \
{ \
	if ( (l) && (s) ) \
{ \
	if ( (l) ->gc.stipple ) xlayer_freeLayer ( &((l) ->gc.stipple) ) ; \
	(l) ->gc.stipple = xlayer_Dup ( (s) ) ; \
} \
}
//////////////////////////////////////////////////////////////////////////
/*Modify Rect*/
#define xlayer_getModifyRect(l,r) \
{ \
	memset ( &(r) , 0 , sizeof(r) ) ; \
	if ((l)) \
{ \
	(r).a.x = (l) ->gc.minp.x ; \
	(r).a.y = (l) ->gc.minp.y ; \
	if ( (l) ->gc.maxp.x < (l) ->gc.minp.x ) \
	(r).b.x = 0 ; \
else \
	(r).b.x = (l) ->gc.maxp.x - (l) ->gc.minp.x + 1 ; \
	if ( (l) ->gc.maxp.y < (l) ->gc.minp.y ) \
	(r).b.y = 0 ; \
else \
	(r).b.y = (l) ->gc.maxp.y - (l) ->gc.minp.y + 1 ; \
} \
}

#define xlayer_resetModifyRect(l) \
{ \
	if (l) \
{ \
	(l) ->gc.minp.x = (l) ->width > 0 ? (l) ->width - 1 : 0 ; \
	(l) ->gc.minp.y = (l) ->height > 0 ? (l) ->height - 1 : 0 ; \
	(l) ->gc.maxp.x = (l) ->gc.maxp.y = 0 ; \
} \
}
//////////////////////////////////////////////////////////////////////////
/*Moveto*/
#define xlayer_Moveto(l,x,y)\
{\
	if (l) \
{ \
	xlayer_valid_xy((l),(x),(y)) ; \
	xlayer_mv((l),(x),(y)) ; \
} \
}
//////////////////////////////////////////////////////////////////////////
/*Pixcel*/
#define xlayer_getPixcel(l,x,y,r) \
{ \
	if (l) \
{ \
	xlayer_Moveto ((l),(x),(y)) ; \
	(r) = xlayer_get (l) ; \
} \
}

#define xlayer_setPixcel(l,x,y) \
{ \
	if (l) \
{ \
	xlayer_Moveto ((l),(x),(y)) ; \
	(l) ->gc.pixcel = (l) ->gc.pen ? *((l) ->gc.pen) : (l) ->gc.f ; \
	xlayer_setp (l) ; \
} \
}
//////////////////////////////////////////////////////////////////////////
#define xlayer_Line(l,s,e) xlayer_Linex(l,(s).x,(s).y,(e).x,(e).y)
void xlayer_Linex ( xLayer * layer , int x1 , int y1 , int x2 , int y2 ) ;
void xlayer_Lines ( xLayer * layer , xPoint * ps , u32 n , u32 mode ) ;
//////////////////////////////////////////////////////////////////////////
void xlayer_Resize ( xLayer * layer , int w , int h ) ;
void xlayer_TranSize ( xLayer * layer , int w , int h ) ;

#define xlayer_Combine(l,segment,d,s,size) xlayer_Combinex((l),(segment),(d).x,(d).y,(s).x,(s).y,(size).x,(size).y )
void xlayer_Combinex ( xLayer * layer , xLayer * segment , int x1 , int y1 , int x2 , int y2 , int w , int h ) ;
//////////////////////////////////////////////////////////////////////////
#define xlayer_Rect(l,p,s,f) xlayer_Rectx((l),(p).x,(p).y,(s).x,(s).y,(f))
void xlayer_Rectx ( xLayer * layer , int x , int y , int w , int h , u32 fill ) ;
//////////////////////////////////////////////////////////////////////////
#define xlayer_Subscreen(l,subscreen,p,s) xlayer_Subscreenx((l),(subscreen),(p).x,(p).y,(s).x,(s).y)
int xlayer_Subscreenx ( xLayer * layer , xLayer ** subscreen , int x , int y , int w , int h ) ;
//////////////////////////////////////////////////////////////////////////
int xlayer_TranTo16 ( xLayer * l ) ;
int xlayer_TranFrom16 ( xLayer * l ) ;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
XSTREAM xlayer_Layer2bitmap ( xLayer * layer , char * filename ) ;
xLayer * xlayer_Load4bitmap ( char * filename ) ;
//////////////////////////////////////////////////////////////////////////
#endif
