#ifndef __H_XSTREAM_H__
#define __H_XSTREAM_H__
#include "m_afx.h"
//////////////////////////////////////////////////////////////////////////
typedef struct _xstream
{
  unsigned int attri ;
  unsigned char *p;
  unsigned char *end;
  unsigned char *data;
  unsigned int size;
  
  unsigned int ex_attri ;
  unsigned char * ex ;
} xstream ;
typedef xstream * XSTREAM ;

#define SIZEOFSTREAM (sizeof(xstream))

#define XSTREAM_ATTRI_FREEABLE (1)
#define XSTREAM_ATTRI_S_FREEABLE (2)
//////////////////////////////////////////////////////////////////////////
void xstream_init_ex ( XSTREAM s , char * data , int size ) ;
void xstream_init ( XSTREAM s , int size ) ;
XSTREAM xstream_alloc_ex ( char * data , int size ) ;
XSTREAM xstream_alloc ( int size ) ;
void xstream_free ( XSTREAM * s ) ;

void xstream_set ( XSTREAM s , unsigned char * data , int size ) ;
XSTREAM xstream_dup ( XSTREAM s ) ;

void xstream_begin ( XSTREAM s ) ;
void xstream_reset ( XSTREAM s ) ;
void xstream_memset ( XSTREAM s , uptr data , u32 len ) ;
unsigned int xstream_size ( XSTREAM s ) ;
unsigned int xstream_vsize ( XSTREAM s ) ;
unsigned int xstream_pre ( XSTREAM s ) ;
unsigned int xstream_post ( XSTREAM s ) ;
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
#define s_push(s)       ((s)->p)
#define s_pop(s,v)      ((s)->p = v)

#define s_to_begin(s)   ((s)->p = (s)->data)
#define s_finish(s)     ((s)->end = (s)->p)
#define s_valid(s)      ((s)->p <= (s)->end)
#define s_is_end(s)     ((s)->p == (s)->end)
#define s_is_zero(s)    ((s)->end == (s)->data)
#define s_to_zero(s)    ((s)->p = (s)->end = (s)->data)

#define s_left(s)       ((int)((s)->end-(s)->p))
#define s_tail(s)       ((s)->size-((s)->p-(s)->data))
#define s_len(s)        ((s)->end-(s)->data)

#define s_skip(s,n)     ((s)->p += n)
#define s_sskip(s,n)    (s_left(s)>n?(s)->p += n,1:0)
#define s_back(s,n)     ((s)->p -= n)

#define in16_le(s,v)    { v = *((s)->p++); v += *((s)->p++) << 8; }
#define in32_le(s,v)    { in16_le(s,v) ; v += *((s)->p++) << 16; v += *((s)->p++) << 24; }

#define out16_le(s,v)   { *((s)->p++) = (v) & 0xff; *((s)->p++) = ((v) >> 8) & 0xff; }
#define out32_le(s,v)   { out16_le(s, (v) & 0xffff); out16_le(s, ((v) >> 16) & 0xffff); }

#define in16_be(s,v)    { v = *((s)->p++); s_next_be(s,v); }
#define in32_be(s,v)    { in16_be(s,v); s_next_be(s,v); s_next_be(s,v); }
#define out16_be(s,v)   { *((s)->p++) = ((v) >> 8) & 0xff; *((s)->p++) = (v) & 0xff; }
#define out32_be(s,v)   { out16_be(s, ((v) >> 16) & 0xffff); out16_be(s, (v) & 0xffff); }

#define in16(s,v)       in16_le(s,v)
#define in32(s,v)       in32_le(s,v)
#define out16(s,v)      out16_le(s,v)
#define out32(s,v)      out32_le(s,v)

#define in8(s,v)        (v = *((s)->p++))
#define in8a(s,v,n)     { memcpy(v,(s)->p,n); (s)->p += n; }
#define out8(s,v)       (*((s)->p++) = v)
#define out8a(s,v,n)    { memcpy((s)->p,v,n); (s)->p += n; }
#define out8zero(s,n)   { memset((s)->p,0,n); (s)->p += n; }

#define s_next_be(s,v)  (v = ((v) << 8) + *((s)->p++))
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
#endif