
#include "comm_head.h"
#include "common.h"
#include "setting.h"
#include "policy.h"

extern ST_globalInfo gglobalInfo;

/*
des:
    连接鱼头
param:
    poctoShare[IN]:鱼头鱼爪共享信息
return:
    失败：-1
    成功：大于等于0的socketfd套接字
*/
static int create_head_socket(PST_octoShare poctoShare)
{
    int socketFd = -1;
    int maxTimes = 3;
    int sleepTime = 3;
    int times = 0;

    while (1)
    {
        socketFd = xsk_init_client(HEAD_IP, poctoShare->headListenPort, 3, NULL);
        if (socketFd < 0)
        {
            if (times > maxTimes)
            {
                break;
            }
            times++;
            sleep(sleepTime);
        }
        else
        {
            break;
        }
    }
    return socketFd;
}

/*
des:
    创建与鱼头认证的通信json
param:
    retbuf[IN,OUT]:鱼头鱼爪共享信息
*/
static void build_auth_json(char *retbuf)
{
    cJSON *jsonroot = NULL;
    char *jsonout = NULL;

    if (retbuf == NULL)
    {
        return;
    }
    // 创建根节点对象
    jsonroot = cJSON_CreateObject();

    // 添加数据
    cJSON_AddStringToObject(jsonroot, "modeName", MODE_NAME);

    // 解析成字符串
    jsonout = cJSON_Print(jsonroot);
    // 赋值
    strcpy(retbuf, jsonout);
    // 释放json对象的空间
    cJSON_Delete(jsonroot);
    // 释放jsonout的空间
    free(jsonout);
}
/*
des:
   解析鱼爪认证结果json
param:
   buf[IN]:需要解析的json字符串
return:
   认证成功:返回0
   认证失败:小于0
*/
static int parse_leg_auth_result_json(char *buf)
{
    int ret = -1;

    cJSON *root = cJSON_Parse(buf);
    if (root == NULL)
    {
        goto __finish;
    }

    cJSON *authResult = cJSON_GetObjectItem(root, "authResult");
    if (authResult == NULL)
    {
        goto __finish;
    }
    if (strcmp(authResult->valuestring, "start") != 0)
    {
        DBGPRINT(VERR, "parse_leg_auth_result_json() authResult->valuestring[%s] not [start] error.", authResult->valuestring);
        goto __finish;
    }

    ret = 0;
__finish:
    if (root)
    {
        cJSON_Delete(root);
    }
    return ret;
}

/*
des:
    与鱼头进行认证通信
param:
    socketFd[IN]:与鱼头通信的套接字
return:
    失败：-1
    成功：大于等于0的socketfd套接字
*/
static int auth_with_head(int socketFd)
{
    int ret = -1;
    char sendBuf[BUFFER_SIZE] = {0};
    int len = 0;
    char *recvStr = NULL;
    int tmpRet = -1;

    // 向鱼头汇报己方的modeName
    build_auth_json(sendBuf);
    DBGPRINT(VDBG, "auth_with_head() build_auth_json() sendBuf[%s]", sendBuf);
    len = xsk_send((char *)sendBuf, strlen(sendBuf), socketFd);
    if (len <= 0)
    {
        DBGPRINT(VERR, "auth_with_head() xsk_send auth error.");
        goto __finish;
    }
    // 接收认证信息
    tmpRet = xsk_recv(&recvStr, &len, socketFd);
    if (tmpRet <= 0)
    {
        DBGPRINT(VERR, "auth_new_so() xsk_recv auth buf error.");
        goto __finish;
    }
    // 接收鱼头认证结果，返回start，继续工作，返回stop，停止工作
    tmpRet = parse_leg_auth_result_json(recvStr);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "auth_new_so() xsk_recv auth buf error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "auth_new_so() xsk_recv auth buf[%s].", recvStr);
    ret = 0;
__finish:
    xfree(recvStr);
    return ret;
}

/*
des:
    鱼爪与鱼头通信方法，分为连接、认证、通信保持三个阶段，通信保持可持续接收鱼头信息
param:
    poctoShare[IN]:鱼头鱼爪共享信息
*/
void comm_head(PST_octoShare poctoShare)
{
    int tmpRet = -1;
    DBGPRINT(VDBG, "comm_head() start.");
    sleep(1);
    // 创建与鱼头的连接
    gglobalInfo.commSocketfd = create_head_socket(poctoShare);
    if (gglobalInfo.commSocketfd < 0)
    {
        DBGPRINT(VERR, "comm_head() create_head_socket error.");
        goto __finish;
    }
    
    DBGPRINT(VDBG, "file:%s, line:%d.",__FILE__, __LINE__);
    // 与鱼头进行认证通信
    tmpRet = auth_with_head(gglobalInfo.commSocketfd);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "comm_head() auth_with_head error.");
        goto __finish;
    }
    // 建立连接，标志置1
    gglobalInfo.contentFlag = 1;
    // 开启鱼头套接字监听，监控策略，监控鱼头存活状态
    recv_head_policy(gglobalInfo.commSocketfd);
__finish:
    DBGPRINT(VDBG, "comm_head() close.");
    xsk_close(&gglobalInfo.commSocketfd);
}
