#include "sm4.h"
#include "lib.h"
#include "afx/m_afx.h"
#include "afx/xbase64.h"
#include "afx/xlog.h"

#define DATA_LEN_TAG 4

int get_sm4_len(int valueLen)
{
	switch (valueLen % 16)
	{
	case 0:
		return valueLen;
	default:
		return ((valueLen / 16) + 1) * 16;
	}
}

// 获取加密结果  1.key  2.待加密数据  3.待加密数据长度  4.base64结果	5:base54结果长度
int get_sm4_and_base_enc_result(unsigned char *key, unsigned char *value, int valueLen, unsigned char **baseResult, int *baseResultLen)
{
	int ret = -1;
	int tmpLen = 0;
	int trueLen = 0;
	sm4_context sm4Stu;
	unsigned char *resule = NULL;

	// 初始化sm4结构体
	memset(&sm4Stu, 0, sizeof(sm4Stu));
	sm4_setkey_enc(&sm4Stu, key);

	// 保存真实数据长度
	trueLen = htonl(valueLen);
	// value_len转换16倍数
	tmpLen = get_sm4_len(valueLen);

	resule = (unsigned char *)xmalloc(tmpLen + DATA_LEN_TAG);
	*baseResult = (unsigned char *)xmalloc((tmpLen + DATA_LEN_TAG) * 2);
	memcpy(resule, (void *)&trueLen, sizeof(trueLen));

	// sm4加密
	sm4_crypt_ecb(&sm4Stu, SM4_ENCRYPT, valueLen, value, (unsigned char *)resule + DATA_LEN_TAG);
	// base64加密
	*baseResultLen = EncodeBase64((char *)resule, tmpLen + DATA_LEN_TAG, (char *)*baseResult);
	if (*baseResultLen < 0)
	{
		goto __finish;
	}
	ret = 0;
__finish:
	if (resule != NULL)
	{
		xfree(resule);
	}
	return ret;
}

// 解密  1.key  2.解密数据  3.解密数据长度  4.解密结果 5.原数据长度
int get_sm4_and_base_dec_result(unsigned char *key, unsigned char *value, int valueLen, unsigned char **baseResult, int *baseResultLen)
{
	int ret = -1;
	int tmpLen = 0;
	int len = 0;
	int trueLen = 0;
	sm4_context sm4Stu;
	unsigned char *resule = NULL;

	// 初始化sm4结构体
	memset(&sm4Stu, 0, sizeof(sm4Stu));
	sm4_setkey_dec(&sm4Stu, key);

	resule = (unsigned char *)xmalloc(valueLen * 4);
	*baseResult = (unsigned char *)xmalloc(valueLen * 4);
	// 解base64
	len = DecodeBase64((char *)value, valueLen, (char *)resule);
	if (len < 0)
	{
		goto __finish;
	}

	memcpy(&trueLen, (void *)resule, DATA_LEN_TAG);
	*baseResultLen = ntohl(trueLen);

	// value_len转换16倍数
	tmpLen = get_sm4_len(len - DATA_LEN_TAG);

	// 数据解密
	sm4_crypt_ecb(&sm4Stu, SM4_DECRYPT, tmpLen, resule + DATA_LEN_TAG, *baseResult);

	ret = 0;
__finish:
	if (resule)
	{
		xfree(resule);
	}
	return ret;
}
