#include "load_config.h"
#include "setting.h"
#include "afx/xlog.h"
#include "afx/m_afx.h"
#include "afx/xlist.h"
#include "afx/xconfig.h"

extern ST_globalInfo gglobalInfo;

int get_conf_value_int(xlist *plist, char *tag, char *key, int *ret_data)
{
    char *ret_buf = NULL;
    ret_buf = get_config_value(plist, tag, key);
    if (ret_buf)
    {
        int n = xatoi(ret_buf);
        *ret_data = n;
    }
    else
    {
        return -1;
    }
    return 0;
}

int get_conf_value_str(xlist *plist, char *tag, char *key, char *ret_data)
{
    char *ret_buf = NULL;
    ret_buf = get_config_value(plist, tag, key);
    if (ret_buf)
    {
        strcpy(ret_data, ret_buf);
    }
    else
    {
        return -1;
    }
    return 0;
}

/*
des:
    获取self配置信息
param:
    path[IN]:self配置文件路径
return:
    成功：0
    失败：小于0
*/
int get_info_by_self_conf(char *path)
{
    int ret = -1;
    int tmpRet = -1;
    xlist *plist = NULL;

    // 判断文件是否存在 是否可读
    if (access(path, F_OK) != 0 || access(path, R_OK) != 0)
    {
        DBGPRINT(VERR, "get_info_by_conf() configure path[%s] not exist or cant be read", path);
        goto __finish;
    }

    // 初始化配置文件存储链表
    plist = xlist_init();
    if (!plist)
    {
        DBGPRINT(VERR, "get_info_by_conf() plist init error.");
        goto __finish;
    }

    // 加载文件
    load_config(plist, path, NULL);

    tmpRet = get_conf_value_int(plist, MODE_NAME, "auditTime", &(gglobalInfo.confInfo.auditTime));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "auditTime[%d]", gglobalInfo.confInfo.auditTime);
    tmpRet = get_conf_value_int(plist, MODE_NAME, "warningTime", &(gglobalInfo.confInfo.warningTime));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "warningTime[%d]", gglobalInfo.confInfo.warningTime);
    tmpRet = get_conf_value_int(plist, MODE_NAME, "comPareTime", &(gglobalInfo.confInfo.comPareTime));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "comPareTime[%d]", gglobalInfo.confInfo.comPareTime);
    tmpRet = get_conf_value_str(plist, MODE_NAME, "serverIp", gglobalInfo.confInfo.serverIp);
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "serverIp[%s]", gglobalInfo.confInfo.serverIp);
    tmpRet = get_conf_value_int(plist, MODE_NAME, "serverPort", &(gglobalInfo.confInfo.serverPort));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "serverPort[%d]", gglobalInfo.confInfo.serverPort);
    tmpRet = get_conf_value_int(plist, MODE_NAME, "selfPort", &(gglobalInfo.confInfo.selfPort));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "selfPort[%d]", gglobalInfo.confInfo.selfPort);

    /*----------------------------------新增------------------------------*/
    // 新增发布平台ip
    tmpRet = get_conf_value_str(plist, MODE_NAME, "publishingPlatformIp", gglobalInfo.increasePublishingInfo.publishingPlatformIp);
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "publishingPlatformIp[%s]", gglobalInfo.increasePublishingInfo.publishingPlatformIp);
    
    // 新增发布平台端口
    tmpRet = get_conf_value_int(plist, MODE_NAME, "publishingPlatformPort", &(gglobalInfo.increasePublishingInfo.publishingPlatformPort));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "publishingPlatformPort[%d]", gglobalInfo.increasePublishingInfo.publishingPlatformPort);
    
    // 发布平台心跳间隔
    tmpRet = get_conf_value_int(plist, MODE_NAME, "publishingPlatformHeartbeat", &(gglobalInfo.increasePublishingInfo.publishingPlatformHeartbeat));
    if (tmpRet < 0)
    {
        DBGPRINT(VDBG, "file:%s, line:%d, using DEFAULT_PUBLISHINGPLATFORMHEARTBEAT.", __FILE__, __LINE__);
        gglobalInfo.increasePublishingInfo.publishingPlatformHeartbeat = DEFAULT_PUBLISHINGPLATFORMHEARTBEAT;
    }
    DBGPRINT(VDBG, "file:%s, line:%d, publishingPlatformHeartbeat:[%d]", __FILE__, __LINE__, gglobalInfo.increasePublishingInfo.publishingPlatformHeartbeat);
    
    // 情报板ip
    tmpRet = get_conf_value_str(plist, MODE_NAME, "controllerIp", gglobalInfo.increasePublishingInfo.controllerIp);
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "controllerIp[%s]", gglobalInfo.increasePublishingInfo.controllerIp);
    
    // 情报板监听端口
    tmpRet = get_conf_value_int(plist, MODE_NAME, "controllerPort", &(gglobalInfo.increasePublishingInfo.controllerPort));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    DBGPRINT(VDBG, "controllerPort[%d]", gglobalInfo.increasePublishingInfo.controllerPort);
    
    // 情报板心跳间隔
    tmpRet = get_conf_value_int(plist, MODE_NAME, "controllerHeartbeat", &(gglobalInfo.increasePublishingInfo.controllerHeartbeat));
    if (tmpRet < 0)
    {
        DBGPRINT(VDBG, "file:%s, line:%d, using DEFAULT_CONTROLLERHEARTBEAT.", __FILE__, __LINE__);
        if (gglobalInfo.increasePublishingInfo.publishingPlatformHeartbeat > DEFAULT_CONTROLLERHEARTBEAT)
        {
            gglobalInfo.increasePublishingInfo.controllerHeartbeat = DEFAULT_CONTROLLERHEARTBEAT;
        }
        else
        {
            gglobalInfo.increasePublishingInfo.controllerHeartbeat = MINIMUM_CONTROLLERHEARTBEAT;
        }
    }
    DBGPRINT(VDBG, "file:%s, line:%d, controllerHeartbeat:%d", __FILE__, __LINE__, gglobalInfo.increasePublishingInfo.controllerHeartbeat);
    
    ret = 0;
__finish:
    if (plist)
        free_config(plist);
    return ret;
}

/*
des:
    从public.conf中获取配置信息
param:
    publicConfPath[IN]:publicConfPath配置文件路径
return:
    成功:0
    失败:小于0
*/
static int get_public_conf_setting(char *publicConfPath)
{
    int ret = -1;
    int tmpRet = -1;
    xlist *plist = NULL;

    // 判断文件是否存在 是否可读
    if (access(publicConfPath, F_OK) != 0 || access(publicConfPath, R_OK) != 0)
    {
        goto __finish;
    }

    // 链表初始化
    plist = xlist_init();
    if (!plist)
    {
        goto __finish;
    }
    // 加载配置文件
    load_config(plist, publicConfPath, NULL);

    // 获取自身需求信息
    // server
    tmpRet = get_conf_value_str(plist, "psms", "ip", gglobalInfo.confInfo.spmsInfo.ip);
    if (tmpRet < 0)
    {
        goto __finish;
    }
    if (strcmp(gglobalInfo.confInfo.spmsInfo.ip, "") == 0)
    {
        goto __finish;
    }
    tmpRet = get_conf_value_str(plist, "controller", "controllerIp", gglobalInfo.confInfo.spmsInfo.controllerIp);
    if (tmpRet < 0)
    {
        goto __finish;
    }
    tmpRet = get_conf_value_int(plist, "psms", "tcpPort", &(gglobalInfo.confInfo.spmsInfo.tcpPort));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    tmpRet = get_conf_value_int(plist, "psms", "udpPort", &(gglobalInfo.confInfo.spmsInfo.udpPort));
    if (tmpRet < 0)
    {
        goto __finish;
    }
    tmpRet = get_conf_value_str(plist, "terminal", "terminalIp", gglobalInfo.confInfo.terminalInfo.ip);
    if (tmpRet < 0)
    {
        goto __finish;
    }
    if (strcmp(gglobalInfo.confInfo.spmsInfo.controllerIp, "") == 0 && strcmp(gglobalInfo.confInfo.terminalInfo.ip, "") == 0)
    {

        goto __finish;
    }


    ret = 0;
__finish:
    if (plist)
    {
        free_config(plist);
    }
    return ret;
}

/*
des:
    从mian.conf中获取配置信息
param:
    mainConfPath[IN]:mainConfPath配置文件路径
return:
    成功:0
    失败:小于0
*/
static int get_main_conf_setting(char *mainConfPath)
{
    int ret = -1;
    int tmpRet = -1;
    xlist *plist = NULL;

    // 判断文件是否存在 是否可读
    if (access(mainConfPath, F_OK) != 0 || access(mainConfPath, R_OK) != 0)
    {
        goto __finish;
    }
    // 链表初始化
    plist = xlist_init();
    if (!plist)
    {
        goto __finish;
    }
    // 加载配置文件
    load_config(plist, mainConfPath, NULL);

    tmpRet = get_conf_value_int(plist, "octopusMain", "encodeType", &(gglobalInfo.confInfo.encodeType));
    if (tmpRet < 0)
    {
        goto __finish;
    }

    ret = 0;
__finish:
    if (plist)
    {
        free_config(plist);
    }
    return ret;
}

/*
des:
    加载配置文件信息
param:
    poctoShare[IN]:鱼头鱼爪共享信息
return:
    成功：0
    失败：小于0
*/
int load_config_info(PST_octoShare poctoShare)
{
    int ret = -1;
    int tmpRet = -1;
    char selfConfigPath[BUFFER_SIZE] = "";
    char publicConfPath[BUFFER_SIZE] = "";
    char mainConfPath[BUFFER_SIZE] = "";

    // 组self配置文件路径
    sprintf(selfConfigPath, "%s/../config/%s.conf", poctoShare->headPath, MODE_NAME);
    DBGPRINT(VDBG, "load_config_info() selfConfigPath[%s].", selfConfigPath);
    // 加载self配置文件，获取信息
    tmpRet = get_info_by_self_conf(selfConfigPath);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "load_config_info() get_info_by_self_conf() publicConfig error.");
        goto __finish;
    }
    // 组public配置文件路径
    sprintf(publicConfPath, "%s/../../config/public.conf", poctoShare->headPath);
    DBGPRINT(VDBG, "load_config_info() publicConfPath[%s].", publicConfPath);
    // 加载public配置文件，获取信息
    tmpRet = get_public_conf_setting(publicConfPath);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "load_config_info() get_public_conf_setting() error.");
        goto __finish;
    }
    // 组main配置文件路径
    sprintf(mainConfPath, "%s/../config/main.conf", poctoShare->headPath);
    DBGPRINT(VDBG, "load_config_info() mainConfPath[%s].", mainConfPath);
    // 加载main配置文件，获取信息
    tmpRet = get_main_conf_setting(mainConfPath);
    if (tmpRet < 0)
    {
        goto __finish;
    }

    ret = 0;
__finish:
    return ret;
}
