#include "common.h"
#include "leg_share.h"
#include "comm_head.h"
#include "work.h"
#include "load_config.h"
#include "setting.h"

int octo_leg_join(void *data)
{
    int ret = -1;
    int tmpRet = -1;
    pthread_t workTid = -1;
    PST_octoShare poctoShare = (PST_octoShare)data;

    DBGPRINT(VLOG, "octo_leg_join() modeName[%s] join start.", poctoShare->modeName);

    // 工作线程
    tmpRet = work_thread(&workTid);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "octo_leg_join() modeName[%s] work_thread start error.", poctoShare->modeName);
        goto __finish;
    }
    //////////////////////////////////////////////////////////////
    // 必备部分（未经允许不得随意改动）
    //////////////////////////////////////////////////////////////
    // 与鱼头的通信进程，负责认证、策略接受
    comm_head(poctoShare);
    //////////////////////////////////////////////////////////////
    // 必备部分结束
    //////////////////////////////////////////////////////////////
    // pthread_join(workTid, NULL);
__finish:
    DBGPRINT(VLOG, "octo_leg_join() modeName[%s] join end.", poctoShare->modeName);
    return 0;
}

int octo_leg_free(void *data)
{
    PST_octoShare poctoShare = (PST_octoShare)data;
    
    // 释放鱼爪垃圾数据
    free_leg_info(data);
    DBGPRINT(VLOG, "octo_leg_free() modeName[%s] free end.", poctoShare->modeName);
    return 0;
}

// 头文件开放接口
char *octo_leg_init(void *data)
{
    int ret = -1;
    int tmpRet = -1;
    PST_octoShare poctoShare = (PST_octoShare)data;

    DBGPRINT(VLOG, "octo_leg_init() modeName[%s] start.", MODE_NAME);
    //////////////////////////////////////////////////////////////
    // 必备部分（未经允许不得随意改动）
    //////////////////////////////////////////////////////////////
    // 赋值so名称
    strcpy(poctoShare->modeName, MODE_NAME);
    // 赋值so类型
    poctoShare->runAlways = ALWAYS_TYPE;
    // 赋值工作方法和释放方法的函数指针
    poctoShare->join_handler = (octo_leg_join_handler *)octo_leg_join;
    poctoShare->free_handler = (octo_leg_free_handler *)octo_leg_free;
    //////////////////////////////////////////////////////////////
    // 必备部分结束
    //////////////////////////////////////////////////////////////

    // 鱼爪启动信息初始化
    tmpRet = init_leg_info(poctoShare);
    if (tmpRet < 0)
    {
        DBGPRINT(VLOG, "octo_leg_init() init_leg_info error.");
        goto __finish;
    }

    // 加载配置
    tmpRet = load_config_info(poctoShare);
    if (tmpRet < 0)
    {
        DBGPRINT(VLOG, "octo_leg_init() load_config_info error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "octo_leg_init() poctoShare->modeName[%s].", poctoShare->modeName);

    ret = 0;
__finish:
    return poctoShare->modeName;
}


