#include <time.h>
#include "afx/m_afx.h"
#include "operate_file.h"
#include "afx/xcjson.h"
#include "leg_share.h"
#include "setting.h"
#include "common.h"
#include "afx/xiconv.h"
#include <sys/types.h>
#include <dirent.h>

extern ST_globalInfo gglobalInfo;
extern ST_reportFileInfo reportFileInfo;

/*
des:
    将连接ip组成审计json
parm：
    clientIp[IN]:客户端连接IP
    serverIp[IN]:服务器IP
ruturn:
    返回拼成的json
*/
char *create_ip_audit_json(char *clientIp, char *serverIp)
{
    int ret = -1;
    int tmpRet = -1;
    time_t unix_time = 0;
    int packetDataLen = 0;
    char *jsonout = NULL;
    char contentBuf[BUFFER_SIZE] = "";
    cJSON *jsonroot = NULL;
    cJSON *expand = NULL;

    // 获取当前时间
    unix_time = time(NULL);

    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建expand对象
    expand = cJSON_CreateObject();

    sprintf(contentBuf, "发现新的连接,来源IP:%s,目的IP:%s", clientIp, serverIp);
    cJSON_AddStringToObject(jsonroot, "content", contentBuf);
    cJSON_AddNumberToObject(jsonroot, "unixTime", unix_time);
    cJSON_AddNumberToObject(expand, "auditType", AUDIT_TYPE_IP);
    cJSON_AddStringToObject(expand, "srcIP", clientIp);
    cJSON_AddStringToObject(expand, "dstIP", serverIp);
    cJSON_AddItemToObject(jsonroot, "expand", expand);

    jsonout = cJSON_Print(jsonroot);

    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }

    return jsonout;
}

/*
des:
    将情报板内容变化组成告警json
parm：
    oldContent[IN]:情报板上一次显示内容
    newContent[IN]:情报板当前显示内容
ruturn:
    返回拼成的json
*/
char *create_screen_content_warnning_json(char *oldContent, char *newContent)
{
    int ret = -1;
    int tmpRet = -1;
    time_t unix_time = 0;
    int packetDataLen = 0;
    char *jsonout = NULL;
    char contentBuf[BUFFER_SIZE] = "";
    int contentBufLen = 0;
    char secondContent[BUFFER_SIZE] = "";
    char *gOldContent = NULL;
    int oldContentLen = 0;
    char *gNewContent = NULL;
    int newContentLen = 0;
    char *uContentBuf = NULL;
    char dealRstBuf[BUFFER_SIZE] = "已监测";
    int dealRstBufLen = 0;
    char *udealRstBuf = NULL;
    cJSON *jsonroot = NULL;
    cJSON *expand = NULL;

    // 获取当前时间
    unix_time = time(NULL);

    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建expand对象
    expand = cJSON_CreateObject();

    // oldContent转码
    oldContentLen = strlen(oldContent);
    gOldContent = xmalloc(oldContentLen * 3);
    if (gOldContent == NULL)
    {
        DBGPRINT(VERR, "create_screen_content_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xu2g(oldContent, oldContentLen, gOldContent, oldContentLen * 3);
    if (tmpRet < 0)
    {
        sprintf(contentBuf, "触发内容改变告警,内容由“%s”", oldContent);
    }
    else
    {
        sprintf(contentBuf, "触发内容改变告警,内容由“%s”", gOldContent);
    }

    // newContent转码
    newContentLen = strlen(newContent);
    gNewContent = xmalloc(newContentLen * 3);
    if (gNewContent == NULL)
    {
        DBGPRINT(VERR, "create_screen_content_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xu2g(newContent, newContentLen, gNewContent, newContentLen * 3);
    if (tmpRet < 0)
    {
        sprintf(secondContent, "改变为“%s”", newContent);
    }
    else
    {
        sprintf(secondContent, "改变为“%s”", gNewContent);
    }

    // contentBuf转码
    strcat(contentBuf, secondContent);
    contentBufLen = strlen(contentBuf);
    uContentBuf = xmalloc(contentBufLen * 3);
    if (uContentBuf == NULL)
    {
        DBGPRINT(VERR, "create_screen_content_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xg2u(contentBuf, contentBufLen, uContentBuf, contentBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "content", contentBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "content", uContentBuf);
    }
    // dealRstBuf转码
    dealRstBufLen = strlen(dealRstBuf);
    udealRstBuf = xmalloc(dealRstBufLen * 3);
    if (udealRstBuf == NULL)
    {
        DBGPRINT(VERR, "create_screen_content_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xg2u(dealRstBuf, dealRstBufLen, udealRstBuf, dealRstBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "dealRst", dealRstBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "dealRst", udealRstBuf);
    }

    cJSON_AddNumberToObject(jsonroot, "unixTime", unix_time);
    cJSON_AddNumberToObject(expand, "warningType", WARNNING_TYPE_SCREEN_CONTENT_CHANGED);
    cJSON_AddItemToObject(jsonroot, "expand", expand);

    jsonout = cJSON_Print(jsonroot);

    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }

    xfree(gOldContent);
    xfree(gNewContent);
    xfree(uContentBuf);
    xfree(udealRstBuf);

    return jsonout;
}

/*
des:
    将连接ip组成告警json
parm：
    clientIp[IN]:客户端连接IP
    serverIp[IN]:服务器IP
ruturn:
    返回拼成的json
*/
char *create_ip_warnning_json(char *clientIp, char *serverIp)
{
    int ret = -1;
    int tmpRet = -1;
    time_t unix_time = 0;
    int packetDataLen = 0;
    char *jsonout = NULL;
    char contentBuf[BUFFER_SIZE] = "";
    char dealRstBuf[BUFFER_SIZE] = "已阻止";
    cJSON *jsonroot = NULL;
    cJSON *expand = NULL;

    // 获取当前时间
    unix_time = time(NULL);

    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建expand对象
    expand = cJSON_CreateObject();

    sprintf(contentBuf, "触发准入告警,来源IP:%s,目的IP:%s", clientIp, serverIp);
    cJSON_AddStringToObject(jsonroot, "content", contentBuf);
    cJSON_AddStringToObject(jsonroot, "dealRst", dealRstBuf);
    cJSON_AddNumberToObject(jsonroot, "unixTime", unix_time);
    cJSON_AddNumberToObject(expand, "warningType", WARNNING_TYPE_IP);
    cJSON_AddStringToObject(expand, "srcIP", clientIp);
    cJSON_AddStringToObject(expand, "dstIP", serverIp);
    cJSON_AddItemToObject(jsonroot, "expand", expand);

    jsonout = cJSON_Print(jsonroot);

    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }

    return jsonout;
}

/*
des:
    将数据包内容组成告警json
parm：
    warningAndReviewBuf[IN]:待写入的告警信息
ruturn:
    返回拼成的json
*/
char *create_warnning_json(char *warningAndReviewBuf)
{
    int ret = -1;
    int tmpRet = -1;
    time_t unix_time = 0;
    char *jsonout = NULL;
    char contentBuf[BUFFER_SIZE] = "";
    int contentBufLen = 0;
    char *uContentBuf = NULL;
    char dealRstBuf[BUFFER_SIZE] = "已阻止";
    int dealRstBufLen = 0;
    char *udealRstBuf = NULL;
    char *index = NULL;
    int warningAndReviewBufLen = 0;
    char *uWarningAndReviewBuf = NULL;
    char *baseText = NULL;
    cJSON *jsonroot = NULL;
    cJSON *expand = NULL;

    // 获取当前时间
    unix_time = time(NULL);

    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建expand对象
    expand = cJSON_CreateObject();

    // contentBuf转码
    sprintf(contentBuf, "触发白名单告警");
    contentBufLen = strlen(contentBuf);
    uContentBuf = xmalloc(contentBufLen * 3);
    if (uContentBuf == NULL)
    {
        DBGPRINT(VERR, "create_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xg2u(contentBuf, contentBufLen, uContentBuf, contentBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "content", contentBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "content", uContentBuf);
    }
    // dealRstBuf转码
    dealRstBufLen = strlen(dealRstBuf);
    udealRstBuf = xmalloc(dealRstBufLen * 3);
    if (udealRstBuf == NULL)
    {
        DBGPRINT(VERR, "create_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xg2u(dealRstBuf, dealRstBufLen, udealRstBuf, dealRstBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "dealRst", dealRstBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "dealRst", udealRstBuf);
    }

    // 文本内容转码
    warningAndReviewBufLen = strlen(warningAndReviewBuf);
    uWarningAndReviewBuf = xmalloc(warningAndReviewBufLen * 3);
    if (uWarningAndReviewBuf == NULL)
    {
        DBGPRINT(VERR, "create_warnning_json() xmalloc() uWarningAndReviewBuf error.");
        goto __finish;
    }
    tmpRet = xg2u(warningAndReviewBuf, warningAndReviewBufLen, uWarningAndReviewBuf, warningAndReviewBufLen * 3);
    if (tmpRet < 0)
    {
        // 文本内容转base64
        int tmpLen = 0;
        tmpLen = strlen(warningAndReviewBuf);
        baseText = xmalloc(tmpLen * 3);
        EncodeBase64(warningAndReviewBuf, tmpLen, baseText);
    }
    else
    {
        // 文本内容转base64
        int tmpLen = 0;
        tmpLen = strlen(uWarningAndReviewBuf);
        baseText = xmalloc(tmpLen * 3);
        EncodeBase64(uWarningAndReviewBuf, tmpLen, baseText);
    }

    cJSON_AddNumberToObject(jsonroot, "unixTime", unix_time);
    cJSON_AddNumberToObject(expand, "warningType", WARNNING_TYPE_DANGER_WORD);
    cJSON_AddStringToObject(expand, "hitPtoType", "0x31,0x30");
    cJSON_AddStringToObject(expand, "dangerWord", "");
    cJSON_AddStringToObject(expand, "content", baseText);
    cJSON_AddItemToObject(jsonroot, "expand", expand);

    jsonout = cJSON_Print(jsonroot);

    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }

    xfree(baseText);
    xfree(uContentBuf);
    xfree(udealRstBuf);
    xfree(uWarningAndReviewBuf);

    return jsonout;
}

/*
des:
    将图片内容md5组成告警json
parm：
    bmpMd5[IN]:图片内容md5
ruturn:
    返回拼成的json
*/
char *create_bmp_warnning_json(char *bmpName, char *bmpValue, char *bmpMd5)
{
    int ret = -1;
    int tmpRet = -1;
    time_t unix_time = 0;
    char *jsonout = NULL;
    char contentBuf[BUFFER_SIZE] = "";
    int contentBufLen = 0;
    char *uContentBuf = NULL;
    char dealRstBuf[BUFFER_SIZE] = "已阻止";
    int dealRstBufLen = 0;
    char *udealRstBuf = NULL;
    char *index = NULL;
    cJSON *jsonroot = NULL;
    cJSON *expand = NULL;

    // 获取当前时间
    unix_time = time(NULL);

    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建expand对象
    expand = cJSON_CreateObject();

    // contentBuf转码
    sprintf(contentBuf, "触发图片上载告警");
    contentBufLen = strlen(contentBuf);
    uContentBuf = xmalloc(contentBufLen * 3);
    if (uContentBuf == NULL)
    {
        DBGPRINT(VERR, "create_bmp_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xg2u(contentBuf, contentBufLen, uContentBuf, contentBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "content", contentBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "content", uContentBuf);
    }
    // dealRstBuf转码
    dealRstBufLen = strlen(dealRstBuf);
    udealRstBuf = xmalloc(dealRstBufLen * 3);
    if (udealRstBuf == NULL)
    {
        DBGPRINT(VERR, "create_warnning_json() xmalloc() error.");
        goto __finish;
    }
    tmpRet = xg2u(dealRstBuf, dealRstBufLen, udealRstBuf, dealRstBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "dealRst", dealRstBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "dealRst", udealRstBuf);
    }

    cJSON_AddNumberToObject(jsonroot, "unixTime", unix_time);
    cJSON_AddNumberToObject(expand, "warningType", WARNNING_TYPE_BMP);
    cJSON_AddStringToObject(expand, "bmpName", bmpName);
    cJSON_AddStringToObject(expand, "bmpMd5", bmpMd5);
    cJSON_AddStringToObject(expand, "content", bmpValue);
    cJSON_AddItemToObject(jsonroot, "expand", expand);

    jsonout = cJSON_Print(jsonroot);

    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }

    xfree(uContentBuf);
    xfree(udealRstBuf);

    return jsonout;
}

/*
des:
    将数据包内容组成审计json
parm：
    auditBuf[IN]:待写入的审计消息
ruturn:
    返回拼成的json
*/
char *create_audit_json(char *auditBuf)
{
    int ret = -1;
    int tmpRet = -1;
    time_t unix_time = 0;

    char *jsonout = NULL;
    char contentBuf[BUFFER_SIZE] = "有新的内容信息传送";
    int contentBufLen = 0;
    char *index = NULL;
    int auditBufLen = 0;
    char *baseText = NULL;
    char *uContentBuf = NULL;
    cJSON *jsonroot = NULL;
    cJSON *expand = NULL;
    char *uAuditBuf = NULL;

    // 获取当前时间
    unix_time = time(NULL);

    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建expand对象
    expand = cJSON_CreateObject();

    // contentBuf转码
    contentBufLen = strlen(contentBuf);
    uContentBuf = xmalloc(contentBufLen * 3);
    if (uContentBuf == NULL)
    {
        DBGPRINT(VERR, "create_audit_json() xmalloc() uContentBuf error.");
        goto __finish;
    }
    tmpRet = xg2u(contentBuf, contentBufLen, uContentBuf, contentBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "content", contentBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "content", uContentBuf);
    }

    // 文本内容转码
    auditBufLen = strlen(auditBuf);
    uAuditBuf = xmalloc(auditBufLen * 3);
    if (uAuditBuf == NULL)
    {
        DBGPRINT(VERR, "create_audit_json() xmalloc() uAuditBuf error.");
        goto __finish;
    }
    tmpRet = xg2u(auditBuf, auditBufLen, uAuditBuf, auditBufLen * 3);
    if (tmpRet < 0)
    {
        // 文本内容转base64
        int tmpLen = 0;
        tmpLen = strlen(auditBuf);
        baseText = xmalloc(tmpLen * 3);
        EncodeBase64(auditBuf, tmpLen, baseText);
    }
    else
    {
        // 文本内容转base64
        int tmpLen = 0;
        tmpLen = strlen(uAuditBuf);
        baseText = xmalloc(tmpLen * 3);
        EncodeBase64(uAuditBuf, tmpLen, baseText);
    }

    cJSON_AddNumberToObject(jsonroot, "unixTime", unix_time);
    cJSON_AddNumberToObject(expand, "auditType", AUDIT_TYPE_DANGER_WORD);
    cJSON_AddStringToObject(expand, "hitPtoType", "0x31,0x30");
    cJSON_AddStringToObject(expand, "content", baseText);
    cJSON_AddItemToObject(jsonroot, "expand", expand);
    jsonout = cJSON_Print(jsonroot);
    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }
    xfree(baseText);
    xfree(uContentBuf);
    xfree(uAuditBuf);
    return jsonout;
}

/*
des:
    将图片md5组成审计json
parm：
    bmpMd5[IN]:图片md5
ruturn:
    返回拼成的json
*/
char *create_bmp_audit_json(char *bmpName, char *bmpValue, char *bmpMd5)
{
    int ret = -1;
    int tmpRet = -1;
    time_t unix_time = 0;
    char *jsonout = NULL;
    char contentBuf[BUFFER_SIZE] = "有新的图片信息传送";
    int contentBufLen = 0;
    char *uContentBuf = NULL;
    cJSON *jsonroot = NULL;
    cJSON *expand = NULL;

    // 获取当前时间
    unix_time = time(NULL);

    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建expand对象
    expand = cJSON_CreateObject();

    // contentBuf转码
    contentBufLen = strlen(contentBuf);
    uContentBuf = xmalloc(contentBufLen * 3);
    if (uContentBuf == NULL)
    {
        DBGPRINT(VERR, "create_bmp_audit_json() xmalloc() uContentBuf error.");
        goto __finish;
    }
    tmpRet = xg2u(contentBuf, contentBufLen, uContentBuf, contentBufLen * 3);
    if (tmpRet < 0)
    {
        cJSON_AddStringToObject(jsonroot, "content", contentBuf);
    }
    else
    {
        cJSON_AddStringToObject(jsonroot, "content", uContentBuf);
    }

    cJSON_AddNumberToObject(jsonroot, "unixTime", unix_time);
    cJSON_AddNumberToObject(expand, "auditType", AUDIT_TYPE_DBMP);
    cJSON_AddStringToObject(expand, "hitPtoType", "0x31,0x30");
    cJSON_AddStringToObject(expand, "bmpName", bmpName);
    cJSON_AddStringToObject(expand, "bmpMd5", bmpMd5);
    cJSON_AddStringToObject(expand, "content", bmpValue);
    cJSON_AddItemToObject(jsonroot, "expand", expand);
    jsonout = cJSON_Print(jsonroot);
    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }
    xfree(uContentBuf);
    return jsonout;
}

/*
des:
    将连接的IP组成json对象写入审计文件
parm：
    clientIp[IN]:客户端连接IP
    serverIp[IN]:服务器IP
    fp[IN]:审计文件的文件句柄
return：
    -1：失败
    0：成功
*/
int write_ip_audit_file(char *clientIp, char *serverIp, int fp)
{
    int ret = -1;
    int tmpRet = -1;
    int json_buf_len = 0;
    char *json_buf = NULL;

    // 组json
    json_buf = create_ip_audit_json(clientIp, serverIp);
    if (json_buf == NULL)
    {
        DBGPRINT(VERR, "write_ip_audit_file() create_ip_audit_json() error.");
        goto __finish;
    }

    // 写json长度
    json_buf_len = strlen(json_buf);
    tmpRet = write(fp, &json_buf_len, sizeof(json_buf_len));
    if (tmpRet != sizeof(json_buf_len))
    {
        DBGPRINT(VERR, "write_ip_audit_file() write() json_buf_len error.");
        goto __finish;
    }

    // 写json
    tmpRet = write(fp, json_buf, json_buf_len);
    if (tmpRet != json_buf_len)
    {
        DBGPRINT(VERR, "write_ip_audit_file() write() json_buf error.");
        goto __finish;
    }

    ret = 0;
__finish:

    xfree(json_buf);

    return ret;
}

/*
des:
    将情报板的旧内容与新内容写入告警文件
parm：
    oldContent[IN]:情报板上一次显示内容
    newContent[IN]:情报板当前显示内容
    fp[IN]:告警文件的文件句柄
return：
    -1：失败
    0：成功
*/
int write_screen_content_warnning_file(char *oldContent, char *newContent, int fp)
{
    int ret = -1;
    int tmpRet = -1;
    int json_buf_len = 0;
    char *json_buf = NULL;

    // 组json
    json_buf = create_screen_content_warnning_json(oldContent, newContent);
    if (json_buf == NULL)
    {
        DBGPRINT(VERR, "write_screen_content_warnning_file() create_screen_content_warnning_json() error.");
        goto __finish;
    }

    // 写json长度
    json_buf_len = strlen(json_buf);
    tmpRet = write(fp, &json_buf_len, sizeof(json_buf_len));
    if (tmpRet != sizeof(json_buf_len))
    {
        DBGPRINT(VERR, "write_screen_content_warnning_file() write() json_buf_len error.");
        goto __finish;
    }

    // 写json
    tmpRet = write(fp, json_buf, json_buf_len);
    if (tmpRet != json_buf_len)
    {
        DBGPRINT(VERR, "write_screen_content_warnning_file() write() json_buf error.");
        goto __finish;
    }

    ret = 0;
__finish:

    xfree(json_buf);

    return ret;
}

/*
des:
    将连接的IP组成json对象写入告警文件
parm：
    clientIp[IN]:客户端连接IP
    serverIp[IN]:服务器IP
    fp[IN]:告警文件的文件句柄
return：
    -1：失败
    0：成功
*/
int write_ip_warnning_file(char *clientIp, char *serverIp, int fp)
{
    int ret = -1;
    int tmpRet = -1;
    int json_buf_len = 0;
    char *json_buf = NULL;

    // 组json
    json_buf = create_ip_warnning_json(clientIp, serverIp);
    if (json_buf == NULL)
    {
        DBGPRINT(VERR, "write_ip_warnning_file() create_ip_warnning_json() error.");
        goto __finish;
    }

    // 写json长度
    json_buf_len = strlen(json_buf);
    tmpRet = write(fp, &json_buf_len, sizeof(json_buf_len));
    if (tmpRet != sizeof(json_buf_len))
    {
        DBGPRINT(VERR, "write_ip_warnning_file() write() json_buf_len error.");
        goto __finish;
    }

    // 写json
    tmpRet = write(fp, json_buf, json_buf_len);
    if (tmpRet != json_buf_len)
    {
        DBGPRINT(VERR, "write_ip_warnning_file() write() json_buf error.");
        goto __finish;
    }

    ret = 0;
__finish:

    xfree(json_buf);

    return ret;
}

/*
des:
    将解析到的数据包内容组成json对象写入告警文件
parm：
    warningAndReviewBuf[IN]:待写入的告警信息
    fp[IN]:待写入文件的文件描述符
    dangerWord[IN]:触发的危险词
return：
    -1：失败
    0：成功
*/
int write_warnning_file(char *warningAndReviewBuf, int fp)
{
    int ret = -1;
    int tmpRet = -1;
    int json_buf_len = 0;
    char *json_buf = NULL;

    if (strcmp(warningAndReviewBuf, "") == 0)
    {
        ret = 0;
        goto __finish;
    }

    // 去除，号
    if (warningAndReviewBuf[strlen(warningAndReviewBuf) - 1] == ',')
    {
        warningAndReviewBuf[strlen(warningAndReviewBuf) - 1] = '\0';
    }

    DBGPRINT(VDBG, "write_warnning_file() warningAndReviewBuf[%s].", warningAndReviewBuf);

    // 组json
    json_buf = create_warnning_json(warningAndReviewBuf);
    if (json_buf == NULL)
    {
        DBGPRINT(VERR, "write_warnning_file() create_to_file_json() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "write_warnning_file() json_buf[%s].", json_buf);

    // 写json长度
    json_buf_len = strlen(json_buf);
    tmpRet = write(fp, &json_buf_len, sizeof(json_buf_len));
    if (tmpRet != sizeof(json_buf_len))
    {
        DBGPRINT(VERR, "write_warnning_file() write() json_buf_len error.");
        goto __finish;
    }

    // 写json
    tmpRet = write(fp, json_buf, json_buf_len);
    if (tmpRet != json_buf_len)
    {
        DBGPRINT(VERR, "write_warnning_file() write() json_buf error.");
        goto __finish;
    }

    ret = 0;
__finish:

    xfree(json_buf);

    return ret;
}

/*
des:
    将解析到的数据包内容组成json对象写入告警文件
parm：
    bmpMd5[IN]:图片MD5
    fp[IN]:待写入文件的文件描述符
return：
    -1：失败
    0：成功
*/
int write_bmp_warnning_file(char *bmpName, char *bmpValue, char *bmpMd5, int fp)
{
    int ret = -1;
    int tmpRet = -1;
    int json_buf_len = 0;
    char *json_buf = NULL;

    // 组json
    json_buf = create_bmp_warnning_json(bmpName, bmpValue, bmpMd5);
    if (json_buf == NULL)
    {
        DBGPRINT(VERR, "write_bmp_warnning_file() create_to_file_json() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "write_bmp_warnning_file() json_buf[%s].", json_buf);

    // 写json长度
    json_buf_len = strlen(json_buf);
    tmpRet = write(fp, &json_buf_len, sizeof(json_buf_len));
    if (tmpRet != sizeof(json_buf_len))
    {
        DBGPRINT(VERR, "write_bmp_warnning_file() write() json_buf_len error.");
        goto __finish;
    }

    // 写json
    tmpRet = write(fp, json_buf, json_buf_len);
    if (tmpRet != json_buf_len)
    {
        DBGPRINT(VERR, "write_bmp_warnning_file() write() json_buf error.");
        goto __finish;
    }

    ret = 0;
__finish:

    xfree(json_buf);

    return ret;
}

/*
des:
    将解析到的数据包内容组成json对象写入审计文件
parm：
    auditBuf[IN]:待写入的审计消息
    fp[IN]:待写入文件的文件描述符
return：
    -1：失败
    0：成功
*/
int write_audit_file(char *auditBuf, int fp)
{
    int ret = -1;
    int tmpRet = -1;
    int json_buf_len = 0;
    char *json_buf = NULL;

    if (strcmp(auditBuf, "") == 0)
    {
        ret = 0;
        goto __finish;
    }

    // 去除，号
    if (auditBuf[strlen(auditBuf) - 1] == ',')
    {
        auditBuf[strlen(auditBuf) - 1] = '\0';
    }

    DBGPRINT(VDBG, "write_audit_file() auditBuf[%s].", auditBuf);

    // 组json
    json_buf = create_audit_json(auditBuf);
    if (json_buf == NULL)
    {
        DBGPRINT(VERR, "write_audit_file() create_audit_json() error.");
        goto __finish;
    }
    // 写json长度
    json_buf_len = strlen(json_buf);
    tmpRet = write(fp, &json_buf_len, sizeof(json_buf_len));
    if (tmpRet != sizeof(json_buf_len))
    {
        DBGPRINT(VERR, "write_audit_file() write() json_buf_len error.");
        goto __finish;
    }
    // 写json
    tmpRet = write(fp, json_buf, json_buf_len);
    if (tmpRet != json_buf_len)
    {
        DBGPRINT(VERR, "write_audit_file() write() json_buf error.");
        goto __finish;
    }

    ret = 0;
__finish:

    xfree(json_buf);

    return ret;
}

/*
des:
    将解析到的数据包内容组成json对象写入审计文件
parm：
    auditBuf[IN]:待写入的审计消息
    fp[IN]:待写入文件的文件描述符
return：
    -1：失败
    0：成功
*/
int write_bmp_audit_file(char *bmpName, char *bmpValue, char *bmpMd5, int fp)
{
    int ret = -1;
    int tmpRet = -1;
    int json_buf_len = 0;
    char *json_buf = NULL;

    // 组json
    json_buf = create_bmp_audit_json(bmpName, bmpValue, bmpMd5);
    if (json_buf == NULL)
    {
        DBGPRINT(VERR, "write_bmp_audit_file() create_audit_json() error.");
        goto __finish;
    }
    // 写json长度
    json_buf_len = strlen(json_buf);
    tmpRet = write(fp, &json_buf_len, sizeof(json_buf_len));
    if (tmpRet != sizeof(json_buf_len))
    {
        DBGPRINT(VERR, "write_bmp_audit_file() write() json_buf_len error.");
        goto __finish;
    }
    // 写json
    tmpRet = write(fp, json_buf, json_buf_len);
    if (tmpRet != json_buf_len)
    {
        DBGPRINT(VERR, "write_bmp_audit_file() write() json_buf error.");
        goto __finish;
    }

    ret = 0;
__finish:

    xfree(json_buf);

    return ret;
}

/*
des:
    读文件内容后拼成json
parm：
    fileName[IN]:待读的文件名
    reportType[IN]:
return:
    拼完后的json
*/
char *read_file_and_create_json(char *fileName, int reportType)
{
    int ret = -1;
    int tmpRet = -1;
    int tmpBufLen = 0;
    char *tmpBuf = NULL;
    char *jsonout = NULL;
    FILE *fp = NULL;
    cJSON *rootArry = NULL;
    cJSON *jsonroot = NULL;
    cJSON *contnetArray = NULL;

    // 创建root_array数组
    rootArry = cJSON_CreateArray();
    // 创建jsonroot对象
    jsonroot = cJSON_CreateObject();
    // 创建content_array数组
    contnetArray = cJSON_CreateArray();

    cJSON_AddNumberToObject(jsonroot, "transType", TCP_TRANS_TYPE);
    cJSON_AddNumberToObject(jsonroot, "reportType", reportType);
    cJSON_AddStringToObject(jsonroot, "modeName", MODE_NAME);

    tmpBuf = xmalloc(file_size(fileName) + MINBUFFER_SIZE);
    if (tmpBuf == NULL)
    {
        DBGPRINT(VDBG, "read_file_and_create_json() tmpBuf xmalloc() error.");
        goto __finish;
    }

    fp = file_open(fileName, "rb");
    // 读文件
    while (fread(&tmpBufLen, 1, 4, fp) == 4)
    {
        cJSON *checkRoot = NULL;

        fread(tmpBuf, 1, tmpBufLen, fp);
        DBGPRINT(VDBG, "read_file_and_create_json() fread tmpBuf[%s].", tmpBuf);

        // 判断tmp_buf是否为完整的json对象
        checkRoot = cJSON_Parse(tmpBuf);
        if (checkRoot == NULL)
        {
            DBGPRINT(VDBG, "read_file_and_create_json() checkRoot is NULL.");
            continue;
        }
        cJSON_AddItemToArray(contnetArray, checkRoot);
    }
    cJSON_AddItemToObject(jsonroot, "content", contnetArray);

    cJSON_AddItemToArray(rootArry, jsonroot);

    jsonout = cJSON_Print(rootArry);

__finish:

    if (fp != NULL)
    {
        fclose(fp);
    }
    if (rootArry)
    {
        cJSON_Delete(rootArry);
    }

    xfree(tmpBuf);

    return jsonout;
}

/*
des:
    关闭文件描述、打开新文件描述符、获取上一次文件内容、向鱼头汇报信息、删除上一次文件
*/
void send_warnning_already_file()
{
    int ret = -1;
    int tmpRet = -1;
    char *jsonSend = NULL;
    char oldWarnningFileName[BUFFER_SIZE] = "";
    char newWarnningFileName[BUFFER_SIZE] = "";
    char rmFileCmd[BUFFER_SIZE] = "";

    // 关闭文件描述符
    close(reportFileInfo.warnningFileInfo.fp);

    // 获取新的随机数，将上一次随机数置为旧值
    reportFileInfo.warnningFileInfo.oldRand = reportFileInfo.warnningFileInfo.newRand;
    reportFileInfo.warnningFileInfo.newRand = rand();

    // 打开新的文件描述符
    sprintf(newWarnningFileName, "%s/../../data/sendCache/%s/%d.warnning", gglobalInfo.octoShare.headPath, MODE_NAME, reportFileInfo.warnningFileInfo.newRand);
    reportFileInfo.warnningFileInfo.fp = open(newWarnningFileName, O_CREAT | O_TRUNC | O_RDWR, 0666);

    // 获取上一次文件内容
    sprintf(oldWarnningFileName, "%s/../../data/sendCache/%s/%d.warnning", gglobalInfo.octoShare.headPath, MODE_NAME, reportFileInfo.warnningFileInfo.oldRand);
    tmpRet = file_size(oldWarnningFileName);
    if (tmpRet <= 0)
    {
        DBGPRINT(VDBG, "send_audit_already_file() oldWarnningFileName[%s]  is NULL.", oldWarnningFileName);
        goto __remove;
    }
    jsonSend = read_file_and_create_json(oldWarnningFileName, WARNNING_REPORT_TYPE);
    if (jsonSend == NULL)
    {
        DBGPRINT(VDBG, "send_warnning_already_file() read_file_and_create_json() error.");
        goto __finish;
    }
    // 反馈结果
    tmpRet = xsk_send((char *)jsonSend, strlen(jsonSend), gglobalInfo.commSocketfd);
    if (tmpRet <= 0)
    {
        DBGPRINT(VERR, "send_warnning_already_file() xsk_send error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "send_warnning_already_file() xsk_send ok[%s].", jsonSend);
__remove:
    // 删除文件
    tmpRet = remove(oldWarnningFileName);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "send_warnning_already_file() remove() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "send_warnning_already_file() oldWarnningFileName[%s] already delete.", oldWarnningFileName);

__finish:

    xfree(jsonSend);

    return;
}

/*
des:
    关闭文件描述、打开新文件描述符、获取上一次文件内容、向鱼头汇报信息、删除上一次文件
*/
void send_audit_already_file()
{
    int ret = -1;
    int tmpRet = -1;
    char *jsonSend = NULL;
    char oldAuditFileName[BUFFER_SIZE] = "";
    char newAuditFileName[BUFFER_SIZE] = "";
    char rmFileCmd[BUFFER_SIZE] = "";

    // 关闭文件描述符
    close(reportFileInfo.auditFileInfo.fp);

    // 获取新的随机数，将上一次随机数置为旧值
    reportFileInfo.auditFileInfo.oldRand = reportFileInfo.auditFileInfo.newRand;
    reportFileInfo.auditFileInfo.newRand = rand();

    // 打开新的文件描述符
    sprintf(newAuditFileName, "%s/../../data/sendCache/%s/%d.audit", gglobalInfo.octoShare.headPath, MODE_NAME, reportFileInfo.auditFileInfo.newRand);
    reportFileInfo.auditFileInfo.fp = open(newAuditFileName, O_CREAT | O_TRUNC | O_RDWR, 0666);

    // 获取上一次文件内容
    sprintf(oldAuditFileName, "%s/../../data/sendCache/%s/%d.audit", gglobalInfo.octoShare.headPath, MODE_NAME, reportFileInfo.auditFileInfo.oldRand);
    tmpRet = file_size(oldAuditFileName);
    if (tmpRet <= 0)
    {
        DBGPRINT(VDBG, "send_audit_already_file() oldAuditFileName[%s]  is NULL.", oldAuditFileName);
        goto __remove;
    }

    jsonSend = read_file_and_create_json(oldAuditFileName, AUDIT_REPORT_TYPE);
    if (jsonSend == NULL)
    {
        DBGPRINT(VDBG, "send_audit_already_file() read_file_and_create_json() error.");
        goto __finish;
    }
    // 反馈结果
    tmpRet = xsk_send((char *)jsonSend, strlen(jsonSend), gglobalInfo.commSocketfd);
    if (tmpRet <= 0)
    {
        DBGPRINT(VERR, "send_audit_already_file() xsk_send error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "send_audit_already_file() xsk_send ok[%s].", jsonSend);
__remove:
    // 删除文件
    tmpRet = remove(oldAuditFileName);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "send_audit_already_file() remove() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "send_audit_already_file() oldAuditFileName[%s] already delete.", oldAuditFileName);

__finish:

    xfree(jsonSend);

    return;
}
/*
des:
    根据文件名，发送文件
parm：
    tmpAbpath[IN]:文件名
*/
void send_last_file(char *tmpAbpath)
{
    int ret = -1;
    int tmpRet = -1;
    char *jsonSend = NULL;
    char rmFileCmd[BUFFER_SIZE] = "";

    tmpRet = file_size(tmpAbpath);
    if (tmpRet <= 0)
    {
        DBGPRINT(VDBG, "send_last_file() tmpAbpath[%s]  is NULL.", tmpAbpath);
        goto __remove;
    }

    if (strstr(tmpAbpath, "audit") != NULL)
    {
        jsonSend = read_file_and_create_json(tmpAbpath, AUDIT_REPORT_TYPE);
        if (jsonSend == NULL)
        {
            DBGPRINT(VDBG, "send_last_file() read_file_and_create_json() error.");
            goto __finish;
        }
    }
    else
    {
        jsonSend = read_file_and_create_json(tmpAbpath, WARNNING_REPORT_TYPE);
        if (jsonSend == NULL)
        {
            DBGPRINT(VDBG, "send_last_file() read_file_and_create_json() error.");
            goto __finish;
        }
    }
    DBGPRINT(VDBG, "send_last_file() jsonSend[%s].", jsonSend);

    // 反馈结果
    tmpRet = xsk_send((char *)jsonSend, strlen(jsonSend), gglobalInfo.commSocketfd);
    if (tmpRet <= 0)
    {
        DBGPRINT(VERR, "send_last_file() xsk_send error.");
        goto __finish;
    }
    else
    {
        DBGPRINT(VDBG, "send_last_file() xsk_send ok[%s].", jsonSend);
    }
__remove:
    // 删除文件
    tmpRet = remove(tmpAbpath);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "send_last_file() remove() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "send_last_file() tmpAbpath[%s] already delete.", tmpAbpath);

__finish:

    xfree(jsonSend);

    return;
}

/*
des:
    检查目录下有没有遗留的文件，并将遗留文件汇报给鱼头
*/
void find_last_file()
{
    int ret = -1;
    int tmpRet = -1;
    char filePath[BUFFER_SIZE] = "";
    DIR *pdir = NULL;
    struct dirent *drent = NULL;

    sprintf(filePath, "%s/../../data/sendCache/%s", gglobalInfo.octoShare.headPath, MODE_NAME);
    DBGPRINT(VDBG, "find_last_file() filePath[%s].", filePath);

    if (!(pdir = opendir(filePath)))
    {
        DBGPRINT(VERR, "find_last_file() opendir filePath[%s] error.", filePath);
        goto __finish;
    }
    while ((drent = readdir(pdir)) != NULL)
    {
        struct stat st;
        char tmpAbpath[BUFFER_SIZE] = {0};
        // 把当前目录.，上一级目录..及隐藏文件都去掉，避免死循环遍历目录
        if ((!strncmp(drent->d_name, ".", sizeof("."))) || (!strncmp(drent->d_name, "..", sizeof(".."))))
        {
            continue;
        }
        sprintf(tmpAbpath, "%s/%s", filePath, drent->d_name);
        DBGPRINT(VDBG, "find_last_file() tmpAbpath[%s].", tmpAbpath);
        lstat(tmpAbpath, &st);
        // 判定是否为普通或执行文件
        if ((!S_ISDIR(st.st_mode)) && (!S_ISLNK(st.st_mode)) && (!S_ISFIFO(st.st_mode)) && (!S_ISSOCK(st.st_mode)) && (!S_ISCHR(st.st_mode)) && (!S_ISBLK(st.st_mode)))
        {
            send_last_file(tmpAbpath);
        }
    }

    ret = 0;
__finish:
    if (NULL != pdir)
    {
        closedir(pdir);
    }
    return;
}

/*
des:
    判断有无sendError路径，没有创建
return：
    成功：等于0
    失败：小于0
*/
int judge_send_cache()
{
    int ret = -1;
    int tmpRet = -1;
    char sendCachePath[BUFFER_SIZE] = "";

    sprintf(sendCachePath, "%s/../../data/sendCache/%s", gglobalInfo.octoShare.headPath, MODE_NAME);

    // 判断sendCachePath文件
    tmpRet = ensure_exist(sendCachePath);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "open_error_cache() mkdir sendCachePath error.");
        goto __finish;
    }

    ret = 0;
__finish:

    return ret;
}

/*
des:
    按行读取文件内容
*/
int read_line(int fd, int *pos, char *lineBuf)
{

    char tmp = 0;

    memset(lineBuf, 0, sizeof(lineBuf));
    while (1)
    {
        read(fd, &tmp, sizeof(char));
        if (tmp == '\n' || tmp == '\0')
            break;

        (*pos)++;

        strcat(lineBuf, &tmp);
    }
}
