#ifndef __H_OPERATE_FILE_H__
#define __H_OPERATE_FILE_H__
#include <stdio.h>
#include <stdlib.h>
#include "protocol_parse.h"

#define WARNNING_TYPE_IP 1
#define WARNNING_TYPE_DANGER_WORD 2
#define WARNNING_TYPE_SCREEN_CONTENT_CHANGED 3
#define WARNNING_TYPE_BMP 4

#define AUDIT_TYPE_IP 1
#define AUDIT_TYPE_DANGER_WORD 2
#define AUDIT_TYPE_DBMP 3

#pragma pack(push, 1)
// 记录用于保存审计、告警文件名的随机数
typedef struct _st_fileInfo
{
    int fp;
    int oldRand;
    int newRand;
} ST_fileInfo, *PST_fileInfo;
// 记录文件信息
typedef struct _st_reportFileInfo
{
    ST_fileInfo auditFileInfo;
    ST_fileInfo warnningFileInfo;
} ST_reportFileInfo, *PST_reportFileInfo;
#pragma pack(pop)

/*
des:
    关闭文件描述、打开新文件描述符、获取上一次文件内容、向鱼头汇报信息、删除上一次文件
*/
extern void send_audit_already_file();
/*
des:
    关闭文件描述、打开新文件描述符、获取上一次文件内容、向鱼头汇报信息、删除上一次文件
*/
extern void send_warnning_already_file();
/*
des:
    将解析到的数据包内容组成json对象写入审计文件
parm：
    auditBuf[IN]:待写入的审计消息
    fp[IN]:待写入文件的文件描述符
return：
    -1：失败
    0：成功
*/
int write_audit_file(char *auditBuf, int fp);
/*
des:
    将解析到的数据包内容组成json对象写入告警文件
parm：
    warningAndReviewBuf[IN]:待写入的告警信息
    fp[IN]:待写入文件的文件描述符
    dangerWord[IN]:触发的危险词
return：
    -1：失败
    0：成功
*/
int write_warnning_file(char *warningAndReviewBuf, int fp);

/*
des:
    将连接的IP组成json对象写入告警文件
parm：
    clientIp[IN]:客户端连接IP
    serverIp[IN]:服务器IP
return：
    -1：失败
    0：成功
*/
int write_ip_warnning_file(char *clientIp, char *serverIp, int fp);
/*
des:
    将解析到的数据包内容组成json对象写入告警文件
parm：
    bmpMd5[IN]:图片MD5
    fp[IN]:待写入文件的文件描述符
return：
    -1：失败
    0：成功
*/
int write_bmp_warnning_file(char *bmpName, char *bmpValue, char *bmpMd5, int fp);
/*
des:
    将连接的IP组成json对象写入审计文件
parm：
    clientIp[IN]:客户端连接IP
    serverIp[IN]:服务器IP
return：
    -1：失败
    0：成功
*/
int write_ip_audit_file(char *clientIp, char *serverIp, int fp);

/*
des:
    将解析到的数据包内容组成json对象写入审计文件
parm：
    auditBuf[IN]:待写入的审计消息
    fp[IN]:待写入文件的文件描述符
return：
    -1：失败
    0：成功
*/
int write_bmp_audit_file(char *bmpName, char *bmpValue, char *bmpMd5, int fp);

/*
des:
    将情报板的旧内容与新内容写入告警文件
parm：
    oldContent[IN]:情报板上一次显示内容
    newContent[IN]:情报板当前显示内容
    fp[IN]:告警文件的文件句柄
return：
    -1：失败
    0：成功
*/
int write_screen_content_warnning_file(char *oldContent, char *newContent, int fp);

/*
des:
    检查目录下有没有遗留的文件，并将遗留文件汇报给鱼头
*/
void find_last_file();
/*
des:
    判断有无sendError路径，没有创建
return：
    成功：等于0
    失败：小于0
*/
int judge_send_cache();

#endif