#include "packet.h"
#include "setting.h"
#include "common.h"
#include "lib.h"

extern ST_globalInfo gglobalInfo;

unsigned char gpacketProtocolFlag[3] = {0x10, 0x20, 0x30};
unsigned char gpacketProtocolVer[2] = {0x00, 0x01};
unsigned char gencodeType = 0x00;
unsigned char gencodesm4Type = 0x01;

/*
des:
   检查机器的大小端
return:
   小端:1
   大端:0
*/
int check_big_little()
{
    int a = 1;
    return *(char *)&a;
}

/*
des:
    与管理平台通信的tcp发送方法
param:
    socketFd[IN]:发送的套接字
    data[IN]:发送的数据内容
    datasize[IN]:发送的数据内容长度
return:
    成功:返回发送的字符串长度
    失败:小于0
*/
int psms_tcp_send(int socketFd, char *data, int datasize)
{
    int ret = -1;
    int len = -1;
    int bigLittle = 0;
    char *encJson = NULL;
    int encJsonLen = 0;
    ST_packetHead packetHead;
    PST_packetHead ppacketHead = &packetHead;

    memset(ppacketHead, 0, sizeof(ST_packetHead));

    // 选择不加密
    if (gglobalInfo.confInfo.encodeType == 0)
    {
        // 开始组包头
        bigLittle = check_big_little();
        // 如果为小端，长度需要转化为网络字节序传输
        if (bigLittle == 1)
        {
            int tmpLen = 0;
            tmpLen = htonl(datasize);
            memcpy(ppacketHead->clength, &tmpLen, sizeof(ppacketHead->clength));
        }
        else
        {
            memcpy(ppacketHead->clength, &datasize, sizeof(ppacketHead->clength));
        }
        memcpy(ppacketHead->protocolFlag, gpacketProtocolFlag, sizeof(ppacketHead->protocolFlag));
        ppacketHead->encodeType = gencodeType;

        // 发送包头
        len = snd(socketFd, (char *)ppacketHead, sizeof(ST_packetHead), 0);
        if (len != sizeof(ST_packetHead))
        {
            DBGPRINT(VERR, "psms_tcp_send() snd len[%d] != sizeof(ST_packetHead)[%d] error.", len, sizeof(ST_packetHead));
            goto __finish;
        }
        len = 0;
        len = snd(socketFd, data, datasize, 0);
        if (len != datasize)
        {
            DBGPRINT(VERR, "psms_tcp_send() snd len[%d] != datasize[%d] error.", len, datasize);
            goto __finish;
        }
    }
    // 选择sm4加密
    else if (gglobalInfo.confInfo.encodeType == 1)
    {
        unsigned char key[16] = {0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01};
        // 加密json
        get_sm4_and_base_enc_result(key, (unsigned char *)data, datasize, (unsigned char **)&encJson, &encJsonLen);
        // 开始组包头
        bigLittle = check_big_little();
        // 如果为小端，长度需要转化为网络字节序传输
        if (bigLittle == 1)
        {
            int tmpLen = 0;
            tmpLen = htonl(encJsonLen);
            memcpy(ppacketHead->clength, &tmpLen, sizeof(ppacketHead->clength));
        }
        else
        {
            memcpy(ppacketHead->clength, &encJsonLen, sizeof(ppacketHead->clength));
        }
        memcpy(ppacketHead->protocolFlag, gpacketProtocolFlag, sizeof(ppacketHead->protocolFlag));
        memcpy(ppacketHead->protocolVer, gpacketProtocolVer, sizeof(ppacketHead->protocolVer));
        ppacketHead->encodeType = gencodesm4Type;

        // 发送包头
        len = snd(socketFd, (char *)ppacketHead, sizeof(ST_packetHead), 0);
        if (len != sizeof(ST_packetHead))
        {
            DBGPRINT(VERR, "psms_tcp_send() snd len[%d] != sizeof(ST_packetHead)[%d] error.", len, sizeof(ST_packetHead));
            goto __finish;
        }
        len = 0;
        len = snd(socketFd, encJson, encJsonLen, 0);
        if (len != encJsonLen)
        {
            DBGPRINT(VERR, "psms_tcp_send() snd len[%d] != encJsonLen[%d] error.", len, encJsonLen);
            goto __finish;
        }
    }

    ret = len;
__finish:

    xfree(encJson);

    return ret;
}

/*
des:
    与管理平台通信的数据接收方法
param:
    socketFd[IN]:接收套接字
    data[IN,OUT]:传入空字符串指针地址，返回接收的数据内容
    datasize[IN,OUT]:传入int指针，返回实际接收的数据长度
return:
    成功:返回接收的字符串长度
    失败:小于0
*/
int psms_recv(int socketFd, char **data, int *datasize)
{
    int ret = -1;
    int len = -1;
    int bigLittle = 0;
    int nextRecvLen = -1;
    char *data_ = NULL;
    ST_packetHead packetHead;
    PST_packetHead ppacketHead = &packetHead;

    DBGPRINT(VDBG, "file:%s,line:%d...", __FILE__,__LINE__);
    memset(ppacketHead, 0, sizeof(ST_packetHead));
    // 接收包头
    len = rcv(socketFd, (char *)ppacketHead, sizeof(ST_packetHead), 0);
    if (len != sizeof(ST_packetHead))
    {
        DBGPRINT(VERR, "psms_recv() rcv len[%d] != sizeof(PACKET_HEAD)[%d] error.", len, sizeof(ST_packetHead));
        goto __finish;
    }
    // 解析包头
    // 协议头判断
    if (memcmp(ppacketHead->protocolFlag, gpacketProtocolFlag, sizeof(gpacketProtocolFlag)) != 0)
    {
        DBGPRINT(VERR, "psms_recv() ppacket_head->protocol_flag[%02x][%02x][%02x] != gpacket_protocol_flag error.", ppacketHead->protocolFlag[0], ppacketHead->protocolFlag[1], ppacketHead->protocolFlag[2]);
        goto __finish;
    }
    
#if 0    
    // 协议版本判断
    if (memcmp(ppacketHead->protocolVer, gpacketProtocolVer, sizeof(gpacketProtocolVer)) != 0)
    {
        DBGPRINT(VERR, "psms_recv() ppacket_head->protocol_ver[%02x][%02x] != gpacket_protocol_ver error.", ppacketHead->protocolVer[0], ppacketHead->protocolVer[1]);
        goto __finish;
    }
#endif

    bigLittle = check_big_little();
    // 如果为小端，长度需要转化为网络字节序传输
    if (bigLittle == 1)
    {
        int tmpLen = 0;
        memcpy(&tmpLen, ppacketHead->clength, sizeof(ppacketHead->clength));
        nextRecvLen = ntohl(tmpLen);
    }
    else
    {
        memcpy(&nextRecvLen, ppacketHead->clength, sizeof(ppacketHead->clength));
    }

    DBGPRINT(VDBG, "psms_recv() nextRecvLen[%d].", nextRecvLen);
    data_ = xmalloc(nextRecvLen);

    // 接收包内容
    len = rcv(socketFd, (char *)data_, nextRecvLen, 0);
    if (len != nextRecvLen)
    {
        DBGPRINT(VERR, "psms_recv() rcv datalen[%d] != nextRecvLen[%d] error.", len, nextRecvLen);
        xfree(data_);
        goto __finish;
    }

    // 无需解密
    if (ppacketHead->encodeType == gencodeType)
    {
        *data = data_;
        *datasize = len;
        ret = len;
    }
    // 采用sm4解密
    else if (ppacketHead->encodeType == gencodesm4Type)
    {
        unsigned char *jsonDec = NULL;
        int jsonDecLen = 0;
        unsigned char key[16] = {0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01};
        // 解密
        get_sm4_and_base_dec_result(key, (unsigned char *)data_, strlen(data_), &jsonDec, &jsonDecLen);
        xfree(data_);
        *data = jsonDec;
        *datasize = jsonDecLen;
        ret = jsonDecLen;
    }

__finish:
    return ret;
}
