#include "policy.h"
#include "common.h"
#include "work.h"
#include "setting.h"
#include "afx/xiconv.h"

extern ST_globalInfo gglobalInfo;

/*
des:
    设置libYuanHengTrans.conf
param:
    pusbConfInfo[IN,OUT]:存放usb中配置文件信息的结构体
return：
    成功：等于0
    失败：小于0

*/
int set_libYuanHengTrans_config(PST_globalInfo pusbConfInfo)
{
    int ret = -1;
    int tmpRet = -1;
    xlist *plist = NULL;
    char libYuanHengTransConfPath[BUFFER_SIZE] = {0};
    char heartbeat[BUFFER_SIZE] = "";
    char port[BUFFER_SIZE] = "";

    sprintf(libYuanHengTransConfPath, "%s/../../agent/config/libYuanHengTrans.so.conf", gglobalInfo.octoShare.headPath);
    // 判断文件是否存在 是否可读
    if (access(libYuanHengTransConfPath, F_OK) != 0 || access(libYuanHengTransConfPath, R_OK) != 0 || access(libYuanHengTransConfPath, W_OK) != 0)
    {
        DBGPRINT(VERR, "set_libYuanHengTrans_config() libYuanHengTransConfPath[%s] not exist or cant be read\n", libYuanHengTransConfPath);
        goto __finish;
    }
    // 链表初始化
    plist = xlist_init();
    if (!plist)
    {
        DBGPRINT(VERR, "set_libYuanHengTrans_config() xlist_init init error.");
        goto __finish;
    }
    // 加载配置文件
    load_config(plist, libYuanHengTransConfPath, NULL);
    // 设置发布平台ip
    tmpRet = set_config_value(plist, "libYuanHengTrans.so", "publishingPlatformIp", pusbConfInfo->increasePublishingInfo.publishingPlatformIp);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "set_libYuanHengTrans_config() set_config_value() publishingPlatformIp error.");
        goto __finish;
    }
    // 设置发布平台端口
    sprintf(port, "%d", pusbConfInfo->increasePublishingInfo.publishingPlatformPort);
    tmpRet = set_config_value(plist, "libYuanHengTrans.so", "publishingPlatformPort", port);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "set_libYuanHengTrans_config() set_config_value() publishingPlatformPort error.");
        goto __finish;
    }
    // 设置发布平台心跳频率
    sprintf(heartbeat, "%d", pusbConfInfo->increasePublishingInfo.publishingPlatformHeartbeat);
    tmpRet = set_config_value(plist, "libYuanHengTrans.so", "publishingPlatformHeartbeat", heartbeat);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "set_libYuanHengTrans_config() set_config_value() publishingPlatformHeartbeat error.");
        goto __finish;
    }
    // 写入配置文件
    tmpRet = write_config(plist, libYuanHengTransConfPath);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "set_libYuanHengTrans_config() write_config() error.");
        goto __finish;
    }
    ret = 0;
__finish:

    if (plist)
    {
        free_config(plist);
    }
    return ret;
}

/*
des:
    根据配置链表，生成配置文件
param：
    plist[IN]:存储配置信息的链表
    confPath[IN]:配置文件名
*/
int write_config(xlist *plist, char *confPath)
{
    int ret = -1;
    int tmpRet = -1;
    char tmpConfigContent[BIGBUFFER_SIZE] = "";
    FILE *fp = NULL;

    // 以覆盖方式打开文件
    fp = fopen("./tmp.conf", "w+");

    // 遍历第一层链表
    while (plist != NULL && plist->next != NULL)
    {
        xlist *tagList = NULL;
        xlist *contentList = NULL;
        char tagName[BUFFER_SIZE] = "";

        // 得到以标题为key的链表
        tagList = xlist_get(plist, plist->key);

        // 组标题
        sprintf(tagName, "[%s]\n", tagList->key);
        DBGPRINT(VDBG, "tagName[%s]\n", tagName);
        // 写入标题
        tmpRet = fwrite(tagName, 1, strlen(tagName), fp);
        if (tmpRet != strlen(tagName))
        {
            DBGPRINT(VERR, "write_config() fwrite() tagName error.\n");
            goto __finish;
        }

        // 获取改标题下内容
        contentList = (xlist *)(tagList->value);
        // 跳过标题
        contentList = contentList->next;

        // 遍历内容链表
        while (contentList != NULL && contentList->next != NULL)
        {
            char conten[BUFFER_SIZE] = "";

            // 组写入内容
            sprintf(conten, "%s=%s\n", contentList->key, contentList->value);
            DBGPRINT(VDBG, "content[%s]\n", conten);
            // 写入内容
            tmpRet = fwrite(conten, 1, strlen(conten), fp);
            if (tmpRet != strlen(conten))
            {
                DBGPRINT(VERR, "write_config() fwrite() content error.\n");
                goto __finish;
            }

            // 指向下一项
            contentList = contentList->next;
        }
        // 指向下一项
        plist = plist->next;
    }

    // 将文件流指针定位到文件开头
    fseek(fp, 0, SEEK_SET);

    // 读临时配置文件
    tmpRet = fread(tmpConfigContent, 1, sizeof(tmpConfigContent), fp);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "write_config() fread() content error.\n");
        goto __finish;
    }
    DBGPRINT(VDBG, "tmpConfigContent[%s]\n", tmpConfigContent);

    // 将覆盖到配置文件
    tmpRet = file_write(tmpConfigContent, strlen(tmpConfigContent), confPath, "w+");
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "write_config() fread() file_write() error.\n");
        goto __finish;
    }

    ret = 0;
__finish:

    // 关闭文件
    fclose(fp);

    // 删除临时配置文件
    remove("./tmp.conf");

    return ret;
}

/*
des:
    解析允许连接IP数组，添加至链表中
parm:
    content[IN]:josn中的content数组
*/
int parse_allow_ip_array(cJSON *content)
{
    int ret = -1;
    int tmpRet = -1;
    int i = 0;
    int allowIpArraySize = 0;
    cJSON *allowIpArray = NULL;

    // 清空链表
    xlist_clean(gglobalInfo.allowIpList);

    // 初始化
    gglobalInfo.allowIpList = xlist_init();
    if (gglobalInfo.allowIpList == NULL)
    {
        DBGPRINT(VERR, "parse_allow_ip_array() xlist_init() allowIpList error.");
        goto __finish;
    }

    // 解析allowIpArray
    allowIpArray = cJSON_GetObjectItem(content, "allowIp");
    if (allowIpArray == NULL)
    {
        DBGPRINT(VERR, "parse_allow_ip_array() allowIpArray == NULL.");
        goto __finish;
    }

    allowIpArraySize = cJSON_GetArraySize(allowIpArray);
    DBGPRINT(VDBG, "parse_allow_ip_array() policy allowIpArraySize[%d].", allowIpArraySize);
    if (allowIpArraySize < 0)
    {
        DBGPRINT(VERR, "parse_allow_ip_array() policy allowIpArraySize[%d] <= 0.", allowIpArraySize);
        goto __finish;
    }
    else if (allowIpArraySize == 0)
    {
        gglobalInfo.enterAllowIpflag = ALLOW_ALL_IP;
        DBGPRINT(VDBG, "parse_allow_ip_array() allowIpArray is ALLOW_ALL_IP.");
        ret = 0;
        goto __finish;
    }
    for (i = 0; i < allowIpArraySize; i++)
    {
        char *tmpbuf = NULL;
        cJSON *contentIndexN = NULL;

        contentIndexN = cJSON_GetArrayItem(allowIpArray, i);
        if (contentIndexN == NULL)
        {
            DBGPRINT(VERR, "parse_allow_ip_array() contentIndexN i[%d] == NULL.", i);
            continue;
        }
        if (contentIndexN->valuestring == NULL)
        {
            DBGPRINT(VERR, "parse_allow_ip_array() contentIndexN->valuestring==NULL.");
            continue;
        }

        tmpbuf = xmalloc(sizeof(int));

        // 插入链表
        xlist_add(gglobalInfo.allowIpList, contentIndexN->valuestring, XLIST_STRING, tmpbuf);
        gglobalInfo.enterAllowIpflag = ALLOW_SPECIAL_IP;
        DBGPRINT(VDBG, "parse_allow_ip_array() allowIpArray is ALLOW_SPECIAL_IP.");
        DBGPRINT(VDBG, "parse_allow_ip_array() xlist_add() ip[%s].", contentIndexN->valuestring);
    }
    ret = 0;
__finish:

    return ret;
}

/*
des:
    解析危险词数组，添加至链表中
parm:
    content[IN]:josn中的content数组
*/
int parse_danger_word_array(cJSON *content)
{
    int ret = -1;
    int tmpRet = -1;
    int i = 0;
    int dangerWordArraySize = 0;
    cJSON *dangerWordArray = NULL;

    // 清空链表
    xlist_clean(gglobalInfo.writeList);
    // 初始化
    gglobalInfo.writeList = xlist_init();
    if (gglobalInfo.writeList == NULL)
    {
        DBGPRINT(VERR, "parse_danger_word_array() xlist_init() writeList error.");
        goto __finish;
    }
    // 清空审批白名单
    xlist_clean(gglobalInfo.reviewWriteList);
    // 初始化
    gglobalInfo.reviewWriteList = xlist_init();
    if (gglobalInfo.reviewWriteList == NULL)
    {
        DBGPRINT(VERR, "parse_danger_word_array() xlist_init() reviewWriteList error.");
        goto __finish;
    }

    // 解析dangerWordArray
    dangerWordArray = cJSON_GetObjectItem(content, "dangerWord");
    if (dangerWordArray == NULL)
    {
        DBGPRINT(VERR, "parse_danger_word_array() dangerWordArray == NULL.");
        goto __finish;
    }

    dangerWordArraySize = cJSON_GetArraySize(dangerWordArray);
    DBGPRINT(VDBG, "parse_danger_word_array() policy dangerWordArraySize[%d].", dangerWordArraySize);
    if (dangerWordArraySize < 0)
    {
        DBGPRINT(VERR, "parse_danger_word_array() policy dangerWordArraySize[%d] <= 0.", dangerWordArraySize);
        goto __finish;
    }
    else if (dangerWordArraySize == 0)
    {
        DBGPRINT(VDBG, "parse_danger_word_array()  dangerWordArray is NULL.");
        ret = 0;
        goto __finish;
    }
    for (i = 0; i < dangerWordArraySize; i++)
    {
        char *tmpbuf = NULL;
        cJSON *contentIndexN = NULL;
        char gString[BUFFER_SIZE] = "";

        contentIndexN = cJSON_GetArrayItem(dangerWordArray, i);
        if (contentIndexN == NULL)
        {
            DBGPRINT(VERR, "parse_danger_word_array() contentIndexN i[%d] == NULL.", i);
            continue;
        }
        if (contentIndexN->valuestring == NULL)
        {
            DBGPRINT(VERR, "parse_danger_word_array() contentIndexN->valuestring==NULL.");
            continue;
        }
        if (strcmp(contentIndexN->valuestring, "") == 0)
        {
            DBGPRINT(VERR, "parse_danger_word_array() contentIndexN->valuestring== .");
            continue;
        }

        tmpbuf = xmalloc(sizeof(int));

        // 转码
        tmpRet = xu2g(contentIndexN->valuestring, strlen(contentIndexN->valuestring), gString, 4096);
        if (tmpRet < 0)
        {
            // 插入链表
            xlist_add(gglobalInfo.writeList, contentIndexN->valuestring, XLIST_STRING, tmpbuf);
            DBGPRINT(VDBG, "parse_allow_ip_array() xlist_add() [%s].", contentIndexN->valuestring);
        }
        else
        {
            // 插入链表
            xlist_add(gglobalInfo.writeList, gString, XLIST_STRING, tmpbuf);
            DBGPRINT(VDBG, "parse_allow_ip_array() xlist_add() dangerWord[%s].", gString);
        }
    }

    ret = 0;
__finish:

    return ret;
}

/*
des:
    解析图片白名单md5数组，添加至链表中
parm:
    content[IN]:josn中的content数组
*/
int parse_bmp_white_array(cJSON *content)
{
    int ret = -1;
    int tmpRet = -1;
    int i = 0;
    int whiteListBMPSize = 0;
    cJSON *whiteListBMP = NULL;

    // 清空链表
    xlist_clean(gglobalInfo.bmpWriteList);
    // 初始化
    gglobalInfo.bmpWriteList = xlist_init();
    if (gglobalInfo.bmpWriteList == NULL)
    {
        DBGPRINT(VERR, "parse_bmp_white_array() xlist_init() bmpWriteList error.");
        goto __finish;
    }
    // 清空审批白名单
    xlist_clean(gglobalInfo.bmpReviewWriteList);
    // 初始化
    gglobalInfo.bmpReviewWriteList = xlist_init();
    if (gglobalInfo.bmpReviewWriteList == NULL)
    {
        DBGPRINT(VERR, "parse_bmp_white_array() xlist_init() bmpReviewWriteList error.");
        goto __finish;
    }

    // 解析whiteListBMP
    whiteListBMP = cJSON_GetObjectItem(content, "whiteListBMP");
    if (whiteListBMP == NULL)
    {
        DBGPRINT(VERR, "parse_bmp_white_array() whiteListBMP == NULL.");
        goto __finish;
    }

    whiteListBMPSize = cJSON_GetArraySize(whiteListBMP);
    DBGPRINT(VDBG, "parse_bmp_white_array() policy whiteListBMPSize[%d].", whiteListBMPSize);
    if (whiteListBMPSize < 0)
    {
        DBGPRINT(VERR, "parse_bmp_white_array() policy whiteListBMPSize[%d] <= 0.", whiteListBMPSize);
        goto __finish;
    }
    else if (whiteListBMPSize == 0)
    {
        DBGPRINT(VDBG, "parse_bmp_white_array()  whiteListBMP is NULL.");
        ret = 0;
        goto __finish;
    }
    for (i = 0; i < whiteListBMPSize; i++)
    {
        char *tmpbuf = NULL;
        cJSON *contentIndexN = NULL;

        contentIndexN = cJSON_GetArrayItem(whiteListBMP, i);
        if (contentIndexN == NULL)
        {
            DBGPRINT(VERR, "parse_bmp_white_array() bmpWriteList i[%d] == NULL.", i);
            continue;
        }
        if (contentIndexN->valuestring == NULL)
        {
            DBGPRINT(VERR, "parse_bmp_white_array() contentIndexN->valuestring==NULL.");
            continue;
        }
        if (strcmp(contentIndexN->valuestring, "") == 0)
        {
            DBGPRINT(VERR, "parse_bmp_white_array() contentIndexN->valuestring== .");
            continue;
        }

        tmpbuf = xmalloc(sizeof(int));

        // 插入链表
        xlist_add(gglobalInfo.bmpWriteList, contentIndexN->valuestring, XLIST_STRING, tmpbuf);
        DBGPRINT(VDBG, "parse_bmp_white_array() xlist_add() bmpWriteList[%s].", contentIndexN->valuestring);
    }

    ret = 0;
__finish:

    return ret;
}

/*
des:
    还原默认设置
return:
    成功：等于0
    失败：小于0
*/
int restore_default_settings()
{
    int ret = -1;
    int tmpRet = -1;
    gglobalInfo.auditTime = gglobalInfo.confInfo.auditTime;
    gglobalInfo.warningTime = gglobalInfo.confInfo.warningTime;
    xlist_clean(gglobalInfo.writeList);
    xlist_clean(gglobalInfo.allowIpList);
    xlist_clean(gglobalInfo.bmpWriteList);

    gglobalInfo.writeList = xlist_init();
    if (gglobalInfo.writeList == NULL)
    {
        DBGPRINT(VERR, "restore_default_settings() xlist_init() writeList error");
        goto __finish;
    }

    gglobalInfo.allowIpList = xlist_init();
    if (gglobalInfo.allowIpList == NULL)
    {
        DBGPRINT(VERR, "restore_default_settings() xlist_init() allowIpList error");
        goto __finish;
    }

    gglobalInfo.bmpWriteList = xlist_init();
    if (gglobalInfo.bmpWriteList == NULL)
    {
        DBGPRINT(VERR, "restore_default_settings() xlist_init() bmpWriteList error");
        goto __finish;
    }

    ret = 0;
__finish:
    return ret;
}

/*
des:
   解析策略json
param:
   data[IN]:需要解析的策略内容
return:
   解析成功：0
   解析失败：-1
*/
static int parse_policy_json(char *data)
{
    int ret = -1;
    int tmpRet = -1;
    cJSON *root = NULL;
    cJSON *policyType = NULL;
    cJSON *policyModeName = NULL;
    cJSON *policyId = NULL;
    cJSON *policyCancle = NULL;
    cJSON *policyContent = NULL;
    cJSON *runAlways = NULL;
    cJSON *content = NULL;
    cJSON *auditTime = NULL;
    cJSON *warningTime = NULL;
    cJSON *comPareTime = NULL;
    /*------------------------------------增加---------------------------------------*/
    cJSON *publishingPlatformIp = NULL;
    char *publishingPlatformIpString = NULL;
    cJSON *publishingPlatformPort = NULL;
    cJSON *publishingPlatformHeartbeat = NULL;

    DBGPRINT(VDBG, "file:%s, line:%d.",__FILE__, __LINE__);
    
    root = cJSON_Parse(data);
    if (root == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() cJSON_Parse error.");
        goto __finish;
    }
    // 解析policyType
    policyType = cJSON_GetObjectItem(root, "policyType");
    if (policyType == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() policyType == NULL.");
        goto __finish;
    }
    // 解析policyModeName
    policyModeName = cJSON_GetObjectItem(root, "policyModeName");
    if (policyModeName == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() policyName == NULL.");
        goto __finish;
    }
    // 解析policyId
    policyId = cJSON_GetObjectItem(root, "policyId");
    if (policyId == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() policyId == NULL.");
        goto __finish;
    }
    // 解析policyCancle
    policyCancle = cJSON_GetObjectItem(root, "policyCancle");
    if (policyCancle == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() policyCancle == NULL.");
        goto __finish;
    }
    if (policyCancle->valueint == 1)
    {
        // 还原默认设置
        tmpRet = restore_default_settings();
        if (tmpRet < 0)
        {
            DBGPRINT(VDBG, "parse_policy_json() restore_default_settings() error.");
            goto __finish;
        }
        DBGPRINT(VDBG, "parse_policy_json() restore_default_settings() ok.");
        ret = 0;
        goto __finish;
    }
    // 解析policyContent
    policyContent = cJSON_GetObjectItem(root, "policyContent");
    if (policyContent == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() policyContent == NULL.");
        goto __finish;
    }
    // 解析runAlways
    runAlways = cJSON_GetObjectItem(policyContent, "runAlways");
    if (runAlways == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() runAlways == NULL.");
        goto __finish;
    }
    // 解析content
    content = cJSON_GetObjectItem(policyContent, "content");
    if (content == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() content == NULL.");
        goto __finish;
    }
    // 解析auditTime
    auditTime = cJSON_GetObjectItem(content, "auditTime");
    if (auditTime == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() auditTime == NULL.");
        goto __finish;
    }
    DBGPRINT(VDBG, "parse_policy_json() auditTime[%d].", auditTime->valueint);
    gglobalInfo.auditTime = auditTime->valueint;
    // 解析warningTime
    warningTime = cJSON_GetObjectItem(content, "warningTime");
    if (warningTime == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() warningTime == NULL.");
        goto __finish;
    }
    DBGPRINT(VDBG, "parse_policy_json() warningTime[%d].", warningTime->valueint);
    gglobalInfo.warningTime = warningTime->valueint;
    // 解析comPareTime
    comPareTime = cJSON_GetObjectItem(content, "comPareTime");
    if (comPareTime == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() comPareTime == NULL.");
        goto __finish;
    }
    DBGPRINT(VDBG, "parse_policy_json() comPareTime[%d].", comPareTime->valueint);
    gglobalInfo.comPareTime = comPareTime->valueint;
    // 解析dangerWord数组
    tmpRet = parse_danger_word_array(content);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "parse_policy_json() parse_danger_word_array error.");
        goto __finish;
    }
    // 解析图片数组
    tmpRet = parse_bmp_white_array(content);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "parse_policy_json() parse_bmp_white_array error.");
        goto __finish;
    }

    // 解析allowIp数组
    tmpRet = parse_allow_ip_array(content);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "parse_policy_json() parse_danger_word_array error.");
        goto __finish;
    }

    // 解析发布平台ip
    publishingPlatformIp = cJSON_GetObjectItem(content, "publishingPlatformIp");
    if (publishingPlatformIp == NULL || !cJSON_IsString(publishingPlatformIp))
    {
        DBGPRINT(VERR, "parse_policy_json() publishingPlatformIp == NULL or invalid.");
        goto __finish;
    }
    publishingPlatformIpString = cJSON_GetStringValue(publishingPlatformIp);
    if (publishingPlatformIpString == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() publishingPlatformIpString == NULL.");
        goto __finish;
    }
    strncpy(gglobalInfo.increasePublishingInfo.publishingPlatformIp, publishingPlatformIpString, sizeof(gglobalInfo.increasePublishingInfo.publishingPlatformIp) - 1);

    // 解析发布平台端口
    publishingPlatformPort = cJSON_GetObjectItem(content, "publishingPlatformPort");
    if (publishingPlatformPort == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() publishingPlatformPort == NULL.");
        goto __finish;
    }
    DBGPRINT(VDBG, "parse_policy_json() publishingPlatformPort[%d].", publishingPlatformPort->valueint);
    gglobalInfo.increasePublishingInfo.publishingPlatformPort = publishingPlatformPort->valueint;

    // 解析发布平台心跳频率
    publishingPlatformHeartbeat = cJSON_GetObjectItem(content, "publishingPlatformHeartbeat");
    if (publishingPlatformHeartbeat == NULL)
    {
        DBGPRINT(VERR, "parse_policy_json() publishingPlatformHeartbeat == NULL.");
        goto __finish;
    }
    DBGPRINT(VDBG, "parse_policy_json() publishingPlatformHeartbeat[%d].", publishingPlatformHeartbeat->valueint);
    gglobalInfo.increasePublishingInfo.publishingPlatformHeartbeat = publishingPlatformHeartbeat->valueint;

    // (新增三项)设置libYuanHengTrans.conf
    tmpRet = set_libYuanHengTrans_config(&gglobalInfo);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "set_libYuanHengTrans_config() set_libYuanHengTrans_config() error.");
        goto __finish;
    }
    ret = 0;
__finish:

    if (root)
    {
        cJSON_Delete(root);
    }

    return ret;
}

/*
des:
   策略处理
param:
   socketFd[IN]:与鱼头通信的套接字
   data[IN]:待解析的json数据
*/
static void policy_deal(int socketFd, char *data)
{
    int tmpRet = -1;
    DBGPRINT(VDBG, "file:%s, line:%d.",__FILE__, __LINE__);

    // 上锁
    pthread_mutex_lock(&(gglobalInfo.threadMutex));

    // 解析json
    tmpRet = parse_policy_json(data);

    // 解锁
    pthread_mutex_unlock(&(gglobalInfo.threadMutex));

    // 同步操作在此处编码
    //  work_func();
__finish:

    return;
}

/*
des:
    接受鱼头数据的阻塞方法，包含策略等信息
param:
    socketFd[IN]:与鱼头通信的套接字
return:
    成功：0
    失败：小于0
*/
int recv_head_policy(int socketFd)
{
    int ret = -1;
    int gaptime = 3;

    DBGPRINT(VDBG, "recv_head_policy() start.");
    while (1)
    {
        int tmpRet = -1;
        int maxfd = 0;
        fd_set rset;
        struct timeval sm_tv;

        sm_tv.tv_sec = gaptime;
        sm_tv.tv_usec = 0;

        FD_ZERO(&rset);
        FD_SET(socketFd, &rset);
        maxfd = socketFd + 1;
        
        // DBGPRINT(VDBG, "file:%s, line:%d, socketFd:%d, maxfd:%d.", __FILE__, __LINE__, socketFd, maxfd);

        
        tmpRet = select(maxfd, &rset, NULL, NULL, &sm_tv);
        if (tmpRet < 0)
        {
            DBGPRINT(VERR, "recv_head_policy() select error.");
            break;
        }
        else if (tmpRet == 0)
        {
            // DBGPRINT(VDBG, "recv_head_policy() continue[%d].", getpid());
            continue;
        }
        else
        {
            if (FD_ISSET(socketFd, &rset))
            {
                int len = 0;
                char *recv_str = NULL;
                len = xsk_recv(&recv_str, &len, socketFd);
                if (len <= 0)
                {
                    DBGPRINT(VERR, "recv_head_policy() xsk_recv error, len:%d", len);
                    break;
                }
                DBGPRINT(VDBG, "file:%s, line:%d.",__FILE__, __LINE__);
                policy_deal(socketFd, recv_str);
                xfree(recv_str);
            }
        }
    }

__finish:
    DBGPRINT(VDBG, "file:%s, line:%d, recv_head_policy() end.", __FILE__, __LINE__);
    xsk_close(&socketFd);
    return ret;
}
