#include <sys/types.h>
#include <sys/wait.h>
#include <stdio.h>
#include <stdlib.h>

#include "protocol_parse.h"
#include "afx/xsocket.h"
#include "setting.h"
#include "common.h"
#include "operate_file.h"
#include "review.h"
#include <string.h>
#include "packet.h"
#include "yuanheng.h"
#include <time.h>

extern ST_reportFileInfo reportFileInfo;
extern ST_globalInfo gglobalInfo;
extern char cacheFileNameOld[BUFFER_SIZE];
extern char cacheFileNameNew[BUFFER_SIZE];

// 数据包起始位
unsigned char packetHead = {0x02};
// 数据包指令码
unsigned char packetOrder[] = {0x31, 0x30};
// 数据包标志
unsigned char packetFlag = {0x2b};
// 数据包偏移
unsigned char packetDataLen[] = {0x00, 0x00, 0x00, 0x00};
// 数据内容
unsigned char *packetData = "[list]\r\nitem_no=1\r\nitem0=900,1,0,\\fs6464\\c000255000000\\b000000000000\\C108028\\S00--\r\n";
// 数据包结束位
unsigned char packetEnd = {0x03};

/*
des:
    根据正则规则，匹配字符串，res存储返回结果
return:
    成功：等于0
    失败：小于0
 */
static int rule_parse_string(char *regex_parten, char *text, char *res, int type)
{
    int ret = RUN_FAILURE;
    int tmpRet = -1;
    char *pos = NULL;
    regmatch_t pmatch[1];
    const size_t nmatch = 1;
    regex_t reg = {0};

    /*
     *	编译正则模式
     */
    tmpRet = regcomp(&reg, regex_parten, REG_EXTENDED);
    if (RUN_SUCCESS == tmpRet)
    {
        /*
         *	执行正则表达式和缓存的比较
         */
        tmpRet = regexec(&reg, text, nmatch, pmatch, 0);
        if (REG_NOMATCH != tmpRet)
        {
            /*
             *	将匹配到的字符串存储到buffer中
             */
            if (type)
            {
                strncpy(res, text + pmatch[0].rm_so, strlen(text) - (pmatch[0].rm_so));
            }
            else
            {
                strncpy(res, text + pmatch[0].rm_so, (pmatch[0].rm_eo - pmatch[0].rm_so));
            }
        }
        else
        {
            DBGPRINT(VERR, "rule_parse_string regexec regex is NULL.");
            goto __finish;
        }
    }
    else
    {
        DBGPRINT(VERR, "rule_parse_string regcomp failed.");
        goto __finish;
    }

    ret = 0;
__finish:

    regfree(&reg);

    return ret;
}

/*
des:
    转发服务器发向客户端的消息
parm:
    clientSockfd[IN]:客户端连接
    serverSockfd[IN]:服务器连接
return:
    0：发送完毕
    -1：发送失败
*/
int server_to_client(int clientSockfd, int serverSockfd)
{
    int ret = -1;
    int tmpRet = -1;
    char recv_str[MINBUFFER_SIZE] = {0};
    int recvLen = 0;
    int len1 = 0;
    char zeroBuf[1] = "";
__next:
    // 接收消息
    recvLen = recv(serverSockfd, recv_str, sizeof(recv_str), 0);
    if (recvLen <= 0)
    {
        // 没有收到数据, 但是连接是正常的, 继续接收数据
        if (recvLen < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
        {
            goto __next;
        }
        DBGPRINT(VERR, "file:%s, line:%d, error:%d, server->client recv error.", __FILE__, __LINE__, errno);
        goto __finish;
    }
    
    {
        DBGPRINT(VDBG,"serverSockfd:%d, recvLen:%d.\r\n", serverSockfd, recvLen); 
        int i;
        for (i = 0; i < recvLen; i++)
        {
            DBGPRINT(VDBG, "%02x#", recv_str[i]);
        }
        DBGPRINT(VDBG, "\r\n"); 
    }

    // 记录文件
    hexdump(recv_str, recvLen, "/root/yuanHeng.log", "server");

    // 处理和分析数据包
    tmpRet = parse_server_packet(recv_str, recvLen);
    if (tmpRet < 0)
    {
        DBGPRINT(VDBG, "client_to_server() parse_client_packet() this packet not parse.");
        goto __finish;
    }

    send(serverSockfd, zeroBuf, 1, 0);
    // 发送消息
    len1 = send(clientSockfd, recv_str, 144, 0);
    if (len1 < 0)
    {
        DBGPRINT(VERR, "server_to_client() send() error len1[%d].", len1);
        goto __finish;
    }
    DBGPRINT(VDBG, "server_to_client() send() OK len1[%d].", len1);

    ret = 0;
__finish:

    return ret;
}

/*
des:
    检查发布信息是否符合白名单规则
param:
    textContent[IN]:发布信息
return：
    0:合规
    -1:不合规
*/
int check_write_list_rule(char *textContent)
{
    int ret = -1;
    int tmpRet = -1;
    xlist *tmpwriteList = NULL;
    xlist *tmpReviewWriteList = NULL;

    tmpwriteList = gglobalInfo.writeList;
    tmpReviewWriteList = gglobalInfo.reviewWriteList;

    // 有白名单规则才去匹配
    if (xlist_size(gglobalInfo.writeList) == 0)
    {
        ret = 0;
        DBGPRINT(VDBG, "check_write_list_rule() writeList is NULL.");
        goto __finish;
    }

    // 对比白名单链表，排查发布信息
    while (tmpwriteList && tmpwriteList->next)
    {
        char result[BUFFER_SIZE] = "";

        // 正则匹配
        tmpRet = rule_parse_string(tmpwriteList->key, textContent, result, 0);
        if (tmpRet == 0)
        {
            // 符合白名单
            DBGPRINT(VDBG, "check_write_list_rule() textContent [%s] is writeList.", textContent);
            break;
        }

        tmpwriteList = tmpwriteList->next;
    }

    // 策略白名单没匹配到
    if (tmpwriteList->next == NULL)
    {
        DBGPRINT(VERR, "check_write_list_rule() textContent[%s] is not writeList", textContent);
        char *tmpbuf = NULL;

        tmpbuf = xmalloc(sizeof(int));
        if (tmpbuf == NULL)
        {
            DBGPRINT(VERR, "check_write_list_rule() xmalloc() tmpBuf error.");
            goto __finish;
        }

        // 将不合规项添加到临时审批白名单
        xlist_add(gglobalInfo.tmpReviewWirteList, textContent, XLIST_STRING, tmpbuf);
        DBGPRINT(VDBG, "check_write_list_rule() xlist_add() gglobalInfo.tmpReviewWirteList[%s].", textContent);

        // xlist_add(gglobalInfo.reviewWriteList, textContent, XLIST_STRING, tmpbuf);
        // DBGPRINT(VDBG, "check_write_list_rule() xlist_add() reviewWriteList[%s].", textContent);

        // 对比审批白名单链表，排查发布信息
        while (tmpReviewWriteList && tmpReviewWriteList->next)
        {

            if (strcmp(textContent, tmpReviewWriteList->key) == 0)
            {
                // 符合审批白名单
                DBGPRINT(VDBG, "check_write_list_rule() textContent [%s] is reviewWriteList.", textContent);
                break;
            }

            tmpReviewWriteList = tmpReviewWriteList->next;
        }
        // 审批白名单没匹配到
        if (tmpReviewWriteList->next == NULL)
        {
            DBGPRINT(VERR, "check_write_list_rule() textContent[%s] is not reviewWriteList", textContent);
            goto __finish;
        }
    }

    ret = 0;
__finish:

    return ret;
}

/*
des:
    开始按字节接收数据包，接收到结束标志0x03为止
parm：
    clientSockfd[IN]:客户端连接
    recvBuf[IN,OUT]:存放数据包信息
    retLen[IN,OUT]:数据包长度
return：
    成功：0
    失败：-1
*/
int end_recv(int clientSockfd, char *recvBuf, int *retLen)
{
    int ret = -1;
    int tmpRet = -1;
    int flag = 0;

    while (1)
    {
        unsigned char date[2] = "";
    __next:
        // 按字节接收
        tmpRet = recv(clientSockfd, date, sizeof(unsigned char), flag);
        if (tmpRet <= 0)
        {
            if (tmpRet < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
            {
                goto __next;
            }
            DBGPRINT(VERR, "end_recv() recv() date error.");
            break;
        }

        // 当数据包长度超过4096时，返回错误
        if ((*retLen) + sizeof(unsigned char) > BUFFER_SIZE + 128)
        {
            DBGPRINT(VDBG, "end_recv() recv() date more than BUFFER_SIZE.");
            goto __finish;
        }

        // recvBuf存有开始标志
        memcpy(recvBuf + (*retLen), date, sizeof(unsigned char));
        (*retLen)++;

        // 若接收的字节为0x0d，再次接收一个字节接收完毕
        if (date[0] == END_FLAG)
        {
            // 按字节接收
            tmpRet = recv(clientSockfd, date, sizeof(unsigned char), flag);
            if (tmpRet <= 0)
            {
                if (tmpRet < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
                {
                    goto __next;
                }
                DBGPRINT(VERR, "end_recv() recv() date error.");
                break;
            }

            // recvBuf存有开始标志
            memcpy(recvBuf + (*retLen), date, sizeof(unsigned char));
            (*retLen)++;

            DBGPRINT(VDBG, "end_recv() recv() date already.");
            break;
        }
    }

    ret = 0;
__finish:
    return ret;
}

/*
des:
    开始接收数据包第一个字节，判断是否为开始标志0x02
parm：
    clientSockfd[IN]:客户端连接
    recvBuf[IN,OUT]:存放数据包信息
return：
    成功：等于0
    失败：小于0
*/
int start_recv(int clientSockfd, unsigned char *recvBuf)
{
    int ret = -1;
    int tmpRet = -1;
    int flag = 0;
    unsigned char date[2] = "";

__next:
    // 接收第一个字节
    tmpRet = recv(clientSockfd, date, sizeof(unsigned char), flag);
    if (tmpRet <= 0)
    {
        if (tmpRet < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
        {
            goto __next;
        }
        DBGPRINT(VERR, "start_recv() recv() date error.");
        goto __finish;
    }

    // 若接收的第一个字节不为0x59，停止接收关闭连接
    if (date[0] != START_FLAG)
    {
        DBGPRINT(VDBG, "recv_ctos_packet() first recv() date not START_FLAG.");
        goto __finish;
    }

    // 将包头标志拷贝至recvBuf中
    memcpy(recvBuf, date, sizeof(unsigned char));

    ret = 0;
__finish:
    return ret;
}

/*
des:
    接收数据包
    1、接收包头
    2、按字节接收数据
parm：
    clientSockfd[IN]:客户端连接
    recvBuf[IN,OUT]:保存完整数据包内容的地址
return：
    -1：失败
    >0:成功、接收的数据包长度
*/
int recv_ctos_packet(int clientSockfd, unsigned char *recvBuf)
{
    int ret = -1;
    int tmpRet = -1;
    int retLen = 0;

    // 开始接收
    tmpRet = start_recv(clientSockfd, recvBuf);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "recv_ctos_packet() start_recv() error.");
        goto __finish;
    }
    retLen++;

    // 判断结束
    tmpRet = end_recv(clientSockfd, recvBuf, &retLen);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "recv_ctos_packet() end_recv() error.");
        goto __finish;
    }

    DBGPRINT(VDBG, "recv_ctos_packet() recv() recvBuf [%s].", recvBuf);
__finish:

    return retLen;
}

/*
des:
    处理数据包审计、告警及审批操作
param:
    buf[IN]:数据内容
    auditBuf[IN]:审计信息数组
    warningBuf[IN]:告警信息数组
    reviewBuf[IN]:审批信息数组
return：
    0:数据包通过
    -1:阻断数据包

*/
int deal_audit_warning_revirw(char *auditBuf, char *warningBuf, char *reviewBuf)
{
    int ret = -1;
    int tmpRet = -1;

    if (strlen(warningBuf) != 0)
    {
        DBGPRINT(VDBG, "deal_audit_warning_revirw() tihs packet is wanring.");
        // 记入告警文件
        tmpRet = write_warnning_file(warningBuf, reportFileInfo.warnningFileInfo.fp);
        if (tmpRet < 0)
        {
            DBGPRINT(VERR, "deal_audit_warning_revirw() write_warnning_file() error.");
            goto __finish;
        }
    }

    if (strlen(reviewBuf) != 0)
    {
        DBGPRINT(VDBG, "deal_audit_warning_revirw() tihs packet need review.");

        xlist *tmpList = NULL;
        tmpList = gglobalInfo.tmpReviewWirteList;

        while (tmpList && tmpList->next)
        {
            DBGPRINT(VDBG, "deal_audit_warning_revirw() key[%s].", tmpList->key);
            tmpList = tmpList->next;
        }

        // 审批请求
        tmpRet = review_request(reviewBuf, "LIST");
        if (tmpRet < 0)
        {
            DBGPRINT(VERR, "deal_audit_warning_revirw() review_request() error.");
            goto __finish;
        }
        goto __finish;
    }

    // 发布内容为空
    if (strcmp(auditBuf, "") == 0)
    {
        DBGPRINT(VERR, "deal_audit_warning_revirw()  messages is NULL.");
        ret = 0;
        goto __finish;
    }

    // 记入审计文件
    tmpRet = write_audit_file(auditBuf, reportFileInfo.auditFileInfo.fp);
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "deal_audit_warning_revirw() write_audit_file() error.");
        goto __finish;
    }

    ret = 0;

__finish:
    return ret;
}

/*
des:
    处理文字信息
param:
    root[IN]:协议解析后的json结果
return：
    0:数据包通过
    小于零：阻断数据包
*/
int deal_text_json(cJSON *root)
{
    int ret = -1;
    int tmpRet = -1;
    cJSON *value = NULL;
    int valueSize = 0;
    int i = 0;
    char auditBuf[BIGBUFFER_SIZE] = "";
    char warningBuf[BIGBUFFER_SIZE] = "";
    char reviewBuf[BIGBUFFER_SIZE] = "";

    // 清空临时审批白名单
    xlist_clean(gglobalInfo.tmpReviewWirteList);

    // 初始化
    gglobalInfo.tmpReviewWirteList = xlist_init();
    if (gglobalInfo.tmpReviewWirteList == NULL)
    {
        DBGPRINT(VERR, "deal_text_json() xlist_init() tmpReviewWirteList() error.");
        goto __finish;
    }

    // 解析value
    value = cJSON_GetObjectItem(root, "value");
    if (value == NULL)
    {
        DBGPRINT(VERR, "deal_text_json() value == NULL.");
        goto __finish;
    }

    valueSize = cJSON_GetArraySize(value);
    DBGPRINT(VDBG, "deal_text_json() root valueSize[%d].", valueSize);
    if (valueSize < 0)
    {
        DBGPRINT(VERR, "deal_text_json() root valueSize[%d] <= 0.", valueSize);
        goto __finish;
    }
    else if (valueSize == 0)
    {
        DBGPRINT(VDBG, "deal_text_json()  value is NULL.");
        ret = 0;
        goto __finish;
    }
    for (i = 0; i < valueSize; i++)
    {
        cJSON *valueIndexN = NULL;
        cJSON *color = NULL;
        cJSON *text = NULL;
        char colorValue[MINBUFFER_SIZE] = "";
        char textValue[MINBUFFER_SIZE] = "";
        char tmpWarningBuf[BUFFER_SIZE] = "";
        char tmpReviewBuf[BUFFER_SIZE] = "";
        char tmpAuditBuf[BUFFER_SIZE] = "";

        valueIndexN = cJSON_GetArrayItem(value, i);
        if (valueIndexN == NULL)
        {
            DBGPRINT(VERR, "deal_text_json() valueIndexN i[%d] == NULL.", i);
            continue;
        }

        // 解析color
        color = cJSON_GetObjectItem(valueIndexN, "color");
        if (color == NULL)
        {
            DBGPRINT(VERR, "deal_text_json() color == NULL.");
            continue;
        }
        if (color->valuestring != NULL)
        {
            if (strcmp(color->valuestring, "") != 0)
            {
                strcpy(colorValue, color->valuestring);
            }
        }

        // 解析text
        text = cJSON_GetObjectItem(valueIndexN, "text");
        if (text == NULL)
        {
            DBGPRINT(VERR, "deal_text_json() text == NULL.");
            continue;
        }
        if (text->valuestring == NULL)
        {
            DBGPRINT(VERR, "deal_text_json() text->valuestring == NULL.");
            continue;
        }
        strcpy(textValue, text->valuestring);

        // 白名单检查
        tmpRet = check_write_list_rule(text->valuestring);
        if (tmpRet < 0)
        {
            // 组告警内容
            sprintf(tmpWarningBuf, "%s/%s", textValue, colorValue);
            strcat(warningBuf, tmpWarningBuf);
            strcat(warningBuf, ",");
            // 组审批内容
            sprintf(tmpReviewBuf, "%s/%s", textValue, colorValue);
            strcat(reviewBuf, tmpReviewBuf);
            strcat(reviewBuf, ",");
        }
        else
        {
            // 组审计内容
            sprintf(tmpAuditBuf, "%s/%s", textValue, colorValue);
            strcat(auditBuf, tmpAuditBuf);
            strcat(auditBuf, ",");
        }
    }

    // 处理审计、告警、审批
    tmpRet = deal_audit_warning_revirw(auditBuf, warningBuf, reviewBuf);
    if (tmpRet < 0)
    {
        DBGPRINT(VDBG, "deal_text_json() deal_audit_warning_revirw() is warning and review.");
        goto __finish;
    }

    ret = 0;
__finish:
    return ret;
}

/*
des:
    检查发布图片信息是否符合白名单规则
param:
    bmpMd5[IN]:发布图片信息
return：
    0:合规
    -1:不合规
*/
int check_write_bmp_rule(char *bmpMd5)
{
    int ret = -1;
    int tmpRet = -1;
    xlist *tmpBmpWriteList = NULL;
    xlist *tmpBmpReviewWriteList = NULL;

    tmpBmpWriteList = gglobalInfo.bmpWriteList;
    tmpBmpReviewWriteList = gglobalInfo.bmpReviewWriteList;

#if 0
    // 有白名单规则才去匹配
    if (xlist_size(gglobalInfo.bmpWriteList) == 0)
    {
        DBGPRINT(VDBG, "file:%s, line:%d, bmpWriteList is NULL.", __FILE__, __LINE__);
        goto __finish;
    }
#endif

    // 对比白名单链表，排查发布信息
    while (tmpBmpWriteList && tmpBmpWriteList->next)
    {

        if (strcmp(bmpMd5, tmpBmpWriteList->key) == 0)
        {
            // 符合白名单
            DBGPRINT(VDBG, "file:%s, line:%d, bmp:%s, hit mpWriteList.", __FILE__, __LINE__, bmpMd5);
            break;
        }

        tmpBmpWriteList = tmpBmpWriteList->next;
    }

    // 策略白名单没匹配到
    if (tmpBmpWriteList->next == NULL)
    {
        DBGPRINT(VERR, "check_write_bmp_rule() textContent[%s] is not writeList", bmpMd5);
        char *tmpbuf = NULL;

        tmpbuf = xmalloc(sizeof(int));
        if (tmpbuf == NULL)
        {
            DBGPRINT(VERR, "check_write_bmp_rule() xmalloc() tmpBuf error.");
            goto __finish;
        }

        // 将不合规项添加到临时审批白名单
        xlist_add(gglobalInfo.bmpTmpReviewWirteList, bmpMd5, XLIST_STRING, tmpbuf);
        DBGPRINT(VDBG, "file:%s, line:%d, bmp:%s, added to bmpTmpReviewWirteList.", __FILE__, __LINE__, bmpMd5);

        // 对比审批白名单链表，排查发布信息
        while (tmpBmpReviewWriteList && tmpBmpReviewWriteList->next)
        {

            if (strcmp(bmpMd5, tmpBmpReviewWriteList->key) == 0)
            {
                // 符合审批白名单
                DBGPRINT(VERR, "file:%s, line:%d, bmp:%s,  in bmpReviewWriteList.", __FILE__, __LINE__, bmpMd5);
                break;
            }

            tmpBmpReviewWriteList = tmpBmpReviewWriteList->next;
        }
        // 审批白名单没匹配到
        if (tmpBmpReviewWriteList->next == NULL)
        {
            DBGPRINT(VERR, "file:%s, line:%d, bmp:%s,  not in bmpReviewWriteList.", __FILE__, __LINE__, bmpMd5);
            goto __finish;
        }
    }

    ret = 0;
__finish:

    return ret;
}


void test_parse_review_content(char *recv)
{
    int ret = -1;
    int tmpRet = -1;
    cJSON *root = NULL;
    cJSON *pass = NULL;

    DBGPRINT(VDBG, "parse_review_content() recv[%s].", recv);
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);

    // 解析
    root = cJSON_Parse(recv);
    
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);
    if (root == NULL)
    {
        DBGPRINT(VERR, "parse_review_content() cJSON_Parse error.");
        goto __finish;
    }
    // 解析contentArrya
    pass = cJSON_GetObjectItem(root, "pass");
    if (pass == NULL)
    {
        DBGPRINT(VERR, "parse_review_content() pass == NULL.");
        goto __finish;
    }
    if (pass->valueint == 1)
    {
        DBGPRINT(VDBG, "parse_review_content() pass is allowed.");

    }
    else if (pass->valueint == 0)
    {
        DBGPRINT(VERR, "parse_review_content() pass is reject.");
        goto __finish;
    }
    else if (pass->valueint == 2)
    {
        DBGPRINT(VERR, "parse_review_content() pass is timeout.");
        goto __finish;
    }

__finish:
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);

    if (root)
    {
        cJSON_Delete(root);
    }

    return;
}

void parse_review_content(char *recv);

void* test_review_recv_and_parse(void* param)
{
    int ret = -1;
    int tmpRet = -1;
    int tmpLen = 0;
    char *recv = NULL;
    int recvLen = 0;
    int gaptime = 10;
    int timeOut = 180;
    int serverSockfd = -1;

    
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);
    if (param == NULL)
    {
        DBGPRINT(VERR, "review_recv_and_parse() param == NULL.");
        goto __finish;
    }
    memcpy(&serverSockfd, param, sizeof(int));

    DBGPRINT(VDBG, "file:%s,line:%d, serverSockfd:%d.", __FILE__,__LINE__, serverSockfd);

    // 超时判断, 运维平台人工审批， 超时时间是3分钟
    int canRead = xsk_can_read(serverSockfd, timeOut, 0);
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__, __LINE__);
    
    if (!canRead)
    {
        DBGPRINT(VERR, "review_request() xsk_can_read[%d] timeout[3min] error.", canRead);
        goto __finish;
    }

    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);

    // 接收数据
    tmpLen = psms_recv(serverSockfd, &recv, &recvLen);
    if (tmpLen <= 0)
    {
        DBGPRINT(VERR, "review_request() psms_recv error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "review_request() recv policy len[%d][%s].", tmpLen, recv);
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);

    // 上锁
    pthread_mutex_lock(&(gglobalInfo.threadMutex));
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);

    // 解析审批消息
    parse_review_content(recv);
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);

    // 解锁
    pthread_mutex_unlock(&(gglobalInfo.threadMutex));

__finish:

    DBGPRINT(VDBG, "review_recv_and_parse() serverSockfd[%d] ended.", serverSockfd);

    xsk_close(&serverSockfd);
    xfree(recv);
    pthread_exit(NULL);

    return NULL;
}

char *create_review_bmp_json(char *bmpName, char *bmpContent, char *bmpMd5, char *reviewType);
void *review_recv_and_parse(void *param);

int test_review_bmp_request(char *bmpName, char *content, char *bmpMd5, char *reviewType)
{
    int ret = -1;
    int tmpRet = -1;
    int jsonBufLen = 0;
    char *jsonBuf = NULL;
    int socketFd = -1;
    int sendLen = 0;
    pthread_t tid;
    void *tmpParam;

    // 组json
    jsonBuf = create_review_bmp_json(bmpName, content, bmpMd5, reviewType);
    if (jsonBuf == NULL)
    {
        DBGPRINT(VERR, "review_bmp_request() create_review_json() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "file:%s,line:%d, jsonBuf:%s.", __FILE__,__LINE__, jsonBuf);

    // 连接管理平台
    socketFd = xsk_init_client(gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, 10, NULL);
    if (socketFd < 0)
    {
        DBGPRINT(VERR, "file:%s, line:%d, xsk_init_client error.", __FILE__,__LINE__);
        goto __finish;
    }
    DBGPRINT(VDBG, "file:%s,line:%d, ip:%s, port:%d, socketFd:%d.", __FILE__,__LINE__, gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, socketFd);

    /*将图片审批请求发送到运维平台*/
    sendLen = psms_tcp_send(socketFd, jsonBuf, strlen(jsonBuf));
    if (sendLen < 0)
    {
        DBGPRINT(VERR, "review_bmp_request() psms_tcp_send sendLen[%d].", sendLen);
        goto __finish;
    }

    DBGPRINT(VDBG, "file:%s,line:%d, sendLen:%d.", __FILE__,__LINE__, sendLen);

    // 审批接收线程
    // 线程传参，在堆区，线程内释放
    tmpParam = xmalloc(sizeof(int));
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);
    
    memcpy(tmpParam, &socketFd, sizeof(int));
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);
    
    // 开启线程
    tmpRet = pthread_create(&tid, NULL, (void *)review_recv_and_parse, (void *)tmpParam);
    DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);
    
    if (tmpRet != 0)
    {
        DBGPRINT(VDBG, "file:%s,line:%d.", __FILE__,__LINE__);
        DBGPRINT(VERR, "review_bmp_request() error ret[%d]", ret);
        xsk_close(&socketFd);
        DBGPRINT(VDBG, "review_bmp_request() server contnet already close.");
        xfree(tmpParam);
        goto __finish;
    }
    // 设置线程分离
    pthread_detach(tid);

    ret = 0;
__finish:

    xfree(jsonBuf);

    return ret;
}
uint32_t read_file(char *p_file,uint8_t *p_cache,uint32_t cache_size,uint32_t *p_length);

/*情报板最大512*1024*/
#define MAX_BMP_BUFFER (3*512*1024+100)
int review_test(char *test_bmp_file)
{
    
    char *bmp_cache = NULL; 
    char *base64Bmp = NULL;
    int fileSize = 0;
    char fileMd5[MINBUFFER_SIZE] = "";
    int tmpRet = -1;
    int ret = -1;

    bmp_cache = xmalloc(MAX_BMP_BUFFER);
    base64Bmp = xmalloc(MAX_BMP_BUFFER);

    /*读取待审批的bmp文件*/
    read_file(test_bmp_file, bmp_cache, MAX_BMP_BUFFER, &fileSize);

    /*计算bmp文件的md5值*/
    GetMD5OfFile(test_bmp_file, fileMd5, 0);
    DBGPRINT(VDBG, "file:%s,line:%d, filesize:%d,md5:%s.", __FILE__,__LINE__,fileSize,fileMd5);

    /*将bmp文件的缓冲区进行base64编码*/
    EncodeBase64(bmp_cache, fileSize, base64Bmp);

    // 匹配图片白名单
    tmpRet = check_write_bmp_rule(fileMd5);
    if (tmpRet < 0)
    {
        // 记入告警和审批
        //write_bmp_warnning_file(test_bmp_file, base64Bmp, fileMd5, reportFileInfo.warnningFileInfo.fp);
        
        // 审批
        review_bmp_request(test_bmp_file, base64Bmp, fileMd5, "BMP");
        goto __finish;
    }
    else
    {
        // 记入审计
        //write_bmp_audit_file(cacheFileNameOld, base64Bmp, fileMd5, reportFileInfo.auditFileInfo.fp);
    }
    
    ret = 0;
__finish:
    xfree(bmp_cache);
    xfree(base64Bmp);
    return ret;
}

/*
des:
    处理图片信息
param:
    root[IN]:协议解析后的json结果
return：
    0:数据包通过
    小于零：阻断数据包
*/
int deal_bmp_json()
{
    int ret = -1;
    int tmpRet = -1;
    int fileSzie = 0;
    char *fileContent = NULL;
    char fileMd5[MINBUFFER_SIZE] = "";
    char *base64Bmp = NULL;

    // 清空临时审批白名单
    xlist_clean(gglobalInfo.bmpTmpReviewWirteList);

    // 初始化
    gglobalInfo.bmpTmpReviewWirteList = xlist_init();
    if (gglobalInfo.bmpTmpReviewWirteList == NULL)
    {
        DBGPRINT(VERR, "deal_bmp_json() xlist_init() bmpTmpReviewWirteList() error.");
        goto __finish;
    }

    // 获取图片内容
    file_content(&fileContent, &fileSzie, cacheFileNameNew);
    DBGPRINT(VDBG, "deal_bmp_json() fileContent[%s].", fileContent);
    DBGPRINT(VDBG, "deal_bmp_json() fileSzie[%d].", fileSzie);
    DBGPRINT(VDBG, "deal_bmp_json() cacheFileNameNew[%s].", cacheFileNameNew);

    // 获取图片md5
    GetMD5OfFile(cacheFileNameNew, fileMd5, 0);
    DBGPRINT(VDBG, "deal_bmp_json() fileMd5[%s].", fileMd5);

    base64Bmp = xmalloc(fileSzie * 3);
    if (base64Bmp == NULL)
    {
        DBGPRINT(VERR, "deal_bmp_json() xmalloc() base64Bmp error.");
        goto __finish;
    }
    EncodeBase64(fileContent, fileSzie, base64Bmp);
    DBGPRINT(VDBG, "deal_bmp_json() fileMd5[%s].", base64Bmp);

    // 匹配图片白名单
    tmpRet = check_write_bmp_rule(fileMd5);
    if (tmpRet < 0)
    {
        // 记入告警和审批
        write_bmp_warnning_file(cacheFileNameOld, base64Bmp, fileMd5, reportFileInfo.warnningFileInfo.fp);
        // 审批
        review_bmp_request(cacheFileNameOld, base64Bmp, fileMd5, "BMP");
        goto __finish;
    }
    else
    {
        // 记入审计
        write_bmp_audit_file(cacheFileNameOld, base64Bmp, fileMd5, reportFileInfo.auditFileInfo.fp);
        DBGPRINT(VDBG, "--------------------1");
    }

    ret = 0;
__finish:

    // 删除文件
    DBGPRINT(VDBG, "delete  [%s]", cacheFileNameNew);
    // remove(cacheFileNameNew);
    xfree(fileContent);
    xfree(base64Bmp);
    DBGPRINT(VDBG, "--------------------2");
    return ret;
}

/*
des:
    处理协议解析后的结果
param:
    result[IN]:协议解析后的json结果
return：
    0:数据包通过
    小于零：阻断数据包
*/
int deal_result()
{
    int ret = -1;
    int tmpRet = -1;

    tmpRet = deal_bmp_json();
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "deal_result() deal_text_json() this BMP packet be blocked");
        goto __finish;
    }

    ret = 0;
__finish:
    DBGPRINT(VDBG, "--------------------3");
    return ret;
}

/*
des:
    把数据包写入到文件中
*/
int write_packet_to_file(unsigned char *buf, int recvLen, int createFileFlag)
{
    int ret = -1;
    int tmpRet = -1;
    int packetSize = 0;
    int fp = 0;
    char fileName[MINBUFFER_SIZE] = "/opt/yuanhengPacket.txt";

    // 覆盖方式打开文件
    if (createFileFlag == 1)
    {
        fp = open(fileName, O_CREAT | O_TRUNC | O_RDWR, 0666);
    }
    // 追加方式打开文件
    else
    {
        fp = open(fileName, O_APPEND | O_RDWR, 0666);
    }

    // 写数据包长度
    tmpRet = write(fp, &recvLen, sizeof(recvLen));
    if (tmpRet != sizeof(recvLen))
    {
        DBGPRINT(VERR, "write_packet_to_file() write() packetSize error.");
        goto __finish;
    }

    // 写数据包
    tmpRet = write(fp, buf, recvLen);
    if (tmpRet != recvLen)
    {
        DBGPRINT(VERR, "write_packet_to_file() write() packet error.");
        goto __finish;
    }

    ret = 0;
__finish:
    return ret;
}

int grep_upload_packet_end(unsigned char *buf)
{
    int ret = -1;

    char total = 0;
    char now = 0;

    // 拷贝第九个字节
    memcpy(&total, buf + 8, 1);

    // 拷贝第十个字节
    memcpy(&now, buf + 9, 1);

    // 判断是否为最后一个包
    if (total == now + 1)
    {
        // 最后一个包
        DBGPRINT(VDBG, "grep_upload_packet_end() this packet is end.");
        goto __finish;
    }

    ret = 0;
__finish:
    return ret;
}

/*
des:
    模拟回复客户端
*/
void reply_client(int clientSockfd, unsigned char *buf)
{
    int ret = -1;
    int flag = 0;
    int sendLen = 0;
    unsigned char sendBuf[] = {0x01, 0x01, 0x00, 0x01, 0x00, 0x0d, 0x73};

    // 判断是否为最后一个包
    ret = grep_upload_packet_end(buf);
    if (ret < 0)
    {
        DBGPRINT(VDBG, "reply_client() this packet do not reply.");

        goto __finish;
    }

    // 发送数据包
    sendLen = snd(clientSockfd, sendBuf, sizeof(sendBuf), flag);
    if (sendLen != sizeof(sendBuf))
    {
        DBGPRINT(VERR, "reply_client() snd() error sendLen[%d]!=sizeof(sendBuf)[%d].", sendLen, sizeof(sendBuf));
        goto __finish;
    }
    DBGPRINT(VDBG, "reply_client() send() OK sendLen[%d]=sizeof(sendBuf)[%d].", sendLen, sizeof(sendBuf));

__finish:
    return;
}

/*
des:
    处理文件上传，先保存模拟服务器回复保存数据包，保存完毕解析数据包，检查白名单，发向服务器
*/
int deal_file_upload(int clientSockfd, int serverSockfd, unsigned char *buf, int recvLen)
{
    int ret = -1;
    int tmpRet = -1;
    int createFileFlag = 0;
    unsigned char *packet;

    // 判断是否需要创建文件
    grep_need_to_create_file(buf, recvLen, &createFileFlag);

    // 保存数据包写文件
    write_packet_to_file(buf, recvLen, createFileFlag);

    // 模拟回复
    reply_client(clientSockfd, buf);

    // 解析数据包
    tmpRet = parse_bmp(buf, recvLen, createFileFlag);
    if (tmpRet < 0)
    {
        goto __finish;
    }

    ret = 0;

__finish:
    return ret;
}

/*
des:
    读取文件内容发向服务端
*/
int send_upload_packet_to_server(int clientSockfd)
{
    int ret = -1;
    int tmpRet = -1;
    int packetSize = 0;
    int flag = 0;
    int sendLen = 0;
    unsigned char sendBuf[] = {0x01, 0x01, 0x00, 0x01, 0x00, 0x0d, 0x73};
    unsigned char *packet = NULL;
    FILE *fp = NULL;
    char fileName[MINBUFFER_SIZE] = "/opt/yuanhengPacket.txt";

    // 读文件发送
    // 只读方式打开文件
    fp = file_open(fileName, "rb");
    if (fp == NULL)
    {
        DBGPRINT(VERR, "send_upload_packet_to_server() file_open() error.");
        goto __finish;
    }

    // 读文件
    while (fread(&packetSize, 1, 4, fp) == 4)
    {
        int newTableFlag = 0;
        int serverSockfd = -1;
        int sendLen = 0;
        char recv[7] = "";
        ST_packetHead packetHead;
        PST_packetHead pPacketHead = &packetHead;

        memset(pPacketHead, 0, sizeof(ST_packetHead));

        // 申请堆区空间
        packet = xmalloc(packetSize);
        if (packet == NULL)
        {
            DBGPRINT(VERR, "send_upload_packet_to_server() xmalloc() packet error.");
            goto __finish;
        }

        fread(packet, 1, packetSize, fp);
        DBGPRINT(VDBG, "send_upload_packet_to_server() fread packet[%s].", packet);

        // 创建tcp客户端
        serverSockfd = xsk_init_client(gglobalInfo.contectInfo.serverIp, gglobalInfo.contectInfo.serverPort, 10, gglobalInfo.contectInfo.clientIp);
        // serverSockfd = xsk_init_client(gglobalInfo.confInfo.serverIp, gglobalInfo.confInfo.serverPort, 10, NULL);
        if (serverSockfd < 0)
        {
            DBGPRINT(VERR, "send_upload_packet_to_server() xsk_init_client error.");
            goto __finish;
        }
        // 发送数据包
        sendLen = snd(serverSockfd, packet, packetSize, flag);
        if (sendLen != packetSize)
        {
            DBGPRINT(VERR, "send_upload_packet_to_server() snd() error sendLen[%d]!=packetSize[%d].", sendLen, packetSize);
            goto __finish;
        }
        DBGPRINT(VDBG, "send_upload_packet_to_server() send() OK sendLen[%d]=packetSize[%d].", sendLen, packetSize);

        rcv(serverSockfd, recv, 7, flag);

        xfree(packet);
        if (serverSockfd >= 0)
        {
            close(serverSockfd);
        }
    }

    // 发送数据包
    sendLen = snd(clientSockfd, sendBuf, sizeof(sendBuf), flag);
    if (sendLen != sizeof(sendBuf))
    {
        DBGPRINT(VERR, "send_upload_packet_to_server() snd() error sendLen[%d]!=sizeof(sendBuf)[%d].", sendLen, sizeof(sendBuf));
        goto __finish;
    }
    DBGPRINT(VDBG, "send_upload_packet_to_server() send() OK sendLen[%d]=sizeof(sendBuf)[%d].", sendLen, sizeof(sendBuf));

    ret = 0;

__finish:
    return ret;
}

/*
des:
    转发客户端向服务器发送的消息
parm：
    clientSockfd[IN]:客户端连接
    serverSockfd[IN]:服务器连接
return:
    成功：等于0
    失败：小于0
*/
int client_to_server(int clientSockfd, int serverSockfd)
{
    int ret = -1;
    int tmpRet = -1;
    int flag = 0;
    int recvLen = 0;
    int sendLen = 0;
    char buf[BUFFER_SIZE] = "";
    DBGPRINT(VDBG, "client_to_server() clientSockfd[%d]", clientSockfd);

// 接收数据包
__next:
    recvLen = recv(clientSockfd, buf, sizeof(buf), flag);
    DBGPRINT(VDBG, "client_to_server() recvLen[%d]", recvLen);
    if (recvLen <= 0)
    {
        // 没有收到数据, 但是连接是正常的, 继续接收数据
        if (recvLen < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
        {
            goto __next;
        }
        DBGPRINT(VERR, "file:%s, line:%d, recvLen:%d, errno:%d, client->server recv error.", __FILE__, __LINE__, recvLen, errno);
        goto __finish;
    }
    DBGPRINT(VDBG, "file:%s, line:%d, client->server recv OK recvLen: %d.", __FILE__,__LINE__,recvLen);

    // 记录文件
    hexdump(buf, recvLen, "/root/yuanHeng.log", "client");

    // 解析数据包
    tmpRet = parse_client_packet(buf, recvLen);
    if (tmpRet < 0)
    {
        DBGPRINT(VDBG, "file:%s, line:%d, parsing client->server packet error.", __FILE__,__LINE__);
        goto __finish;
    }

    // 发送数据包
    sendLen = send(serverSockfd, buf, 11, flag);
    if (sendLen <= 0)
    {
        DBGPRINT(VERR, "parse_client_to_server() snd() error sendLen[%d].", sendLen);
        goto __finish;
    }
    DBGPRINT(VDBG, "parse_client_to_server() send() OK sendLen[%d].", sendLen);

    ret = 0;
__finish:
    return ret;
}

/*
元亨317协议, ECC计算方法是以0x7f为基值，异或buf中全部数据, 
最后再异或0x0d, 得到最终ECC值。
buf:要计算ECC的内存起始地址, 为方便，buf最后应该留出2个字节空间,
存放0x0d和ECC.
参数:
buf: 要计算ecc的缓冲区
len: 缓冲区长度,长度值应该是需要计算ECC数据的长度再加2
*/
char calcEccValue(char *buf, int len)
{
    int i = 0;
    if (len < 3)
    {
        return 0;
    }
    
    char ecc = 0x7f;
    
    for (i = 0; i < len - 2; i++)
    {
        ecc ^= buf[i];
    }

    ecc ^= 0x0d;
    buf[len - 2] = 0x0d;
    buf[len - 1] = ecc;
    return ecc;
}


/*
元亨317协议，通过分析抓包：
播放图片指令需要三个报文,
第一个报文下发待播放的图片地址和停留时间,
第二个报文查询播放列表，
第三个报文设置亮度.
*/

/*
本函数构造播放图片指令的第一个报文,下发待播放的图片地址和停留时间

5948, 固定不变
0101, 固定不变, ID0和ID1
0180, 播放指令
0016, 长度, 从0016后面01开始(不包含0016)，到倒数第二个字节0d(不含0d)之间的数据
前6个字节固定含义, 之后每个页面占用16个字节
0100,固定不变
0500,固定不变
0001,播放页面总个数
…16字节描述一个播放页面,
02,播放页面地址
00,固定不变
0000,固定不变
5000,固定不变
05,停留秒数
00,固定不变
0001,固定不变
0050,固定不变
0000,固定不变
0000,固定不变
…每多一个播放页面, 前面总长度加0x10
0d, 固定不变
f7, 是ECC值，以0x7F为初始值，异或前面的全部数据得到

参数:
szBuf: 构造出的报文缓冲区,
bufLen: 报文缓冲区，按预定，预留两个字节存放0x0d和ECC.


返回值:
0成功, 非0失败
*/

int buildPlayInstructionPacket(char *szBuf, int bufLen,  cJSON *playerArrayJson)
{
    int iTemp = 0;
    int iIndex = 0;
    uint8_t playerSize;
    
    int infoAddr;
    int showTime;
    
    playerSize = (uint8_t)cJSON_GetArraySize(playerArrayJson);
    DBGPRINT(VDBG,"file:%s, line:%d,playerSize:%d.\r\n", __FILE__,__LINE__, playerSize); 
    
    szBuf[0]  = 0x59;
    szBuf[1]  = 0x48;
    szBuf[2]  = 0x01;
    szBuf[3]  = 0x01;
    szBuf[4]  = 0x01;
    szBuf[5]  = 0x80;
    szBuf[6]  = ((uint16_t)(0x6+playerSize*0x10))>>8;
    szBuf[7]  = ((uint16_t)(0x6+playerSize*0x10)) & 0xff;
    szBuf[8]  = 0x01;
    szBuf[9]  = 0x00;
    szBuf[10] = 0x05;
    szBuf[11] = 0x00;
    szBuf[12] = 0x00;
    szBuf[13] = playerSize;

    for (iIndex = 0; iIndex < playerSize; iIndex++)
    {
        cJSON *contentIndexN = NULL;

        contentIndexN = cJSON_GetArrayItem(playerArrayJson, iIndex);
        if (contentIndexN == NULL)
        {
            DBGPRINT(VERR, "file:%s, line:%d, parse_allow_ip_array() contentIndexN i[%d] == NULL.", __FILE__, __LINE__, iIndex);
            continue;
        }
        
        infoAddr = cJSON_GetObjectItem(contentIndexN, "infoAddress")->valueint;
        showTime = cJSON_GetObjectItem(contentIndexN, "showTime")->valueint;
        DBGPRINT(VERR, "file:%s, line:%d, index:%d, infoAddr:%d, showTime:%d.\r\n", __FILE__, __LINE__, iIndex, infoAddr, showTime);

        /*
            02,播放页面地址
            00,固定不变
            0000,固定不变
            5000,固定不变
            05,停留秒数
            00,固定不变
            0001,固定不变
            0050,固定不变
            0000,固定不变
            0000,固定不变
        */

        szBuf[14+iIndex*16] = infoAddr;//播放页面地址
 
        szBuf[15+iIndex*16] = 0x00;
        szBuf[16+iIndex*16] = 0x00;
        szBuf[17+iIndex*16] = 0x00;
        
        szBuf[18+iIndex*16] = 0x50;
        szBuf[19+iIndex*16] = 0x00;

        szBuf[20+iIndex*16] = showTime;//页面停留时间
        
        szBuf[21+iIndex*16] = 0x00;
        
        szBuf[22+iIndex*16] = 0x00;
        szBuf[23+iIndex*16] = 0x01;
        szBuf[24+iIndex*16] = 0x00;
        szBuf[25+iIndex*16] = 0x50;
        
        szBuf[26+iIndex*16] = 0x00;
        szBuf[27+iIndex*16] = 0x00;
        szBuf[28+iIndex*16] = 0x00;
        szBuf[29+iIndex*16] = 0x00;        
        
    }
    
    DBGPRINT(VDBG,"file:%s, line:%d.\r\n", __FILE__,__LINE__); 

    calcEccValue(szBuf, bufLen); 
    DBGPRINT(VDBG,"file:%s, line:%d.\r\n", __FILE__,__LINE__); 

    #if 0
    {
        int i = 0;
        DBGPRINT(VDBG,"file:%s, line:%d, begin:\r\n", __FILE__,__LINE__); 
        for (i = 0; i < bufLen; i++)
        {
            DBGPRINT(VDBG,"%02x", szBuf[i]); 
        }
        DBGPRINT(VDBG,"file:%s, line:%d, end.\r\n", __FILE__,__LINE__);
    }
    #endif
    
    return 0;    
}

/*
本函数构造播放图片指令的第二个报文,查询播放列表

本函数构造报文,查询播放列表
5948 0101 0189 0001 00 0dea

5948, 固定不变
0101, 固定不变, ID0和ID1
0189, 查询播放列表
0001, 数据长度
00, 播放列表类型, 00 表示正常播放列表, ff表示空闲播放列表
0d, 固定不变
ea, 是ECC值，以0x7F为初始值，异或前面的全部数据得到

参数:
szBuf: 构造出的数据报文, 长度应该是11字节.

返回值:
0成功, 非0失败
*/
int buildQueryListPacket(char *szBuf, int bufLen)
{
    int i = 0;
    
    szBuf[0]  = 0x59;
    szBuf[1]  = 0x48;
    szBuf[2]  = 0x01;
    szBuf[3]  = 0x01;
    szBuf[4]  = 0x01;
    szBuf[5]  = 0x89;
    szBuf[6]  = 0x00;
    szBuf[7]  = 0x01;
    szBuf[8]  = 0x00;
    szBuf[9]  = 0x0d;
    szBuf[10] = 0xea;
    
    #if 0
    {
        int i = 0;
        DBGPRINT(VDBG,"file:%s, line:%d, begin:\r\n", __FILE__,__LINE__); 
        for (i = 0; i < bufLen; i++)
        {
            DBGPRINT(VDBG,"%02x", szBuf[i]); 
        }
        DBGPRINT(VDBG,"file:%s, line:%d, end.\r\n", __FILE__,__LINE__);
    }
    #endif  
    return 0;    
}



/*
本函数构造播放图片指令的第三个报文, 设置情报板亮度
5948 0101 0105 0002 0002 0d67
5948, 固定不变
0101, 固定不变, ID0和ID1
0105, 设置亮度指令
0002, 指令长度
00, 亮度模式, 00是手动模式，01是自动模式. 当前固定为手动模式.
02, 亮度等级, 客户端传过来是1-8, 按照100的8等分进行转换.
0d, 固定不变
67, 是ECC值，以0x7F为初始值，异或前面的全部数据得到

参数:
szBuf: 构造出的数据报文, 长度应该是14字节.
bufLen: buf长度, 约定预留2字节给0x0d和ECC.
lightGrade: 亮度等级, 1-8.


返回值:
0成功, 非0失败
*/
int buildLightgradePacket(char *szBuf, uint32_t bufLen, uint32_t lightGrade)
{
    int i = 0;
    
    if (lightGrade > 8)
    {
        return 1;
    }

    szBuf[0] = 0x59;
    szBuf[1] = 0x48;
    szBuf[2] = 0x01;
    szBuf[3] = 0x01;
    szBuf[4] = 0x01;
    szBuf[5] = 0x05;
    szBuf[6] = 0x00;
    szBuf[7] = 0x02;
    szBuf[8] = 0x00;
    szBuf[9] = (uint8_t)(12.5*lightGrade);
    calcEccValue(szBuf,bufLen);

    #if 0
    {
        int i = 0;
        DBGPRINT(VDBG,"file:%s, line:%d, begin:\r\n", __FILE__,__LINE__); 
        for (i = 0; i < bufLen; i++)
        {
            DBGPRINT(VDBG,"%02x", szBuf[i]); 
        }
        DBGPRINT(VDBG,"file:%s, line:%d, end.\r\n", __FILE__,__LINE__);
    }
    #endif    
        
}
void buildSendtoClientMsg(char* buf, char* deviceID, char* orderID, int device_protocol, int result, char* cmd);

static char playInstructionAck[] = {0x01, 0x01, 0x00, 0x01, 0x00, 0x0d, 0x73};
static char queryListAck[]       = {0x01,0x01, 0x00, 0x02, 0xcc, 0x40, 0x0d, 0xfc};
static char setLightAck[]        = {0x01, 0x01, 0x00, 0x01, 0x01, 0x0d, 0x72};

int processPlayOrder(int clientSockfd, char *deviceID, char * orderID, int device_protocol, cJSON *json)
{
    int infoCount;
    int ret = -1;
    int playerArraySize = 0;
    cJSON *playerArrayJson = NULL;
    int playerSize;
    int bufLen = 0;
    int lightGrade;
    cJSON *contentIndexN;
    int sendLen;
    int serverSockfd = -1;
    int recvLen = 0;
    char recvBuf[MINBUFFER_SIZE] = {0};
    

    /*播放指令报文内容长度不超过512，加上其他字符，tcp data不超过600*/
    char szBuf[600] = {0};
    
    infoCount = cJSON_GetObjectItem(json, "infoCount")->valueint;
    DBGPRINT(VDBG,"file:%s, line:%d,deviceID:%s,orderID:%s,device_protocol:%d, infoCount:%d.\r\n", __FILE__,__LINE__, deviceID, orderID, device_protocol, infoCount); 
    
    // 解析播放图片数组
    playerArrayJson = cJSON_GetObjectItem(json, "player");

    /*取第一个图片的亮度设置*/
    contentIndexN = cJSON_GetArrayItem(playerArrayJson, 0);
    if (contentIndexN == NULL)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_GetArrayItem(0) == NULL.", __FILE__, __LINE__);
        return ret;
    }
    lightGrade = cJSON_GetObjectItem(contentIndexN, "lightGrade")->valueint;
    DBGPRINT(VDBG,"file:%s, line:%d,lightGrade:%d.\r\n", __FILE__,__LINE__, lightGrade); 

    // 创建连接情报板控制器的socket
    serverSockfd = xsk_init_client(gglobalInfo.increasePublishingInfo.controllerIp, gglobalInfo.increasePublishingInfo.controllerPort, 10, NULL);
    if (serverSockfd < 0)
    {
        DBGPRINT(VERR, "file:%s, line:%d, xsk_init_client error.\r\n", __FILE__, __LINE__);
        return ret;
    }

    /*发送三条指令，并接收情报板应答*/
    bufLen = 14 + 16*infoCount + 2;
    buildPlayInstructionPacket(szBuf,bufLen,playerArrayJson);
    sendLen = send(serverSockfd, szBuf, bufLen, 0);
    if (sendLen != bufLen)
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. len:%d, errno:%d.\r\n", __FILE__, __LINE__, sendLen, errno);
        goto __finish;
    }
    recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);
    hexdump(recvBuf, recvLen, "/root/yuanHeng.log", "server");

#if 0
    {
        int i = 0;
        DBGPRINT(VDBG,"file:%s, line:%d, begin:", __FILE__,__LINE__);
        for (i = 0; i < recvLen; i++)
        {
            DBGPRINT(VDBG,"%02x",recvBuf[i]); 
        }
        DBGPRINT(VDBG,"file:%s, line:%d, end.", __FILE__,__LINE__);
    }
#endif
    
    bufLen = 11;
    buildQueryListPacket(szBuf,bufLen);
    sendLen = send(serverSockfd, szBuf, bufLen, 0);
    if (sendLen != bufLen)
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%, errno:%d.\r\n", __FILE__, __LINE__, sendLen, errno);
        #if 0
        xsk_close(&serverSockfd);
        // 创建连接情报板控制器的socket
        serverSockfd = xsk_init_client(gglobalInfo.increasePublishingInfo.controllerIp, gglobalInfo.increasePublishingInfo.controllerPort, 10, NULL);
        if (serverSockfd < 0)
        {
            DBGPRINT(VERR, "file:%s, line:%d, xsk_init_client error.\r\n", __FILE__, __LINE__);
            goto __finish ;
        }
        sendLen = send(serverSockfd, szBuf, bufLen, 0);
        #endif
    }
    recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);
    hexdump(recvBuf, recvLen, "/root/yuanHeng.log", "server");
    #if 0
    if (!((recvLen > 0) && (memcmp(recvBuf, queryListAck, sizeof(queryListAck)) == 0)))
    {
        buildSendtoClientMsg(szBuf, deviceID, orderID, device_protocol, 0, "执行失败");
        goto __finish;
    }
    #endif    
#if 0
{
    int i = 0;
    DBGPRINT(VDBG,"file:%s, line:%d, begin:", __FILE__,__LINE__);
    for (i = 0; i < recvLen; i++)
    {
        DBGPRINT(VDBG,"%02x",recvBuf[i]); 
    }
    DBGPRINT(VDBG,"file:%s, line:%d, end.", __FILE__,__LINE__);
}
#endif
    
    bufLen = 12;
    buildLightgradePacket(szBuf,bufLen,lightGrade);    
    sendLen = snd(serverSockfd, szBuf, bufLen, 0);    
    if (sendLen != bufLen)
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%d, errno:%d.\r\n", __FILE__, __LINE__, sendLen, errno);
        
        goto __finish;
    }
    recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);
    hexdump(recvBuf, recvLen, "/root/yuanHeng.log", "server");
#if 0
    {
        int i = 0;
        DBGPRINT(VDBG,"file:%s, line:%d, begin:", __FILE__,__LINE__);
        for (i = 0; i < recvLen; i++)
        {
            DBGPRINT(VDBG,"%02x",recvBuf[i]); 
        }
        DBGPRINT(VDBG,"file:%s, line:%d, end.", __FILE__,__LINE__);
    }
#endif

    buildSendtoClientMsg(szBuf, deviceID, orderID, device_protocol, 1, "播放成功");
    DBGPRINT(VDBG, "file:%s, line:%d,buf_len:%d,  buf:%s.", __FILE__, __LINE__, strlen(szBuf), szBuf);
    // 回复指令处理结果给发布消息客户端
    sendLen = send(clientSockfd, szBuf, strlen(szBuf), 0);
    if (sendLen < 0 )
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%d, errno:%d.", __FILE__, __LINE__, sendLen, errno);
        goto __finish;
    }   
    ret = 0;
    
__finish:
    xsk_close(&serverSockfd);
    return ret; 
}

int processOffscreenOrder(int clientSockfd, char *deviceID, char * orderID, int device_protocol)
{
    int serverSockfd = -1;
    int sendLen = 0;
    int ret = -1;
    char buf[BUFFER_SIZE] = {0};
    char zeroBuf[] = "";
    int tmpRet = -1;
    char recvBuf[MINBUFFER_SIZE] = {0};
    int recvLen = 0;

    DBGPRINT(VDBG,"file:%s, line:%d,deviceID:%s,orderID:%s,device_protocol:%d.\r\n", __FILE__,__LINE__, deviceID, orderID, device_protocol); 

    // 开屏
    unsigned char offBufCmd[] = {0x59, 0x48, 0x01, 0x01, 0x01, 0xff, 0x00, 0x01, 0x00, 0x0d, 0x9C};    
    unsigned char offBufAck[] = {0x01, 0x01, 0x00, 0x01, 0x00, 0x0D, 0x73};

    // 创建连接情报板控制器的socket
    serverSockfd = xsk_init_client(gglobalInfo.increasePublishingInfo.controllerIp, gglobalInfo.increasePublishingInfo.controllerPort, 10, NULL);
    if (serverSockfd < 0)
    {
        DBGPRINT(VERR, "file:%s, line:%d, xsk_init_client error.\r\n", __FILE__, __LINE__);
        return ret;
    }

    /*给情报板下发关屏命令*/
    sendLen = send(serverSockfd, offBufCmd, sizeof(offBufCmd), 0);
    if (sendLen != sizeof(offBufCmd))
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. len:%.\r\n", __FILE__, __LINE__, sendLen);
        
        goto __finish;
    }

__next:
    // 接收情报板回复
    recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);
    if (recvLen <= 0)
    {
        // 没有收到数据, 但是连接是正常的, 继续接收数据
        if (recvLen < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
        {
            goto __next;
        }
        DBGPRINT(VERR, "file:%s, line:%d, error:%d, server->client recv error.", __FILE__, __LINE__, errno);
        xsk_close(&serverSockfd);
        return ret;   
    }
        
    // 记录文件
    hexdump(recvBuf, recvLen, "/root/yuanHeng.log", "server");

    if (memcmp(recvBuf, offBufAck, sizeof(offBufAck)) == 0)
    {
        buildSendtoClientMsg(buf, deviceID, orderID, device_protocol, 1, "关屏成功");
    }
    else
    {
        buildSendtoClientMsg(buf, deviceID, orderID, device_protocol, 0, "执行失败");
    }

    DBGPRINT(VDBG, "file:%s, line:%d,buf_len:%d,  buf:%s.", __FILE__, __LINE__, strlen(buf), buf);
    // 回复指令处理结果给发布消息客户端
    sendLen = send(clientSockfd, buf, strlen(buf), 0);
    if (sendLen < 0 )
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%d.", __FILE__, __LINE__, sendLen);
        goto __finish;
    }

    ret = 0;

__finish:
    xsk_close(&serverSockfd);
    return ret;   
}

int processOnscreenOrder(int clientSockfd, char * deviceID, char * orderID, int device_protocol )
{
    int serverSockfd = -1;
    int sendLen = 0;
    int ret = -1;
    char buf[BUFFER_SIZE] = {0};
    char zeroBuf[] = "";
    char recvBuf[MINBUFFER_SIZE] = {0};
    int recvLen = 0;

    DBGPRINT(VDBG,"file:%s, line:%d,deviceID:%s,orderID:%s,device_protocol:%d.\r\n", __FILE__,__LINE__, deviceID, orderID, device_protocol); 

    // 开屏
    unsigned char onBufCmd[] = {0x59, 0x48, 0x01, 0x01, 0x01, 0xff, 0x00, 0x01, 0xff, 0x0d, 0x63};    
    unsigned char onBufAck1[] = {0x01, 0x01, 0x00, 0x01, 0xff, 0x0D, 0x8C};
    unsigned char onBufAck2[] = {0x01, 0x01, 0x00, 0x01, 0x00, 0x0D, 0x73};

    // 创建连接情报板控制器的socket
    serverSockfd = xsk_init_client(gglobalInfo.increasePublishingInfo.controllerIp, gglobalInfo.increasePublishingInfo.controllerPort, 10, NULL);
    if (serverSockfd < 0)
    {
        DBGPRINT(VERR, "forword_and_parse() xsk_init_client error.");
        return ret;
    }

    /*给情报板下发开屏命令*/
    sendLen = send(serverSockfd, onBufCmd, sizeof(onBufCmd), 0);
    if (sendLen != sizeof(onBufCmd))
    {
        DBGPRINT(VERR, "parse_client_to_server() snd() error sendLen[%d].", sendLen);
        
        goto __finish;
    }

__next:
    // 接收情报板回复
    recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);
    if (recvLen <= 0)
    {
        // 没有收到数据, 但是连接是正常的, 继续接收数据
        if (recvLen < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
        {
            goto __next;
        }
        DBGPRINT(VERR, "file:%s, line:%d, error:%d, server->client recv error.", __FILE__, __LINE__, errno);
        xsk_close(&serverSockfd);
        return ret;   
    }
            
    // 记录文件
    hexdump(recvBuf, recvLen, "/root/yuanHeng.log", "server");
    
    /*发送一个空包给情报板*/
    send(serverSockfd, zeroBuf, sizeof(zeroBuf), 0);

    if ((memcmp(recvBuf, onBufAck1, sizeof(onBufAck1)) == 0) ||
        (memcmp(recvBuf, onBufAck2, sizeof(onBufAck2)) == 0))
    {
        buildSendtoClientMsg(buf, deviceID, orderID, device_protocol, 1, "开屏成功");
    }
    else
    {
        buildSendtoClientMsg(buf, deviceID, orderID, device_protocol, 0, "执行失败");
    }

    DBGPRINT(VDBG, "file:%s, line:%d,buf_len:%d,  buf:%s.", __FILE__, __LINE__, strlen(buf), buf);
    // 回复指令处理结果给发布消息客户端
    sendLen = send(clientSockfd, buf, strlen(buf), 0);
    if (sendLen < 0 )
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%d.", __FILE__, __LINE__, sendLen);
        goto __finish;
    }

    ret = 0;

__finish:
    xsk_close(&serverSockfd);
    return ret;   
}

/*构建回复给发送客户端的json报文*/
void buildSendtoClientMsg(char* buf, char* deviceID, char* orderID, int device_protocol, int result, char* cmd)
{
    cJSON *data = NULL;
    char *tmpBuf = NULL;

    data = cJSON_CreateObject();
    cJSON_AddItemToObject(data, "deviceID", cJSON_CreateString(deviceID));
    cJSON_AddItemToObject(data, "orderID", cJSON_CreateString(orderID));
    cJSON_AddItemToObject(data, "device_protocol", cJSON_CreateNumber(device_protocol)); 

    cJSON_AddItemToObject(data, "result", cJSON_CreateNumber(result));
    cJSON_AddItemToObject(data, "messages", cJSON_CreateString(cmd));  

    tmpBuf = cJSON_Print(data);
    strcpy(buf, tmpBuf);
    cJSON_Delete(data);
    free(tmpBuf);
    return;
}

/*情报板最大尺寸*/
#define VMS_MAX_HIGH    512
#define VMS_MAX_WEIGHT  1024

uint32_t z8to32s(uint8_t *pp)
{
    uint32_t i;

    i=0;
    pp=pp+3;
    i=*pp;

    i=i<<8;
    pp--;
    i=i+*pp;

    i=i<<8;
    pp--;
    i=i+*pp;

    i=i<<8;
    pp--;
    i=i+*pp;

    return(i);
}

uint16_t z8to16s(uint8_t *pp)
{
    uint16_t i;

    i=0;
    pp++;
    i=*pp;
    i=i<<8;
    pp--;
    i=i+*pp;
    return(i);
}



uint32_t read_file(char *p_file,uint8_t *p_cache,uint32_t cache_size,uint32_t *p_length)
{
    FILE *fp1; 
    uint32_t i;

    fp1 = fopen(p_file,"rb");
    if (NULL == fp1)
    {
        return -1;
    }
    
    fseek(fp1,0,SEEK_SET);
    i=0;
    while( (feof(fp1)==0)&&(i<cache_size) )
    {
        *(p_cache+i)=fgetc(fp1);
        i++;
    }
    fclose(fp1);
    
    if(i>=cache_size)
    {
        return -1;
    }
    else
    {
        *p_length=i-1;
        return 0;
    }
}


/*
本函数将bmp文件转换为发给情报板的点阵数据
*/
uint32_t bmp_file2table(char *szBmpFile, uint8_t *pp, uint32_t* pLength)
{
    char * bmp_cache;
    uint32_t i=0;
    uint32_t converId = 0;
    uint32_t flag,py,x,y,length,pm,pn;
    uint8_t r,g,b;
    uint8_t c1r,c1g,c1b;
    uint8_t c2r,c2g,c2b;
    
    uint32_t c1rAddr,c1gAddr,c1bAddr;
    uint32_t c2rAddr,c2gAddr,c2bAddr;
    uint32_t ph;
    uint32_t pw;
    int tmpRet = 0;

    bmp_cache = malloc(MAX_BMP_BUFFER);
    if(bmp_cache == NULL)
    {
        DBGPRINT(VDBG, "file:%s, line:%d, malloc error.\r\n", __FILE__, __LINE__);
        return -1;            
    }
    
    tmpRet = read_file(szBmpFile, bmp_cache, MAX_BMP_BUFFER, &length);
    if (tmpRet != 0)
    {
        DBGPRINT(VDBG, "file:%s, line:%d, read_file error.\r\n", __FILE__, __LINE__);
        return -1;
    }
    flag=0;
    if(bmp_cache[0]!='B')
    {
        DBGPRINT(VERR, "file:%s, line:%d, error.\r\n", __FILE__, __LINE__);
        flag=1;
    }
    if(bmp_cache[1]!='M')
    {
    
        DBGPRINT(VERR, "file:%s, line:%d, error.\r\n", __FILE__, __LINE__);
        flag=1;
    }
    if( z8to32s(&bmp_cache[0x2])!=length )
    {
        DBGPRINT(VERR, "file:%s, line:%d, error.\r\n", __FILE__, __LINE__);    
        flag=1; 
    }
    if( z8to16s(&bmp_cache[0x1a])!=1  )
    {
        DBGPRINT(VERR, "file:%s, line:%d, error.\r\n", __FILE__, __LINE__);        
        flag=1;
    }
#if 0    
    if( z8to32s(&bmp_cache[0x1c])!=24 )
    {
        DBGPRINT(VERR, "file:%s, line:%d, error.\r\n", __FILE__, __LINE__);    
        flag=1;
    }
    if( z8to32s(&bmp_cache[0x1e])!=0  )
    {
        DBGPRINT(VERR, "file:%s, line:%d, error.\r\n", __FILE__, __LINE__);    
        flag=1;
    }
#endif

    ph = z8to32s(&bmp_cache[0x16]);
    pw = z8to32s(&bmp_cache[0x12]);
        
    py=z8to32s(&bmp_cache[0x0a]);
    
    DBGPRINT(VDBG, "file:%s, line:%d, ph:%d,pw:%d,py:%d.\r\n", __FILE__, __LINE__, ph,pw,py);    
#if 0    
    if( (py+ph*pw*3)!=length )
    {
        DBGPRINT(VERR, "file:%s, line:%d, error.\r\n", __FILE__, __LINE__);    
        flag=1;
    }
#endif

    if( flag == 1 )
    {
        DBGPRINT(VERR, "file:%s, line:%d, preprocess bmpfile error.\r\n", __FILE__, __LINE__);
        free(bmp_cache);
        return -1;
    }

    /*位图数据在文件中的排列顺序是从左下角到右上角，以行为主序排列的，
      显示时，是从左下角开始，逐行显示，下面代码逻辑实现此对应关系,
      在bmp 24bits中，低8位表示Blue分量；中8位表示Green分量；高8位表示Red分量
    */
    for (int x = 0; x < pw; x++)
    {
        for (int y = 0; y < ph; y += 2)
        {
            c1bAddr = py+((ph-y-1)*pw + x)*3;
            c1gAddr = c1bAddr + 1;
            c1rAddr = c1bAddr + 2; 

            c2bAddr = c1bAddr - pw*3;
            c2gAddr = c2bAddr + 1;
            c2rAddr = c2bAddr + 2;

            c1b=bmp_cache[c1bAddr];
            c1g=bmp_cache[c1gAddr];
            c1r=bmp_cache[c1rAddr];
            c2b=bmp_cache[c2bAddr];
            c2g=bmp_cache[c2gAddr];
            c2r=bmp_cache[c2rAddr];

            pp[converId] = (char)((((c1r & 0x80) >> 3) | ((c1g & 0x80) >> 2)
                | ((c1r & 0x80) >> 1) | (c1b & 0x80)) | ((((c2r & 0x80) >> 3) | ((c2g & 0x80) >> 2)
                | ((c2r & 0x80) >> 1) | (c2b & 0x80)) >> 4));
            converId++;                    
        }
    }
    *pLength = converId;
    free(bmp_cache);    
    return 0 ;
}

/*
本函数构造下载图片到情报板的报文.



返回值:
0成功, 非0失败
*/
int buildDownloadImagePacket(char *szBmpFile, uint32_t infoAddress)
{
    uint32_t bufLength = 0;
    uint8_t page_cache[VMS_MAX_HIGH*VMS_MAX_WEIGHT/2] = {0};
    uint8_t iIndex = 0;
    char tempPacket[527] = {0};
    uint8_t iMaxPacketNum = 0;
    int iTailPacket = 0;
    int serverSockfd = 0;
    int sendLen = 0;
    int recvLen = 0;
    char recvBuf[MINBUFFER_SIZE] = {0};

     if(bmp_file2table(szBmpFile,page_cache, &bufLength) != 0)
    {
        DBGPRINT(VERR, "file:%s, line:%d, bmp_file2table error.\r\n", __FILE__, __LINE__);
        return -1;
    }   

    iMaxPacketNum = bufLength / 512;
    iTailPacket =  bufLength - iMaxPacketNum*512;
    
    DBGPRINT(VDBG, "file:%s, line:%d, bufLength:%d, iMaxPacketNum:%d.\r, iTailPacket:%d.\r\n", __FILE__, __LINE__, bufLength, iMaxPacketNum, iTailPacket);
    
    tempPacket[0]  = 0x59;
    tempPacket[1]  = 0x48;
    tempPacket[2]  = 0x01;
    tempPacket[3]  = 0x01;
    
    tempPacket[4]  = 0x01;
    tempPacket[5]  = 0x82;
    tempPacket[6]  = 0x02;
    tempPacket[7]  = 0x05;
    
    tempPacket[8]  = 0x05;
    //tempPacket[9]  = 0x00;
    tempPacket[10] = (char)(iMaxPacketNum + 1);/*需要发送的报文总个数*/
    tempPacket[11] = 0x00;
    
    tempPacket[12] = (char)(infoAddress);
    DBGPRINT(VDBG, "file:%s, line:%d, tempPacket[10]:%2x, tempPacket[12]:%2x.\r\n", __FILE__, __LINE__, tempPacket[10], tempPacket[12]);

    // 创建连接情报板控制器的socket
    serverSockfd = xsk_init_client(gglobalInfo.increasePublishingInfo.controllerIp, gglobalInfo.increasePublishingInfo.controllerPort, 10, NULL);
    if (serverSockfd < 0)
    {
        DBGPRINT(VERR, "file:%s, line:%d, xsk_init_client error.\r\n", __FILE__, __LINE__);
        return -1;
    }
    
    for (iIndex = 0; iIndex <iMaxPacketNum; iIndex ++)
    {
        tempPacket[9]  = (char)(iIndex);
        memcpy(&tempPacket[13], page_cache + 512 * iIndex, 512);
        calcEccValue(tempPacket, sizeof(tempPacket));
        DBGPRINT(VDBG, "file:%s, line:%d, iIndex:%d, tempPacket[526]:0x%2x.\r\n", __FILE__, __LINE__, iIndex, tempPacket[526]);
        sendLen = send(serverSockfd, tempPacket, sizeof(tempPacket), 0);
        
        if (sendLen != sizeof(tempPacket))
        {
            DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%d, errno:%d.\r\n", __FILE__, __LINE__, sendLen, errno);
            return -1;
        }

        // 接收情报板回复
        recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);
    }

    /*最后的空报文*/
    {
        
        char tempBuf[] = {0x59,0x48,0x01,0x01,0x01,0x82,0x00,0x05,0x05,0x04,0x05,0x00,0x03,0x0d,0xe2};
        tempBuf[9]  = iMaxPacketNum;
        tempBuf[12] = (char)(infoAddress);
        calcEccValue(tempBuf, sizeof(tempBuf));
        DBGPRINT(VERR, "file:%s, line:%d, tempBuf[9]:%2x, tempBuf[12]:%2x, tempBuf[14]:%2x.\r\n", __FILE__, __LINE__, tempBuf[9], tempBuf[12],tempBuf[14]);
        
        sendLen = send(serverSockfd, tempBuf, sizeof(tempBuf), 0);
        if (sendLen != sizeof(tempBuf))
        {
            DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%d, errno:%d.\r\n", __FILE__, __LINE__, sendLen, errno);
        
            return -1;
        } 
        // 接收情报板回复
        recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);        
    }

    /*最后查询播放列表*/
    {
        char tempBuf[] = {0x59,0x48,0x01,0x01,0x01,0x84,0x00,0x02,0x00,0x03,0x0d,0xe7};
        tempBuf[9] = (char)(infoAddress);
        calcEccValue(tempBuf, sizeof(tempBuf));
        DBGPRINT(VERR, "file:%s, line:%d, tempBuf[9]:%2x, tempBuf[11]:%2x.\r\n", __FILE__, __LINE__, tempBuf[9], tempBuf[11]);
        
        sendLen = send(serverSockfd, tempBuf, sizeof(tempBuf), 0);
        if (sendLen != sizeof(tempBuf))
        {
            DBGPRINT(VERR, "file:%s, line:%d, send error. sendLen:%d, errno:%d.\r\n", __FILE__, __LINE__, sendLen, errno);
        
            return -1;
        } 
        // 接收情报板回复
        recvLen = recv(serverSockfd, recvBuf, sizeof(recvBuf), 0);        
    }

    xsk_close(&serverSockfd);
    return 0;
}

char * g_szBase64FileName = "/tmp/cqhimage.base64";
char * g_szPngFileName = "/tmp/cqhimage.png";
char * g_szBmpFileName = "/tmp/cqhimage.bmp";


/*
处理发送图片到情报板指令。
发送客户端给前置图片信息， 是png文件经过base64编码形成的字符串.
前置的处理步骤：
1: 将字符串存盘变成文件.
2: 调用base64命令, 对文件进行base64解码, 还原成png格式文件。
3: 调用convert命令, 将png格式文件转换为bmp格式文件.
4: 将bmp文件转换为点阵数据。
5: 将点阵数据切割成情报板需要的数据包，发送到情报板.
*/
int processDownloadImageOrder(int clientSockfd, char *deviceID, char *orderID, int device_protocol, cJSON *json)
{
    int ret = -1;
    int tmpRet = -1;
    int tmpLen = 0;
    char* imageContent;
    int infoAddress;
    FILE *fp;
    char cmdBuf[MINBUFFER_SIZE] = {0};
    int exit_status = 0;
    char szBuf[600] = {0};

    infoAddress = cJSON_GetObjectItem(json, "infoAddress")->valueint;
    imageContent = cJSON_GetObjectItem(json, "imageContent")->valuestring;

    tmpLen = strlen(imageContent);
    
    DBGPRINT(VDBG, "file:%s, line:%d, infoAddress:%d, tmpLen:%d.\r\n", __FILE__, __LINE__, infoAddress, tmpLen);

    fp = fopen(g_szBase64FileName, "wb+");
    if (NULL == fp)
    {
        DBGPRINT(VERR, "file:%s, line:%d, fopen error.\r\n", __FILE__, __LINE__);
        return -1;
    }
    
    tmpRet = fwrite(imageContent, 1, tmpLen, fp);
    if (tmpRet != tmpLen)
    {
        DBGPRINT(VERR, "file:%s, line:%d, fwrite error, tmpRet:%d.\r\n", __FILE__, __LINE__, tmpRet);
        goto __finish;
    }
    fclose(fp);

    snprintf(cmdBuf, MINBUFFER_SIZE, "/usr/bin/base64 -d %s > %s; /usr/bin/convert %s %s ", g_szBase64FileName, g_szPngFileName, g_szPngFileName, g_szBmpFileName);
    DBGPRINT(VDBG, "file:%s, line:%d, ...cmdBuf:%s.\r\n", __FILE__, __LINE__, cmdBuf);
    
    system(cmdBuf);
    
    /*补充审批逻辑*/
    if (0 != review_test(g_szBmpFileName))
    {
        buildSendtoClientMsg(szBuf, deviceID, orderID, device_protocol, -1, "发布内容有歧义, 暂缓发布");
    }
    else
    {
        tmpRet = buildDownloadImagePacket(g_szBmpFileName, infoAddress);
        if (tmpRet == 0)
        {
            buildSendtoClientMsg(szBuf, deviceID, orderID, device_protocol, 1, "发布成功");
        }
        else
        {
            buildSendtoClientMsg(szBuf, deviceID, orderID, device_protocol, -2, "发布失败");
            
        }
    }
    DBGPRINT(VDBG, "file:%s, line:%d,buf_len:%d,  buf:%s.", __FILE__, __LINE__, strlen(szBuf), szBuf);    
    // 回复指令处理结果给发布消息客户端
    tmpLen = send(clientSockfd, szBuf, strlen(szBuf), 0);
    if (tmpLen < 0 )
    {
        DBGPRINT(VERR, "file:%s, line:%d, send error. tmpLen:%d.", __FILE__, __LINE__, tmpLen);
        return -1;
    }
    ret = 0;

__finish:
    return ret;   
}

int parseClientOrder(int clientSockfd)
{
    int ret = -1;
    int tmpRet = -1;
    int flag = 0;
    int recvLen = 0;
    int sendLen = 0;
    cJSON *json = NULL;
    char *serialized_json = NULL;
    char *deviceID = NULL;
    char *orderID  = NULL;
    int device_protocol;
    int infoCount;
    int totalBytesReceived = 0;
    
    DBGPRINT(VDBG, "client_to_server() clientSockfd[%d]", clientSockfd);

#if 0
char* buf;
int bufLen = HUGEBUFFER_SIZE;
buf = malloc(bufLen);

// 接收数据包
__next:
    recvLen = rcv(clientSockfd, buf, bufLen, flag);
    DBGPRINT(VDBG, "file:%s, line:%d, recvLen: %d, errno:%d.", __FILE__,__LINE__,recvLen, errno);
    if (recvLen <= 0)
    {
        // 没有收到数据, 但是连接是正常的, 继续接收数据
        if (recvLen < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
        {
            goto __next;
        }
        DBGPRINT(VERR, "file:%s, line:%d, recvLen:%d, errno:%d, client->server recv error.", __FILE__, __LINE__, recvLen, errno);
        goto __finish;
    }
#else
    char buf[HUGEBUFFER_SIZE] = {0};

    totalBytesReceived = 0;
    do
    {
        DBGPRINT(VDBG, "file:%s, line:%d.", __FILE__, __LINE__);
        recvLen = recv(clientSockfd, buf + totalBytesReceived, sizeof(buf) - totalBytesReceived - 1, flag);
        DBGPRINT(VDBG, "file:%s, line:%d, recvLen: %d, errno:%d.", __FILE__,__LINE__,recvLen, errno);

        if (recvLen > 0)
        {
            totalBytesReceived += recvLen;
            if (strchr(buf, '}') != NULL)
            {
                DBGPRINT(VDBG, "file:%s, line:%d.", __FILE__, __LINE__);
                break;
            }
        }
        else if (recvLen < 0 && (errno == EINTR || errno == EWOULDBLOCK || errno == EAGAIN))
        {
            continue;
        }
        else
        {
            break;
        }
    } while (totalBytesReceived < sizeof(buf) - 1 && recvLen >= 0);


#endif
    DBGPRINT(VDBG, "file:%s, line:%d, client->server recv OK totalBytesReceived: %d.", __FILE__,__LINE__,totalBytesReceived);
    
    // 记录文件
    hexdump(buf, recvLen, "/root/yuanHeng.log", "client"); 
    DBGPRINT(VERR, "file:%s, line:%d.", __FILE__,__LINE__);
    
    // 解析 JSON 数据并输出
    json = cJSON_Parse(buf);
    DBGPRINT(VERR, "file:%s, line:%d.", __FILE__,__LINE__);
    if (NULL == json)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_Parse error.", __FILE__,__LINE__);
        // free(buf);
        return ret;
    }
    serialized_json = cJSON_Print(json);
    if (NULL == serialized_json)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_Print error.", __FILE__,__LINE__);
        cJSON_Delete(json);
        // free(buf);        
        return ret;
    }    
    
    DBGPRINT(VDBG, "file:%s, line:%d, serialized_json:%s.", __FILE__,__LINE__,serialized_json);
    deviceID = cJSON_GetObjectItem(json, "deviceID")->valuestring;
    if (NULL == deviceID)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_GetObjectItem error, no deviceID.", __FILE__,__LINE__);
        goto __finish;
    }
    orderID = cJSON_GetObjectItem(json, "orderID")->valuestring;
    if (NULL == orderID)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_GetObjectItem error, no orderID.", __FILE__,__LINE__);
        goto __finish;
    }
    
    device_protocol = cJSON_GetObjectItem(json, "device_protocol")->valueint;

    /*开屏指令*/
    if ( (device_protocol == 28) && (strcmp(orderID, "01") == 0))
    {
        processOnscreenOrder(clientSockfd, deviceID, orderID, device_protocol);
    }

    /*关屏指令*/
    if ( (device_protocol == 28) && (strcmp(orderID, "02") == 0))
    {
        processOffscreenOrder(clientSockfd, deviceID, orderID, device_protocol);
    }

    /*播放指令*/
    if ((device_protocol == 0) && (strcmp(orderID, "04") == 0))
    {
        processPlayOrder(clientSockfd, deviceID, orderID, device_protocol, json);
    }

    /*图片下载指令*/
    if ((device_protocol == 0) && (strcmp(orderID, "03") == 0))
    {
        processDownloadImageOrder(clientSockfd, deviceID, orderID, device_protocol, json);
    }

    /*模拟审批*/
    if ((device_protocol == 0) && (strcmp(orderID, "999") == 0))
    {
        DBGPRINT(VERR, "file:%s, line:%d, orderID:%s.", __FILE__,__LINE__, orderID);
        review_test("/opt/review.bmp");
    }

    
    ret = 0;
    
__finish:
    // 释放 cJSON 对象
    cJSON_Delete(json);
    free(serialized_json);
    // free(buf);    
    return ret;
}



/*
发布平台<---->前置<---->情报板
业务模型里将发布平台作为client, 
情报板作为server.
des:
    开启转发和解析工作
pram：
    param[IN]:提取的客户端连接    
*/
void forword_and_parse(void *param)
{
    int tmpRet = -1;
    int clientSockfd = -1;
    int gaptime = 0;
    int maxfd = 0;
    fd_set rset;
    struct timeval sm_tv;
        
    if (param == NULL)
    {
        DBGPRINT(VERR, "forword_and_parse() param == NULL.");
        goto __finish;
    }
    memcpy(&clientSockfd, param, sizeof(int));
    
    // 记入ip连接审计文件
    write_ip_audit_file(gglobalInfo.contectInfo.clientIp, gglobalInfo.contectInfo.serverIp, reportFileInfo.auditFileInfo.fp);
    
    while (1)
    {
        sm_tv.tv_sec = gaptime;
        sm_tv.tv_usec = 0;
        FD_ZERO(&rset);
        FD_SET(clientSockfd, &rset);
        maxfd = clientSockfd + 1;
        tmpRet = select(maxfd, &rset, NULL, NULL, &sm_tv);
        if (tmpRet < 0)
        {
            DBGPRINT(VERR, "file:%s, line:%d, forword_and_parse() select error.", __FILE__, __LINE__);
            break;
        }
        else if (tmpRet == 0)
        {
            continue;
        }
        else
        {
            if (FD_ISSET(clientSockfd, &rset))
            {
                // 处理发布客户端过来的指令
                tmpRet = parseClientOrder(clientSockfd);
                if (tmpRet < 0)
                {
                    DBGPRINT(VDBG, "forword_and_parse() client_to_server error.");
                    goto __finish;
                }
            }
       }
    }

__finish:
    xfree(param);
    xsk_close(&clientSockfd);
    DBGPRINT(VDBG, "file:%s, line:%d, forword_and_parse() pthread_exit.", __FILE__, __LINE__);
    pthread_exit(NULL);
}


