#include "review.h"
#include "common.h"
#include "packet.h"
#include "afx/xmd5.h"
#include "afx/xiconv.h"

extern ST_globalInfo gglobalInfo;

/*
des:
    获取系统特征码
return:
    成功:0
    失败:小于0
*/
static int get_system_serial_numb()
{
    int ret = -1;
    int tmpRet = -1;
    char uuidPath[BUFFER_SIZE] = "";
    char sysUuid[1024] = "";

    // 组uuid文件路径
    sprintf(uuidPath, "%s/../../config/uuid", gglobalInfo.octoShare.headPath);

    if ((access(uuidPath, F_OK)) == 0)
    {
        FILE *fp = NULL;
        fp = fopen(uuidPath, "rb");
        if (fp == NULL)
        {
            goto __finish;
        }
        else
        {
            fgets(sysUuid, sizeof(sysUuid), fp);
            fclose(fp);
        }
    }
    else
    {
        goto __finish;
    }
    GetMD5OfData(sysUuid, strlen(sysUuid), sizeof(sysUuid), gglobalInfo.pcSerialnumb, 0);
    ret = 0;
__finish:
    return ret;
}

/*
des:
    组审批json
*/
char *create_review_json(char *reviewBuf, char *reviewType)
{
    int ret = -1;
    int tmpRet = -1;
    cJSON *jsonroot = NULL;
    cJSON *content = NULL;
    cJSON *contentArray = NULL;
    int reviewBufLen = 0;
    char *uReviewBuf = NULL;
    char *jsonout = NULL;
    char *baseText = NULL;
    time_t unix_time = 0;

    // 获取当前时间
    unix_time = time(NULL);

    // 获取系统特征码
    tmpRet = get_system_serial_numb();
    if (tmpRet < 0)
    {
        goto __finish;
    }

    // 创建根节点对象
    jsonroot = cJSON_CreateObject();
    // 创建内容节点对象
    content = cJSON_CreateObject();
    // 创建content数组
    contentArray = cJSON_CreateArray();

    // 添加数据
    cJSON_AddStringToObject(jsonroot, "serialNumb", gglobalInfo.pcSerialnumb);
    cJSON_AddNumberToObject(jsonroot, "dataType", DATE_TYPE_REVIEW);

    // 文本内容转码
    reviewBufLen = strlen(reviewBuf);
    uReviewBuf = xmalloc(reviewBufLen * 3);
    if (uReviewBuf == NULL)
    {
        DBGPRINT(VERR, "create_review_json() xmalloc() uAuditBuf uReviewBuf error.");
        goto __finish;
    }
    tmpRet = xg2u(reviewBuf, reviewBufLen, uReviewBuf, reviewBufLen * 3);
    if (tmpRet < 0)
    {
        // 文本内容转base64
        int tmpLen = 0;
        tmpLen = strlen(reviewBuf);
        baseText = xmalloc(tmpLen * 3);
        EncodeBase64(reviewBuf, tmpLen, baseText);
    }
    else
    {
        // 文本内容转base64
        int tmpLen = 0;
        tmpLen = strlen(uReviewBuf);
        baseText = xmalloc(tmpLen * 3);
        EncodeBase64(uReviewBuf, tmpLen, baseText);
    }

    cJSON_AddStringToObject(content, "modeName", MODE_NAME);

    cJSON_AddStringToObject(content, "type", reviewType);

    cJSON_AddStringToObject(content, "content", baseText);

    cJSON_AddNumberToObject(content, "unixTime", unix_time);

    cJSON_AddItemToObject(contentArray, "content", content);

    cJSON_AddItemToObject(jsonroot, "content", contentArray);

    jsonout = cJSON_Print(jsonroot);
    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }

    xfree(baseText);
    xfree(uReviewBuf);
    return jsonout;
}

/*
des:
    组审批json
*/
char *create_review_bmp_json(char *bmpName, char *bmpContent, char *bmpMd5, char *reviewType)
{
    int ret = -1;
    int tmpRet = -1;
    cJSON *jsonroot = NULL;
    cJSON *content = NULL;
    cJSON *contentArray = NULL;
    char *jsonout = NULL;
    time_t unix_time = 0;

    // 获取当前时间
    unix_time = time(NULL);

    // 获取系统特征码
    tmpRet = get_system_serial_numb();
    if (tmpRet < 0)
    {
        goto __finish;
    }

    // 创建根节点对象
    jsonroot = cJSON_CreateObject();
    // 创建内容节点对象
    content = cJSON_CreateObject();
    // 创建content数组
    contentArray = cJSON_CreateArray();

    // 添加数据
    cJSON_AddStringToObject(jsonroot, "serialNumb", gglobalInfo.pcSerialnumb);
    cJSON_AddNumberToObject(jsonroot, "dataType", DATE_TYPE_REVIEW);

    cJSON_AddStringToObject(content, "modeName", MODE_NAME);

    cJSON_AddStringToObject(content, "type", reviewType);

    cJSON_AddStringToObject(content, "bmpName", bmpName);

    cJSON_AddStringToObject(content, "bmpMd5", bmpMd5);

    cJSON_AddStringToObject(content, "content", bmpContent);

    cJSON_AddNumberToObject(content, "unixTime", unix_time);

    cJSON_AddItemToObject(contentArray, "content", content);

    cJSON_AddItemToObject(jsonroot, "content", contentArray);

    jsonout = cJSON_Print(jsonroot);
    ret = 0;
__finish:

    if (jsonroot)
    {
        cJSON_Delete(jsonroot);
    }

    return jsonout;
}

/*
des:
    解析审批内容，同意加入审批白名单，拒绝则不加入
param：
    recv[IN]:接收的审批内容
*/
void parse_review_content(char *recv)
{
    int ret = -1;
    int tmpRet = -1;
    cJSON *root = NULL;
    cJSON *pass = NULL;

    DBGPRINT(VDBG, "parse_review_content() recv[%s].", recv);

    // 解析
    root = cJSON_Parse(recv);
    if (root == NULL)
    {
        DBGPRINT(VERR, "parse_review_content() cJSON_Parse error.");
        goto __finish;
    }

#if 1
    char* jsonbuf = cJSON_Print(root);
    DBGPRINT(VDBG, "file:%s, line:%d. jsonbuf:%s.", __FILE__,__LINE__, jsonbuf);
    xfree(jsonbuf);   
#endif
    
    // 解析contentArrya
    pass = cJSON_GetObjectItem(root, "pass");
    if (pass == NULL)
    {
        DBGPRINT(VERR, "parse_review_content() pass == NULL.");
        goto __finish;
    }
    if (pass->valueint == 1)
    {
        DBGPRINT(VDBG, "file:%s, line:%d. reviewResult:allowed.", __FILE__,__LINE__);

        xlist *tmpList = NULL;
        tmpList = gglobalInfo.tmpReviewWirteList;

        while (tmpList && tmpList->next)
        {
            DBGPRINT(VDBG, "parse_review_content() key[%s].", tmpList->key);
            tmpList = tmpList->next;
        }
        if (gglobalInfo.tmpReviewWirteList == NULL)
        {
            DBGPRINT(VDBG, "file:%s, line:%d. tmpReviewWirteList=NULL", __FILE__,__LINE__);
        }

        if (gglobalInfo.bmpTmpReviewWirteList == NULL)
        {
            DBGPRINT(VDBG, "file:%s, line:%d. bmpTmpReviewWirteList=NULL", __FILE__,__LINE__);
        }

        
        // 添加到审批白名单
        gglobalInfo.reviewWriteList = xlist_dup(gglobalInfo.tmpReviewWirteList);
        if (gglobalInfo.reviewWriteList == NULL)
        {
            DBGPRINT(VERR, "parse_review_content() xlist_dup() reviewWriteList error.");
        }
        gglobalInfo.bmpReviewWriteList = xlist_dup(gglobalInfo.bmpTmpReviewWirteList);
        if (gglobalInfo.bmpReviewWriteList == NULL)
        {
            DBGPRINT(VERR, "parse_review_content() xlist_dup() bmpReviewWriteList error.");
        }
        DBGPRINT(VDBG, "parse_review_content() add WriteList to reviewWriteList.");
    }
    else if (pass->valueint == 0)
    {
        DBGPRINT(VERR, "parse_review_content() pass is reject.");
        goto __finish;
    }
    else if (pass->valueint == 2)
    {
        DBGPRINT(VERR, "parse_review_content() pass is timeout.");
        goto __finish;
    }

__finish:

    if (root)
    {
        cJSON_Delete(root);
    }

    return;
}

/*
des:
    接收并解析审批消息
pram：
    param[IN]:与平台的连接
*/
void *review_recv_and_parse(void *param)
{
    int ret = -1;
    int tmpRet = -1;
    int tmpLen = 0;
    char *recv = NULL;
    int recvLen = 0;
    int serverSockfd = -1;
    int gaptime = 10;
    int timeOut = 180;

    if (param == NULL)
    {
        DBGPRINT(VERR, "review_recv_and_parse() param == NULL.");
        goto __finish;
    }
    memcpy(&serverSockfd, param, sizeof(int));

    DBGPRINT(VDBG, "review_recv_and_parse() serverSockfd[%d].", serverSockfd);

    // 接收
    // 超时判断
    int canRead = xsk_can_read(serverSockfd, timeOut, 0);
    if (!canRead)
    {
        DBGPRINT(VERR, "review_request() xsk_can_read[%d] timeout[3min] error.", canRead);
        goto __finish;
    }

    // 接收数据
    tmpLen = psms_recv(serverSockfd, &recv, &recvLen);
    if (tmpLen <= 0)
    {
        DBGPRINT(VERR, "review_request() psms_recv error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "review_request() recv policy len[%d][%s].", tmpLen, recv);

    // 上锁
    pthread_mutex_lock(&(gglobalInfo.threadMutex));

    // 解析审批消息
    parse_review_content(recv);

    // 解锁
    pthread_mutex_unlock(&(gglobalInfo.threadMutex));

__finish:

    DBGPRINT(VDBG, "review_recv_and_parse() serverSockfd[%d] ended.", serverSockfd);

    xsk_close(&serverSockfd);
    xfree(recv);
    xfree(param);
    pthread_exit(NULL);

    return NULL;
}

/*
des:
    发送审批请求，解析审批内容，添加审批白名单列表
param:
    reviewBuf[IN]:待发送的审批信息
    reviewType[IN]:审批类型
return：
    -1：失败
    0：成功
*/
int review_request(char *reviewBuf, char *reviewType)
{
    int ret = -1;
    int tmpRet = -1;
    int jsonBufLen = 0;
    char *jsonBuf = NULL;
    int socketFd = -1;
    int sendLen = 0;
    pthread_t tid;
    char *tmpParam = NULL;
    char tmpIp[MINBUFFER_SIZE] = "";

    // 获取模拟IP
    if (strcmp(gglobalInfo.confInfo.terminalInfo.ip, "") != 0)
    {

        strcpy(tmpIp, "");
    }
    else
    {
        strcpy(tmpIp, gglobalInfo.confInfo.spmsInfo.controllerIp);
    }

    // 去除，号
    if (reviewBuf[strlen(reviewBuf) - 1] == ',')
    {
        reviewBuf[strlen(reviewBuf) - 1] = '\0';
    }

    // 组json
    jsonBuf = create_review_json(reviewBuf, reviewType);
    if (jsonBuf == NULL)
    {
        DBGPRINT(VERR, "review_request() create_review_json() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "review_request() create_review_json() jsonBuf[%s].", jsonBuf);

    // 连接管理平台
    // socketFd = xsk_init_client(gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, 3, tmpIp);
    socketFd = xsk_init_client_by_interface(gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, 3, tmpIp, "br0");
    if (socketFd < 0)
    {
        DBGPRINT(VERR, "review_request() xsk_init_client_by_interface connect psms ip[%s]port[%d] fakeIp[%s] error.", gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, tmpIp);
        goto __finish;
    }

    sendLen = psms_tcp_send(socketFd, jsonBuf, strlen(jsonBuf));
    if (sendLen < 0)
    {
        DBGPRINT(VERR, "review_request() psms_tcp_send sendLen[%d].", sendLen);
        goto __finish;
    }

    // 审批接收线程
    // 线程传参，在堆区，线程内释放
    tmpParam = xmalloc(sizeof(int));
    memcpy(tmpParam, &socketFd, sizeof(int));
    // 开启线程
    tmpRet = pthread_create(&tid, NULL, (void *)review_recv_and_parse, (void *)tmpParam);
    if (tmpRet != 0)
    {
        DBGPRINT(VERR, "review_request() error ret[%d]", ret);
        xsk_close(&socketFd);
        DBGPRINT(VDBG, "review_request() server contnet already close.");
        xfree(tmpParam);
        goto __finish;
    }
    // 设置线程分离
    pthread_detach(tid);

    ret = 0;
__finish:

    xfree(jsonBuf);

    return ret;
}

/*
des:
    发送审批请求，解析审批内容，添加审批白名单列表
param:
    pprotocolHead[IN]:数据包包头
    reviewBuf[IN]:待发送的审批信息
return：
    -1：失败
    0：成功
*/
int review_bmp_request(char *bmpName, char *content, char *bmpMd5, char *reviewType)
{
    int ret = -1;
    int tmpRet = -1;
    int jsonBufLen = 0;
    char *jsonBuf = NULL;
    int socketFd = -1;
    int sendLen = 0;
    pthread_t tid;
    char *tmpParam = NULL;
    char tmpIp[MINBUFFER_SIZE] = "";

    // 获取模拟IP
    if (strcmp(gglobalInfo.confInfo.terminalInfo.ip, "") != 0)
    {

        strcpy(tmpIp, "");
    }
    else
    {
        strcpy(tmpIp, gglobalInfo.confInfo.spmsInfo.controllerIp);
    }

    // 组json
    jsonBuf = create_review_bmp_json(bmpName, content, bmpMd5, reviewType);
    if (jsonBuf == NULL)
    {
        DBGPRINT(VERR, "review_bmp_request() create_review_json() error.");
        goto __finish;
    }
    DBGPRINT(VDBG, "review_bmp_request() create_review_json() jsonBuf[%s].", jsonBuf);

    // 连接管理平台
    // socketFd = xsk_init_client(gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, 3, tmpIp);
    socketFd = xsk_init_client_by_interface(gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, 3, tmpIp, "br0");
    if (socketFd < 0)
    {
        DBGPRINT(VERR, "review_bmp_request() xsk_init_client_by_interface connect psms ip[%s]port[%d] fakeIp[%s] error.", gglobalInfo.confInfo.spmsInfo.ip, gglobalInfo.confInfo.spmsInfo.tcpPort, tmpIp);
        goto __finish;
    }

    sendLen = psms_tcp_send(socketFd, jsonBuf, strlen(jsonBuf));
    if (sendLen < 0)
    {
        DBGPRINT(VERR, "review_bmp_request() psms_tcp_send sendLen[%d].", sendLen);
        goto __finish;
    }

    // 审批接收线程
    // 线程传参，在堆区，线程内释放
    tmpParam = xmalloc(sizeof(int));
    memcpy(tmpParam, &socketFd, sizeof(int));
    // 开启线程
    tmpRet = pthread_create(&tid, NULL, (void *)review_recv_and_parse, (void *)tmpParam);
    if (tmpRet != 0)
    {
        DBGPRINT(VERR, "review_bmp_request() error ret[%d]", ret);
        xsk_close(&socketFd);
        DBGPRINT(VDBG, "review_bmp_request() server contnet already close.");
        xfree(tmpParam);
        goto __finish;
    }
    // 设置线程分离
    pthread_detach(tid);

    ret = 0;
__finish:

    xfree(jsonBuf);

    return ret;
}

