#ifndef __H_SETTING_H__
#define __H_SETTING_H__
#include "leg_share.h"
#include "common.h"
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include "protocol_parse.h"

#pragma pack(push, 1)

/*------------------------------------增加---------------------------------------*/
// 新增平台发布、心跳
typedef struct _st_increasePublishingInfo
{
   // 发布平台ip
   char publishingPlatformIp[16];
   // 发布平台端口
   int publishingPlatformPort;
   // 发布平台心跳频率
   int publishingPlatformHeartbeat;
   // 控制平台ip
   char controllerIp[16];
   // 发布平台端口
   int controllerPort;
   // 发布平台心跳频率
   int controllerHeartbeat; 
} ST_increasePublishingInfo, *PST_increasePublishingInfo;

// 管理平台配置
typedef struct _st_spmsInfo
{
   // 平台IP
   char ip[BUFFER_SIZE];
   // 控制器IP
   char controllerIp[BUFFER_SIZE];
   // 平台tcp端口
   int tcpPort;
   // 平台udp端口
   int udpPort;
} ST_spmsInfo, *PST_spmsInfo;

// 终端配置
typedef struct _st_terminalInfo
{
   // 终端IP
   char ip[BUFFER_SIZE];
} ST_terminalInfo, *PST_terminalInfo;

typedef struct _st_contectInfo
{
   // 客户端ip
   char clientIp[64];
   // 客户端port
   int clientPort;
   // 服务端ip
   char serverIp[64];
   // 服务端port
   int serverPort;
} ST_contectInfo, *PST_contectInfo;
// 数据包地址信息
typedef struct _st_packetAddrInfo
{
   // 目的地址域
   unsigned char detAddr;
   // 源地址域
   unsigned char srcAddr;
} ST_packetAddrInfo, *PST_packetAddrInfo;
// 配置文件信息
typedef struct _st_confInfo
{
   // 管理平台配置
   ST_spmsInfo spmsInfo;
   // 终端设备配置
   ST_terminalInfo terminalInfo;
   // 加密类型配置
   int encodeType;
   // 审计汇报频率
   int auditTime;
   // 告警汇报频率
   int warningTime;
   // 比较频率
   int comPareTime;
   // 服务器ip
   char serverIp[16];
   // 服务器端口
   int serverPort;
   // 自己开放端口
   int selfPort;
} ST_confInfo, *PST_confInfo;

// 鱼爪所有信息
typedef struct _st_globalInfo
{
   // 设备序列号
   char pcSerialnumb[MINBUFFER_SIZE];
   // 与鱼头的通信套接字
   int commSocketfd;
   // 线程锁
   pthread_mutex_t threadMutex;
   // 屏幕内容记录文件锁
   pthread_mutex_t screenFileMutex;
   // 配置文件信息
   ST_confInfo confInfo;
   // 审计汇报频率
   int auditTime;
   // 告警汇报频率
   int warningTime;
   // 比较频率
   int comPareTime;
   // 连接标志
   int contentFlag;
   // 准入所有ip标志0：有规则；1：准入所有
   int enterAllowIpflag;
   // 审计发布内容信息
   char auditBuf[BIGBUFFER_SIZE];
   // 白名单规则链表
   xlist *writeList;
   // 临时审批白名单规则链表
   xlist *tmpReviewWirteList;
   // 审批白名单规则链表
   xlist *reviewWriteList;

   // 图片白名单规则链表
   xlist *bmpWriteList;
   // 图片临时审批白名单规则链表
   xlist *bmpTmpReviewWirteList;
   // 图片审批白名单规则链表
   xlist *bmpReviewWriteList;

   // 白名单IP链表
   xlist *allowIpList;
   // 鱼头鱼爪的共享信息
   ST_octoShare octoShare;
   // 客户端、服务器IP端口信息
   ST_contectInfo contectInfo;
   // 数据包地址信息
   ST_packetAddrInfo packetAddrInfo;
   /*------------------------------------增加---------------------------------------*/
   //新增平台发布、心跳
   ST_increasePublishingInfo increasePublishingInfo;  
} ST_globalInfo, *PST_globalInfo;
#pragma pack(pop)

/*
des:
   鱼爪启动信息初始化
param:
   pocto_share[IN]:鱼头鱼爪共享信息
return:
   成功：等于0
   失败：小于0
*/
int init_leg_info(PST_octoShare poctoShare);
/*
des:
   释放鱼爪垃圾数据
param:
   pocto_share[IN]:鱼头鱼爪共享信息
*/
extern void free_leg_info(PST_octoShare poctoShare);

#endif
