#include "work.h"
#include "common.h"
#include "setting.h"
#include "policy.h"
#include "operate_file.h"
#include "afx/xiconv.h"
#include "protocol_parse.h"
#include <dirent.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

extern ST_globalInfo gglobalInfo;
extern ST_reportFileInfo reportFileInfo;

/*--------------------------------------增加----------------------------------------*/
// 控制器心跳状态
_Bool controllerHeartbeatFlag = TRUE;

/*
des:
    构建发送给发布平台的json数据包
return；
    成功：等于0
    失败：小于0
*/
int build_publishingPlatform_packet(char *buf)
{
    int tempRet = -1;

    // 构建 JSON 对象
    cJSON *json_obj = cJSON_CreateObject();
    cJSON_AddStringToObject(json_obj, "deviceID", "522050001");
    cJSON_AddStringToObject(json_obj, "orderID", "07");
    cJSON_AddNumberToObject(json_obj, "device_protocol", 28);
    if (controllerHeartbeatFlag)
    {
        cJSON_AddNumberToObject(json_obj, "vmbStatus", 1);
    }
    else
    {
        cJSON_AddNumberToObject(json_obj, "vmbStatus", 0);
    }

    // 将 JSON 对象转换为文本格式
    char *json_text = cJSON_Print(json_obj);
    if (json_text == NULL)
    {
        DBGPRINT(VERR, "build_publishingPlatform_packet() cJSON_PrintUnformatted()  error.");
        goto _finish;
    }
    // 复制 JSON 文本到 buf 数组中
    strcpy(buf, json_text);
    tempRet = 0;
_finish:
    // 释放资源
    xfree(json_obj);
    free(json_text);
    return tempRet;
}

/*
des:
    将连接信息记入审计文件，检查连接ip，若ip不在白名单内，再记入告警文件
return；
    成功：等于0
    失败：小于0
*/
int check_ip_and_write_file()
{
    int ret = -1;
    int tmpRet = -1;
    xlist *tmpList = NULL;

    tmpList = xlist_get(gglobalInfo.allowIpList, gglobalInfo.contectInfo.clientIp);
    if (tmpList == NULL)
    {
        // 记入告警文件
        tmpRet = write_ip_warnning_file(gglobalInfo.contectInfo.clientIp, gglobalInfo.contectInfo.serverIp, reportFileInfo.warnningFileInfo.fp);
        if (tmpRet < 0)
        {
            DBGPRINT(VERR, "check_ip_and_write_file() write_ip_warnning_file() warnning error.");
            goto __finish;
        }

        goto __finish;
    }

    ret = 0;
__finish:

    return ret;
}

static void xgetnameinfo(const struct sockaddr *sa, socklen_t salen,
                         char *host, size_t hostlen,
                         char *serv, size_t servlen, int flags)
{
    int err = getnameinfo(sa, salen, host, hostlen, serv, servlen, flags);
    if (err)
    {
        return;
    }
}

static void ipaddrtostr(const struct sockaddr *sa, socklen_t salen, char *resbuf, size_t reslen, char *port, size_t plen)
{
    xgetnameinfo(sa, salen, resbuf, reslen, port, plen, NI_NUMERICHOST | NI_NUMERICSERV);
}

/*
des:
    从链接中获取源ip和目的ip
parm：
    sockfd[IN]:提取得客户端连接
*/
void get_src_dst_ip(int sockfd)
{
    struct sockaddr_storage ss1;
    int ret;
    char buf1[INET6_ADDRSTRLEN];

    socklen_t sa1len;

    sa1len = sizeof(ss1);

    char port[8];
    ret = getpeername(sockfd, (struct sockaddr *)&ss1, &sa1len);
    if (ret == -1)
    {
        DBGPRINT(VERR, "get_src_dst_ip() getpeername()");
        return;
    }
    ipaddrtostr((const struct sockaddr *)&ss1, sa1len, buf1, sizeof(buf1), port, sizeof(port));
    DBGPRINT(VDBG, "get_src_dst_ip() clientIp[%s] clientPort[%s]", buf1, port);
    strcpy(gglobalInfo.contectInfo.clientIp, buf1);
    gglobalInfo.contectInfo.clientPort = atoi(port);
    {
        char port[8];

        sa1len = sizeof(ss1);
        ret = getsockname(sockfd, (struct sockaddr *)&ss1, &sa1len);
        if (ret)
        {
            DBGPRINT(VERR, "get_src_dst_ip() getsockname()");
            return;
        }
        ipaddrtostr((const struct sockaddr *)&ss1, sa1len, buf1, sizeof(buf1), port, sizeof(port));
        strcpy(gglobalInfo.contectInfo.serverIp, buf1);
        gglobalInfo.contectInfo.serverPort = atoi(port);
        DBGPRINT(VDBG, "get_src_dst_ip() serverIp[%s] serverPort[%s]", buf1, port);
    }
}

/*
des:
    工作主方法
*/
void work_func()
{
    int gaptime = 0;
    int serverSocket = -1;

    // 前置监听33012端口
    serverSocket = xsk_init_listen_socket(gglobalInfo.confInfo.selfPort, 32, NULL, NULL);
    if (serverSocket < 0)
    {
        DBGPRINT(VERR, "work_func() xsk_init_listen_socket error.");
        goto __finish;
    }

    while (1)
    {
        int tmpRet = -1;
        int maxfd = 0;
        fd_set rset;
        struct timeval sm_tv;

        sm_tv.tv_sec = gaptime;
        sm_tv.tv_usec = 0;
        FD_ZERO(&rset);
        FD_SET(serverSocket, &rset);
        maxfd = serverSocket + 1;

        tmpRet = select(maxfd, &rset, NULL, NULL, &sm_tv);
        if (tmpRet < 0)
        {
            DBGPRINT(VERR, "work_func() select error.");
            break;
        }
        else if (tmpRet == 0)
        {
            continue;
        }
        else
        {
            if (FD_ISSET(serverSocket, &rset))
            {
                pthread_t tid;
                int sockfd = -1;
                char clientIp[16] = "";
                char serverIp[16] = "";
                char *tmpParam = NULL;

                sockfd = xsk_accept(serverSocket, NULL);
                if ((sockfd) < 0)
                {
                    DBGPRINT(VERR, "work_func() xsk_accept error.");
                    continue;
                }
                // 通过连接查看来源ip、目的ip
                get_src_dst_ip(sockfd);

                if (gglobalInfo.enterAllowIpflag == ALLOW_SPECIAL_IP)
                {
                    // 检查连接IP
                    tmpRet = check_ip_and_write_file();
                    if (tmpRet < 0)
                    {
                        DBGPRINT(VERR, "work_func() check_ip_and_write_file() ip is not allow.");
                        xsk_close(&sockfd);
                        DBGPRINT(VDBG, "work_func() client contnet already close.");
                        continue;
                    }
                }

                // 线程传参，在堆区，线程内释放
                tmpParam = xmalloc(sizeof(int));
                memcpy(tmpParam, &sockfd, sizeof(int));
                // 开启线程
                tmpRet = pthread_create(&tid, NULL, (void *)forword_and_parse, (void *)tmpParam);
                if (tmpRet != 0)
                {
                    xsk_close(&sockfd);
                    DBGPRINT(VDBG, "work_func() client contnet already close.");
                    xfree(tmpParam);
                    break;
                }

                // 设置线程分离
                pthread_detach(tid);
            }
        }
    }
__finish:

    xsk_close(&serverSocket);
}


/*------------------------------------增加---------------------------------------*/
/*
des:
    定时和发布平台发布心跳
*/

int g_toControllerSockfd = -1;

void yh_free_sockets(void)
{

    if (g_toControllerSockfd > 0)

    {
        close(g_toControllerSockfd);
        DBGPRINT(VDBG, "file:%s, line:%d, close(g_toControllerSockfd).", __FILE__,__LINE__);
    }
}

void *send_heartbeat_to_publishingPlatform()
{
    int tmpRet = -1;
    int sendLen = 0;
    char buf[MINBUFFER_SIZE] = {0};
    int toPublishingPlatformSockfd = -1;
   
    while (1)
    {
        // 创建tcp客户端
        toPublishingPlatformSockfd = xsk_init_client(gglobalInfo.increasePublishingInfo.publishingPlatformIp, gglobalInfo.increasePublishingInfo.publishingPlatformPort, 10, NULL);
        if (toPublishingPlatformSockfd < 0)
        {
            DBGPRINT(VERR, "file:%s, line:%d, xsk_init_client error.", __FILE__,__LINE__);
            goto __oneprobe;
        }
        DBGPRINT(VDBG, "file:%s, line:%d, toPublishingPlatformSockfd:%d.", __FILE__, __LINE__, toPublishingPlatformSockfd);
           
        // 根据和控制器探测结果, 构建发送给发布平台的json数据包
        tmpRet = build_publishingPlatform_packet(buf);
        if (tmpRet != 0)
        {
            DBGPRINT(VERR, "file:%s, line:%d, build_publishingPlatform_packet() error.", __FILE__,__LINE__);
            goto __oneprobe;
        }

        // 发送数据包
        sendLen = snd(toPublishingPlatformSockfd, buf, strlen(buf), 0);
        if (!sendLen)
        {
            DBGPRINT(VERR, "file:%s, line:%d, snd() error.", __FILE__,__LINE__);
            goto __oneprobe;
        }

        DBGPRINT(VDBG, "file:%s, line:%d, send heartbeat to PublishingPlatform successfully.", __FILE__,__LINE__);
        
__oneprobe:
        if (toPublishingPlatformSockfd > 0)
        {
            close(toPublishingPlatformSockfd);
        }
        
        // 心跳频率
        sleep(gglobalInfo.increasePublishingInfo.publishingPlatformHeartbeat);        
    }
    
    return NULL;
}


// 发送给情报板控制器的握手报文，用于心跳探测
static unsigned char g_toControllerSyncData[] = {0x59, 0x48, 0x01, 0x01, 0x01, 0x01, 0x00, 0x01, 0x00, 0x0D, 0x62};

// 情报板控制器对握手报文的回复报文
static unsigned char g_fromControllerAckData[] = {0x01, 0x01, 0x00, 0x01, 0x00, 0x0D, 0x73};

// 情报板控制器在收到错误播放指令时，对握手报文回复不一样，下电重启可恢复。
// 还有什么恢复方法? 最后一个字节应该没有收全。
static unsigned char g_fromControllerAckDataErr[] = {0x01, 0x01, 0x00, 0x02, 0xce, 0xf7, 0x0d};

void *send_heartbeat_to_controller()
{
    int rcvLen = 0;
    int timeout = 1;
    unsigned char rcvBuf[MINBUFFER_SIZE] = {0};
    int sendLen = 0;

    // 创建tcp客户端
    g_toControllerSockfd = xsk_init_client(gglobalInfo.increasePublishingInfo.controllerIp, gglobalInfo.increasePublishingInfo.controllerPort, 10, NULL);
    if (g_toControllerSockfd < 0)
    {
        DBGPRINT(VERR, "file:%s, line:%d, xsk_init_client error.", __FILE__,__LINE__);
        return NULL;
    }
    DBGPRINT(VDBG, "file:%s, line:%d, g_toControllerSockfd:%d.", __FILE__, __LINE__, g_toControllerSockfd);
    
    while (1)
    {

        // 开始一次心跳探测
        controllerHeartbeatFlag = FALSE;
        
        //  发送握手报文
        sendLen = send(g_toControllerSockfd, g_toControllerSyncData, sizeof(g_toControllerSyncData), 0);
        if (!sendLen)
        {
            DBGPRINT(VERR, "file:%s, line:%d, snd() error.controllerHeartbeat:%d.", __FILE__,__LINE__,gglobalInfo.increasePublishingInfo.controllerHeartbeat);
            goto __oneprobe;
        }
        
        // 接收控制器回复的消息
        rcvLen = recv(g_toControllerSockfd, rcvBuf, sizeof(rcvBuf), 0);
        if (!rcvLen || memcmp(rcvBuf, g_fromControllerAckData, sizeof(g_fromControllerAckData)))
        {
            controllerHeartbeatFlag = FALSE;
            DBGPRINT(VERR, "file:%s, line:%d, rcv() error rcvLen[%d].", __FILE__,__LINE__,rcvLen);            
            goto __oneprobe;
        }
        else
        {
            controllerHeartbeatFlag = TRUE;
            DBGPRINT(VDBG, "file:%s, line:%d, heartbeat from controller: %d.", __FILE__,__LINE__, controllerHeartbeatFlag);
        }
        
__oneprobe:
        
        //  心跳频率
        sleep(gglobalInfo.increasePublishingInfo.controllerHeartbeat);
    }

    return NULL;
}

/*
des:
    定时发送告警文件
*/
void *control_send_warnning_file()
{
    int ret = -1;
    int tmpRet = -1;
    time_t tmpTvStart;
    time_t tmpTvEnd;
    char warnningFileName[BUFFER_SIZE] = "";

    tmpTvStart = time(NULL);
    tmpTvEnd = time(NULL);

    gglobalInfo.warningTime = gglobalInfo.confInfo.warningTime;
    reportFileInfo.warnningFileInfo.newRand = rand();
    sprintf(warnningFileName, "%s/../../data/sendCache/%s/%d.warnning", gglobalInfo.octoShare.headPath, MODE_NAME, reportFileInfo.warnningFileInfo.newRand);

    // 打开文件
    reportFileInfo.warnningFileInfo.fp = open(warnningFileName, O_CREAT | O_TRUNC | O_RDWR, 0666);
    // 到时发送文件
    while (1)
    {
        if (gglobalInfo.warningTime == 0)
        {
            DBGPRINT(VDBG, "control_send_warnning_file()  do not have to send warnningFile.");
            sleep(3);
        }
        else if (gglobalInfo.warningTime > 0)
        {
            int sec = (int)(tmpTvEnd - tmpTvStart);
            if (sec >= gglobalInfo.warningTime || sec == 0 || (tmpTvEnd < tmpTvStart))
            {
                send_warnning_already_file();
                tmpTvStart = tmpTvEnd;
            }
            sleep(1);
            tmpTvEnd = time(NULL);
        }
        else
        {
            DBGPRINT(VDBG, "control_send_audit_file() warningTime[%d] error.", gglobalInfo.warningTime);
            sleep(3);
        }
    }

__finish:
    return NULL;
}

/*
des:
    定时发送审计文件
*/
void *control_send_audit_file()
{
    int ret = -1;
    time_t tmpTvStart;
    time_t tmpTvEnd;
    char auditFileName[BUFFER_SIZE] = "";

    tmpTvStart = time(NULL);
    tmpTvEnd = time(NULL);

    gglobalInfo.auditTime = gglobalInfo.confInfo.auditTime;
    reportFileInfo.auditFileInfo.newRand = rand();
    sprintf(auditFileName, "%s/../../data/sendCache/%s/%d.audit", gglobalInfo.octoShare.headPath, MODE_NAME, reportFileInfo.auditFileInfo.newRand);

    // 打开文件
    reportFileInfo.auditFileInfo.fp = open(auditFileName, O_CREAT | O_TRUNC | O_RDWR, 0666);
    // 到时发送文件
    while (1)
    {
        if (gglobalInfo.auditTime == 0)
        {
            DBGPRINT(VDBG, "control_send_audit_file()  do not have to send auditFile.");
            sleep(3);
        }
        else if (gglobalInfo.auditTime > 0)
        {
            int sec = (int)(tmpTvEnd - tmpTvStart);
            if (sec >= gglobalInfo.auditTime || sec == 0 || (tmpTvEnd < tmpTvStart))
            {
                send_audit_already_file();
                tmpTvStart = tmpTvEnd;
            }
            sleep(1);
            tmpTvEnd = time(NULL);
        }
        else
        {
            DBGPRINT(VDBG, "control_send_audit_file() auditTime[%d] error.", gglobalInfo.auditTime);
            sleep(3);
        }
    }

__finish:
    return NULL;
}

/*
des:
    巡检模式的线程工作区
*/
void *thread_workstation()
{
    int ret = -1;
    int tmpRet = -1;
    pthread_t tid1, tid2, tid3, tid4;

    // 判断sendCache是否存在
    tmpRet = judge_send_cache();
    if (tmpRet < 0)
    {
        DBGPRINT(VERR, "start_service() open_send_cache() error.");
        goto __finish;
    }

    while (1)
    {
        if (gglobalInfo.contentFlag == 1)
        {
            // 查找遗留文件
            find_last_file();
            break;
        }
        sleep(1);
    }

    // 控制审计文件发送线程
    tmpRet = pthread_create(&tid1, NULL, control_send_audit_file, NULL);
    if (tmpRet != 0)
    {
        DBGPRINT(VERR, "thread_workstation() pthread_create() control_send_audit_file error.");
        goto __finish;
    }

    // 控制告警文件发送线程
    tmpRet = pthread_create(&tid2, NULL, control_send_warnning_file, NULL);
    if (tmpRet != 0)
    {
        DBGPRINT(VERR, "thread_workstation() pthread_create() control_send_warnning_file error.");
        goto __finish;
    }

#if 1
    /*------------------------------------增加---------------------------------------*/
    // 和发布平台的心跳线程
    tmpRet = pthread_create(&tid3, NULL, send_heartbeat_to_publishingPlatform, NULL);
    if (tmpRet != 0)
    {
        DBGPRINT(VERR, "thread_workstation() pthread_create() send_heartbeat_to_publishingPlatform error.");
        goto __finish;
    }

    // 和控制器的心跳线程
    tmpRet = pthread_create(&tid4, NULL, send_heartbeat_to_controller, NULL);
    if (tmpRet != 0)
    {
        DBGPRINT(VERR, "thread_workstation() pthread_create() send_heartbeat_to_controller error.");
        goto __finish;
    }
#endif

    work_func();

    ret = 0;
    
__finish:
    return NULL;
}

/*
des:
    巡检模式的工作线程，此处需要注意线程结束的处理
param:
    workTid[IN,OUT]:传入线程id指针，返回线程id号
return:
    成功：0
    失败：小于0
*/
int work_thread(pthread_t *workTid)
{
    int ret = -1;
    int tmpRet = -1;
    pthread_t tmpPid = -1;
    tmpRet = pthread_create(&tmpPid, NULL, (void *)thread_workstation, NULL);
    if (tmpRet != 0)
    {
        DBGPRINT(VERR, "work_thread() error tmpRet[%d]", tmpRet);
        goto __finish;
    }
    *workTid = tmpPid;
    ret = 0;
__finish:
    return ret;
}

