#include <stdio.h>
#include <string.h>
#include "afx/m_afx.h"
#include "afx/xcjson.h"
#include "afx/xrelloc.h"
#include "afx/xbase64.h"
#include "afx/xmd5.h"
#include "common.h"
#include "protocol_parse.h"
#include "setting.h"
#include <pthread.h>
#include <unistd.h>
#include <sys/syscall.h>
#include "yuanheng.h"

char cacheFileNameOld[BUFFER_SIZE] = "";
char cacheFileNameNew[BUFFER_SIZE] = "";

/*
des:
    过滤数据包，将不符合解析规则的数据包剔除
parm：
    buf[IN]:数据内容s
    bufSize[IN]:数据长度
return:
    0:符合解析规则
    小于0:不符合解析规则
*/
int grep_content_packet(unsigned char *buf, int bufSize)
{
    int ret = -1;
    ST_protocolHead tmpProtocolHead;
    PST_protocolHead ptmpProtocolHead = &tmpProtocolHead;

    memset(ptmpProtocolHead, 0, sizeof(ST_protocolHead));

    // 判定数据长度是否合规
    if (bufSize <= PACKET_HEAD_LEN + sizeof(ST_protocolEnd))
    {
        DBGPRINT(VERR, "grep_content_packet() len is error bufSize[15]<%d.", bufSize, PACKET_HEAD_LEN + sizeof(ST_protocolEnd));
        goto __finish;
    }
    // 判定数据包开始结束标记是否合规
    if (buf[0] != START_FLAG || buf[bufSize - 2] != END_FLAG)
    {
        DBGPRINT(VERR, "grep_content_packet() start end flag is error.");
        goto __finish;
    }
    // 判定数据包类型是否为10，上传文件
    memcpy(ptmpProtocolHead, buf, sizeof(ST_protocolHead));
    if (ptmpProtocolHead->orderCode[0] != 0x01 || ptmpProtocolHead->orderCode[1] != 0x82)
    {
        DBGPRINT(VERR, "grep_content_packet() order is error.");
        goto __finish;
    }

    ret = 0;
__finish:
    return ret;
}

/*
des:
    从数据包中获取列表名
parm：
    buf[IN]:数据内容
    bufSize[IN,OUT]:
return:
    0:需要创建
    小于0：不需要创建
*/
void grep_need_to_create_file(unsigned char *buf, int bufSize, int *createFileFlag)
{
    int ret = -1;
    int tmpRet = -1;
    char tmpBuf = 0;

    // 拷贝第十个字节
    memcpy(&tmpBuf, buf + 9, 1);

    // 判断是否创建文件
    if (tmpBuf == 0x00)
    {
        *createFileFlag = 1;
        DBGPRINT(VDBG, "grep_need_to_create_file() need to create file.");
    }
    else
    {
        *createFileFlag = 0;
        DBGPRINT(VDBG, "grep_need_to_create_file() not need to create file");
    }

    ret = 0;
__finish:
    return;
}

/*
des:
    解析图片内容
param：
    buf[IN]:数据包内容
    bufSize[IN]:数据包大小
    fileNameLen[IN]:该数据包文件名
    jsonRoot[IN,OUT]:json根
return:
    0:接收完毕
    小于:需接续接收
*/
int parse_bmp(unsigned char *buf, int bufSize, int createFileFlag)
{
    int ret = -1;
    int tmpRet = -1;
    int bmpContentIndex = 0;
    char realBmpContent[BUFFER_SIZE] = "";
    int realbmpContentLen = 0;
    int cacheFileSize = 0;
    int bigLittle = 0;
    unsigned short tmpPgNumber = 0;
    ST_protocolHead tmpProtocolHead;
    PST_protocolHead ptmpProtocolHead = &tmpProtocolHead;

    // 拷贝包头
    memset(ptmpProtocolHead, 0, sizeof(ST_protocolHead));
    memcpy(ptmpProtocolHead, buf, sizeof(ST_protocolHead));

    // 计算数据长度
    realbmpContentLen = bufSize - sizeof(ST_protocolHead) - sizeof(ST_protocolEnd);

    // 拷贝数据
    memcpy(realBmpContent, buf + sizeof(ST_protocolHead), realbmpContentLen);

    // 开始组包头
    bigLittle = check_big_little();
    // 如果为小端，长度需要转化为网络字节序传输
    if (bigLittle == 1)
    {
        tmpPgNumber = htons(ptmpProtocolHead->pgNumber);
    }
    else
    {
        tmpPgNumber = ptmpProtocolHead->pgNumber;
    }
    DBGPRINT(VDBG, "parse_bmp()  pgNumber[%d].", tmpPgNumber);

    //---------------------------写一个文件
    // 写入文件
    //  组缓存文件名，写入文件
    if (strcmp(cacheFileNameOld, "") == 0 && strcmp(cacheFileNameNew, "") == 0)
    {
        sprintf(cacheFileNameOld, "/opt/%d.bmp", tmpPgNumber);
        sprintf(cacheFileNameNew, "/opt/%d.bmp", tmpPgNumber);
    }
    else
    {
        sprintf(cacheFileNameNew, "/opt/%d.bmp", tmpPgNumber);
    }

    if (strcmp(cacheFileNameOld, cacheFileNameNew) != 0)
    {
        // 删除文件
        remove(cacheFileNameOld);
        strcpy(cacheFileNameOld, cacheFileNameNew);
    }

    if (createFileFlag == 0)
    {
        file_write(realBmpContent, realbmpContentLen, cacheFileNameNew, "ab+");
        DBGPRINT(VDBG, "realbmpContentLen[%d]", realbmpContentLen);
    }
    else if (createFileFlag == 1)
    {
        file_write(realBmpContent, realbmpContentLen, cacheFileNameNew, "wb+");
        DBGPRINT(VDBG, "realbmpContentLen[%d]", realbmpContentLen);
    }
    // 判断数据长度
    if (realbmpContentLen == PACKET_DATA_LEN)
    {
        goto __finish;
    }
    //---------------------------写一个文件

    //---------------------------写多个文件
    // 判断数据长度
    // if (realbmpContentLen == PACKET_DATA_LEN)
    // {
    //     char tmpName[BUFFER_SIZE] = "";
    //     sprintf(tmpName, "/root/bmp/%d.bmp", i);
    //     file_write(realBmpContent, realbmpContentLen, tmpName, "wb+");
    //     i++;

    //     goto __finish;
    // }
    //---------------------------写多个文件

    ret = 0;
__finish:

    return ret;
}

/*
des:
    解析数据包
parm：
    buf[IN]:数据包内容
    bufSize[IN]:数据包长度
return:
    0:阻断转发，需处理业务
    小于0:数据包通过
*/
int parse_client_packet(char *buf, int bufSize)
{
    int ret = -1;
    cJSON *json = NULL;
    char *serialized_json = NULL;
    
    // 开屏
    unsigned char onBuf[] = {0x59, 0x48, 0x01, 0x01, 0x01, 0xff, 0x00, 0x01, 0xff, 0x0d, 0x63};
    // 关屏
    unsigned char offBuf[] = {0x59, 0x48, 0x01, 0x01, 0x01, 0xff, 0x00, 0x01, 0x00, 0x0d, 0x9C};

    // 解析 JSON 数据并输出
    json = cJSON_Parse(buf);
    if (NULL == json)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_Parse error.", __FILE__,__LINE__);
        return ret;
    }
    serialized_json = cJSON_Print(json);
    if (NULL == serialized_json)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_Print error.", __FILE__,__LINE__);
        cJSON_Delete(json);
        return ret;
    }    

    DBGPRINT(VDBG, "file:%s, line:%d, serialized_json:%s.", __FILE__,__LINE__,serialized_json);
    
    const char *deviceID = cJSON_GetObjectItem(json, "deviceID")->valuestring;
    if (NULL == deviceID)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_GetObjectItem error, no deviceID.", __FILE__,__LINE__);
        goto __finish;
    }
    const char *orderID = cJSON_GetObjectItem(json, "orderID")->valuestring;
    if (NULL == orderID)
    {
        DBGPRINT(VERR, "file:%s, line:%d, cJSON_GetObjectItem error, no orderID.", __FILE__,__LINE__);
        goto __finish;
    }
    
    int device_protocol = cJSON_GetObjectItem(json, "device_protocol")->valueint;
     
    if (strcmp(orderID, "01") == 0)  // 01 : 开屏
    {
        memcpy(buf, onBuf, sizeof(onBuf));
        for (int i = 0; i < 11; i++)
        {
            DBGPRINT(VDBG, "parse_client_packet() onBuf  0x%02x ", onBuf[i]);
        }
    }
    else if (strcmp(orderID, "02") == 0) // 02 : 关屏
    {
        memcpy(buf, offBuf, sizeof(offBuf));
        for (int i = 0; i < 11; i++)
        {
            DBGPRINT(VDBG, "parse_client_packet() offBuf  0x%02x ", offBuf[i]);
        }
    }
    // 03 发送下载指令
    else if (strcmp(deviceID, "522050001") == 0 && strcmp(orderID, "03") == 0 && device_protocol == 28)
    {
        // 下载
        int infoAddress = cJSON_GetObjectItem(json, "infoAddress")->valueint;
        const char *infoContent = cJSON_GetObjectItem(json, "infoContent")->valuestring;
        const char *imageContent = cJSON_GetObjectItem(json, "imageContent")->valuestring;
        const char *fontType = cJSON_GetObjectItem(json, "fontType")->valuestring;
        const char *fontColor = cJSON_GetObjectItem(json, "fontColor")->valuestring;
    }
    // 04 播放指令
    else if (strcmp(deviceID, "522050001") == 0 && strcmp(orderID, "04") == 0 && device_protocol == 28)
    {
        // 播放
        int width = cJSON_GetObjectItem(json, "width")->valueint;
        int height = cJSON_GetObjectItem(json, "height")->valueint;
        int infoCount = cJSON_GetObjectItem(json, "infoCount")->valueint;
        int infoAddress = cJSON_GetObjectItem(json, "infoAddress")->valueint;
        const char *imageContent = cJSON_GetObjectItem(json, "imageContent")->valuestring;
        int showTime = cJSON_GetObjectItem(json, "showTime")->valueint;
        int lightGrade = cJSON_GetObjectItem(json, "lightGrade")->valueint;
        int playMode = cJSON_GetObjectItem(json, "playMode")->valueint;
        const char *fontType = cJSON_GetObjectItem(json, "fontType")->valuestring;
        const char *fontColor = cJSON_GetObjectItem(json, "fontColor")->valuestring;
        const char *backColor = cJSON_GetObjectItem(json, "backColor")->valuestring;
        double scaleX = cJSON_GetObjectItem(json, "scaleX")->valuedouble;
        double scaleY = cJSON_GetObjectItem(json, "scaleY")->valuedouble;
        int siteX = cJSON_GetObjectItem(json, "siteX")->valueint;
        int siteY = cJSON_GetObjectItem(json, "siteY")->valueint;
        int lineSpace = cJSON_GetObjectItem(json, "lineSpace")->valueint;
        int txtSpace = cJSON_GetObjectItem(json, "txtSpace")->valueint;
        const char *alignMent = cJSON_GetObjectItem(json, "alignMent")->valuestring;
    }
    else
    {
        DBGPRINT(VDBG, "file:%s, line:%d, unknown orderID:%s.", __FILE__, __LINE__, orderID);
    }

    ret = 0;
__finish:

    // 释放 cJSON 对象
    cJSON_Delete(json);
    free(serialized_json);
    return ret;
}

/*
des:
    解析数据包
*/
int parse_server_packet(char *buf, int bufSize)
{
    int ret = -1;

    cJSON *data = cJSON_CreateObject();
    cJSON_AddItemToObject(data, "deviceID", cJSON_CreateString("322030103"));
    cJSON_AddItemToObject(data, "orderID", cJSON_CreateString("02"));
    cJSON_AddItemToObject(data, "device_protocol", cJSON_CreateNumber(28));

    /*情报板回复开指令：0x01, 0x01, 0x00, 0x01, 0xff, 0x0D, 0x8C*/
    if (memcmp(buf, (unsigned char[]){0x01, 0x01, 0x00, 0x01, 0xff, 0x0D, 0x8C}, 7) == 0)
    {
        cJSON_AddItemToObject(data, "result", cJSON_CreateNumber(1));
        cJSON_AddItemToObject(data, "messages", cJSON_CreateString("开屏成功"));
    }
    /*情报板回复关指令：0x01, 0x01, 0x00, 0x01, 0x00, 0x0D, 0x73*/
    else if (memcmp(buf, (unsigned char[]){0x01, 0x01, 0x00, 0x01, 0x00, 0x0D, 0x73}, 7) == 0)
    {
        cJSON_AddItemToObject(data, "result", cJSON_CreateNumber(1));
        cJSON_AddItemToObject(data, "messages", cJSON_CreateString("关屏成功"));
    }
    else
    {
        cJSON_AddItemToObject(data, "result", cJSON_CreateNumber(0));
        cJSON_AddItemToObject(data, "messages", cJSON_CreateString("执行失败"));
    }

    for (int i = 0; i < 7; i++)
    {
        DBGPRINT(VDBG, "parse_server_packet() buf  0x%02x ", buf[i]);
    }
    char *tmpBuf = cJSON_Print(data);
    strcpy(buf, tmpBuf);
    DBGPRINT(VDBG, "parse_server_packet()  buf[%s] .", buf);
    ret = 0;
__finish:

    // 释放 cJSON 对象
    cJSON_Delete(data);
    return ret;
}
